%
% Compute the HHS-SURE criterion.
%
function sOut = comp_hhs_sure( varargin )

global XOrig GX % X and its Gram matrix

% Initialize
y = [];
thetaHat = [];
noiseSigma = -1;
phi1 = -1;
phi2 = -1;
bComputeApprox = 0;

% Process input arguments

lenVarargin = length( varargin );
numArgsIn = floor( lenVarargin/2 );

if 2*numArgsIn ~= lenVarargin, error( 'even ip args expected' ), end

for argInInd = 1:numArgsIn    
    szKey = lower( varargin{2*argInInd-1} );    
    switch szKey        
        case 'y', y = varargin{2*argInInd};                  
        case 'thetaHat', thetaHat = varargin{2*argInInd };        
        case 'noiseSigma', noiseSigma = varargin{2*argInInd};
        case 'phi1', phi1 = varargin{2*argInInd};
        case 'phi2', phi2 = varargin{2*argInInd};
        case 'bComputeApprox', bComputeApprox = varargin{2*argInInd};
        otherwise
            error( sprintf( 'unrecognized option %s', szKey ) );
    end
end

if ( abs( phi1 - phi2 ) < 1e3*eps ) | ( ( phi1 - phi2 ) < eps )
    error( 'phi2 < phi1 not satisfied' );
end

y = y(:); % Make column vectors
theta = theta(:);
N = length( y );
M = length( thetaHat );

e = y - XOrig * thetaHat;
rss = sum( e.^2 );

hhsSureVal = N*noiseSigma^2 + rss; % Initialize

% Compute approximation
thetaHatThres = soft_thres( thetaHat, phi1 - phi2 );
nnzThetaHatThres = sum( abs( thetaHatThres ) > 1e3*eps );
hhsSureValApprox = hhsSureVal + 2*noiseSigma^2*nnzThetaHatThres;

% If we only want the approximate value, return immediately
if bComputeApprox
    sOut = struct( 'hhsSureVal', -1, 'hhsSureValApprox', hhsSureValApprox );
    return;
end

% Otherwise, spend more cpu cycles to compute the exact approximation. 

% Create U matrix
uVec = rect( thetaHat / ( 2 * (phi1-phi2) ) );
U = spdiag( uVec );

% Find the zero-valued elements of thetaHat
thetaHatNzInd = find( abs(thetaHat) < 1e2*eps );
r = length( thetaHatNzInd );

if ( r == 0 ) % No reduction
    disp( 'ALERT! r = 0' );
    K = GX;
    J = -0.5 * U;
else
    % The first thing to do is build the permutation matrices P and Q. They are of
    % size M x M.
    P = speye( M, M );
    Q = speye( M, M );
    
    disp( sprintf( 'r = %g', r ) );
    
    for i = 1:r
        % Swap the i-th row with the thetaHatNzInd_i-th row in P, and do
        % the same for the columns of Q
        P( [i,thetaHatNzInd(i)], : ) = P( [thetaHatNzInd(i),i], : );
        Q( :, [i,thetaHatNzInd(i)] ) = Q( :, [thetaHatNzInd(i),i] );
    end
    
end
    

