% Function to plot a time series with errorbars but with a log scale on the
% y axis. Implemented in a very naive manner.
% Args: x, y, e, linespec, n_fig, sz_tag
% Outputs: none

function errorbar_semilogy( varargin )

% Defaults
%/////////

x = [];
y = [];
e = [];
szLinespec = '';
nFig = -1;
nLineWidth = 3;
szTag = '';

% Parse arguments
%////////////////

nArgIn = floor( length( varargin ) / 2 );
if 2*nArgIn ~= length( varargin ), error( 'expected even # args' ), end

for argInd = 1:nArgIn
    
    szKey = varargin{ 2*argInd - 1 };
    val = varargin{ 2*argInd };
    
    switch szKey
        case 'x', x = val;
        case 'y', y = val;
        case 'e', e = val;
        case 'linespec', szLinespec = val;
        case 'n_fig', nFig = val;
        case 'sz_tag', szTag = val;
        otherwise, disp( sprintf( 'unrecognized key %s', szKey ) )
    end
    
end

if isempty( szTag ), error( 'szTag is empty' ), end

% Find the values for which y is < 0 and refuse to plot those
illegalInd = find( y < 1e-9 );

if ~isempty( illegalInd )
    warning( 'Not plotting illegal y values' ) 
    disp( 'illegalInd:' ), disp( illegalInd(:)' )
    disp( 'y(illegalInd):' ), disp( reshape( y( illegalInd ), 1, length( illegalInd ) ) )
end

szErrorbarLinespec = [ szLinespec(1), szLinespec(3) ];

xDelta = 0.1 * max( abs( diff( x ) ) );

figure( nFig )

goodInd = setdiff( 1:length(x), illegalInd );

plot( x( goodInd ), log10( y( goodInd ) ), szLinespec, 'LineWidth', nLineWidth, 'MarkerSize', 10, ...
    'Tag', szTag );

bSetHold = 0;

if ~ishold
    hold on
    bSetHold = 1;
end

warning off MATLAB:log:logOfZero
yUpperBound = log10( y + e );
yLowerBound = log10( y - e );
warning on MATLAB:log:logOfZero

for xInd = goodInd
     
    plot( x( xInd ) * ones( 1, 2 ), [ yLowerBound( xInd ) yUpperBound( xInd ) ], ...
        szErrorbarLinespec, 'LineWidth', nLineWidth );
    
    xErrorHorizLine = x( xInd ) * ones( 1, 2 ) + xDelta * [ -1 1 ];
    
    plot( xErrorHorizLine, yLowerBound( xInd ) * ones( 1, 2 ), ...
        szErrorbarLinespec, 'LineWidth', nLineWidth );
    plot( xErrorHorizLine, yUpperBound( xInd ) * ones( 1, 2 ), ...
        szErrorbarLinespec, 'LineWidth', nLineWidth );
    
end

if bSetHold
    hold off
end

