% 31-aug-07 Function that goes through the various directories and
% determines how many test cases exist for each file
% Args: dir_names, file_names
% Output: sFilecount
function sFilecount = get_file_count( varargin )

% Defaults
%/////////

caszDirnames = cell( 0 );
caszFilenames = cell( 0 );

% Process input arguments
%////////////////////////

lenVarargin = length( varargin );
numArgsIn = floor( lenVarargin/2 );

if 2*numArgsIn ~= lenVarargin, error( 'even ip args expected' ), end

for argInInd = 1:numArgsIn
    
    szKey = lower( varargin{ 2*argInInd-1 } );    
    ip = varargin{ 2*argInInd };
    
    switch szKey
        case 'dir_names', caszDirnames = ip;
        case 'file_names', caszFilenames = ip;
        otherwise, error( sprintf( 'unrecognized key: %s', szKey ) );
    end
    
end

if ( isempty( caszDirnames ) | isempty( caszFilenames ) )
    error( 'uninitialized variables' );
end

matFilecount = zeros( length( caszFilenames ), length( caszDirnames ) );

bError = false;

for fileInd = 1:length( caszFilenames )
    for dirInd = 1:length( caszDirnames )
        [nFile, nErr] = get_nfiles( caszFilenames{ fileInd }, caszDirnames{ dirInd }, ...
            fileInd == 1 );
        if ( ( nFile < 0 ) & ~bError )
            bError = true;
        end
        matFilecount( fileInd, dirInd ) = nFile;
    end
end

sFilecount = struct( 'filecount', matFilecount, 'bError', bError );
        
% Returns the number of files of a specified name in the directory. Returns
% a -1 if the directory doesn't exist.
%//////////////////////////////////////////////////////////////////////////
function [nFile, nErr] = get_nfiles( szFile, szDir, bVerbose )

% Constants
%///

nIndStart = 1;

bSuppressFurtherMsgs = true;

if ~exist( szDir, 'dir' )
    nFile = -1;
    return;
end

nFile = 0;

nErr = 0;

ind = nIndStart;
lastValidInd = -1;

while true
    
    szTestDir = sprintf( '%s/t%g', szDir, ind );
    
    if exist( szTestDir, 'dir' )
        
        szTestDirFilename = sprintf( '%s/%s', szTestDir, szFile );
        
        if exist( szTestDirFilename, 'file' )
            nFile = nFile + 1;
        else
            if ( ( nErr > 0 ) & bSuppressFurtherMsgs )
                if ( nErr == 1 ), warning( 'Suppressing further messages' ), end                
            else
                fprintf( 1, sprintf( '\tWarning: %s exists but does not have %s\n', ...
                    szTestDir,szFile ) );
            end
            nErr = nErr + 1;
        end
        
    else
        lastValidInd = ind - 1;
        break;
    end
    
    ind = ind + 1;
    
end

if bVerbose
    if ( lastValidInd < nIndStart )
        warning( sprintf( 'Directory %s has no txx subdirectories', szDir ) );
    elseif( lastValidInd >= nIndStart )
        disp( sprintf( 'Directory %s has t1 ... t%g', szDir, lastValidInd ) );
    end
end
