
% Jan 28, 2007. Functional version of binary_s1. The code is sufficiently
% mature that we shouldn't have to keep on relying on a script. This piece
% collects the results but does not do any plotting.

function [sPerfResOut, sHypResOut] = perf_collect( varargin )

% Defaults
%/////////

sszResfile = {};
sszReconFieldname = {};
snrDirVec = [];
nRealizationsMat = [];
bDebug = 0;

% Parse arguments
%////////////////

nArgIn = floor( length( varargin ) / 2 );
if 2*nArgIn ~= length( varargin ), error( 'expected even # args' ), end

for argInd = 1:nArgIn
    szKey = varargin{ 2*argInd - 1 };
    val = varargin{ 2*argInd };
    switch szKey
        case 'ssz_resfile', sszResfile = val;
        case 'ssz_recon_fieldname', sszReconFieldname = val;
        case 'snr_dir', snrDirVec = val;
        case 'snr_dir_nreal', nRealizationsMat = val;
        case 'err_ind_vec', errIndVec = val;
        case 'b_debug', bDebug = val;
        otherwise, error( sprintf( 'unrecognized key: %s', szKey ) )
    end
end

% Check arguments

if length( sszResfile ) ~= length( sszReconFieldname )
    error( 'lengths of sszResfile and sszReconFieldname should be equal' )
end

if ( size( nRealizationsMat, 1 ) ~= length( sszResfile ) ) | ...
   ( size( nRealizationsMat, 2 ) ~= length( snrDirVec ) )
    error( 'unexpected size for nRealizationsMat' )
end
    
% We plot the performance of one or multiple methods. If only plotting for
% one method, we create plots of performance and hyperparameter estimates.
% If plotting for 2 or more methods, only create performance plots.
%//////////////////////////////////////////////////////////////////////////

N_METHODS = length( sszResfile );

switch N_METHODS

    case 0, error( 'nothing to do' )

    case 1

        % Only 1 method

        sPerfRes = irecon_collect( 'resfile', sszResfile{1}, ...
            'snrdir', snrDirVec, ...
            'snrreal', nRealizationsMat, ...
            'spriv', struct( 'szReconFieldname', sszReconFieldname{1} ), ...
            'fcncull', 'irecon_perf_cull' );

        sHypRes = irecon_collect( 'resfile', sszResfile{1}, ...
            'snrdir', snrDirVec, ...
            'snrreal', nRealizationsMat, ...
            'spriv', struct( [] ), ...
            'fcncull', 'irecon_hyp_cull' );

    otherwise

        % 2 or more methods are being compared

        for methodInd = 1:length( sszResfile )

            if bDebug, disp( sprintf( 'methodInd = %g', methodInd ) ), end
            
            sPerfRes( methodInd ) = irecon_collect( 'resfile', sszResfile{methodInd}, ...
                'snrdir', snrDirVec, ...
                'snrreal', nRealizationsMat( methodInd, : ), ...
                'spriv', struct( 'szReconFieldname', sszReconFieldname{methodInd} ), ...
                'fcncull', 'irecon_perf_cull' );
            
            sHypRes( methodInd ) = irecon_collect( 'resfile', sszResfile{methodInd}, ...
                'snrdir', snrDirVec, ...
                'snrreal', nRealizationsMat( methodInd, : ), ...
                'spriv', struct( [] ), ...
                'fcncull', 'irecon_hyp_cull' ); 
            
            if bDebug, disp( 'Pausing...' ), pause, end

        end

end

% Assign to the outputs
%//////////////////////

% This function will return an array of structures. Each array element will
% be a 2-d table with the rows indexing different SNR operating conditions
% and the columns indexing the different error metrics. See
% irecon_perf_cull where this is collected. The error metrics are in the
% following order:
%
% sErr.l0, sErr.numXorLoc, sErr.l1, sErr.lhard, sErr.lsoft, sErr.l2, nnz(xrecon)
%

nArgOut = nargout;

if ( nArgOut > 0 )
    
    sPerfResOut = sPerfRes;
    
    if false % Select the columns specified by errIndVec
        for ind = 1:length( sPerfResOut )
            tmp = sPerfResOut( ind ).mean;
            sPerfResOut( ind ).mean = tmp( :, errIndVec );
            tmp = sPerfResOut( ind ).std;
            sPerfResOut( ind ).std = tmp( :, errIndVec );
        end
    end
    
    nArgOut = nArgOut - 1; % decrement the number of assigned outputs
    
end

if ( nArgOut > 0 )
    sHypResOut = sHypRes;
end

