% Jan 28, 2007. This is the part that plots the performance results.
% Args: snr_db, fig_vec, err_ind_vec, s_perf_res, s_hyp_res, b_logy,
%       ssz_tag
% Output: none

function perf_plot( varargin )

% Defaults
%/////////

sszMetricName = { 'l_0 error', 'znz', 'l_1 error', 'hard', 'soft', 'l_2 error', 'nnz' };
sszPlotColour = { 'rx-', 'gs-', 'b.-', 'kd-', 'mv-' };

snrDbVec = [];
figVec = [];
errIndVec = [];

sPerfRes = struct( [] );
sHypRes = struct( [] );

sszTag = {};

bLogy = 0;

% Parse arguments
%////////////////

nArgIn = floor( length( varargin ) / 2 );
if 2*nArgIn ~= length( varargin ), error( 'expected even # args' ), end

for argInd = 1:nArgIn
    szKey = varargin{ 2*argInd - 1 };
    val = varargin{ 2*argInd };
    switch szKey
        case 'snr_db', snrDbVec = val;
        case 'fig_vec', figVec = val;
        case 'err_ind_vec', errIndVec = val;
        case 's_perf_res', sPerfRes = val;
        case 's_hyp_res', sHypRes = val;
        case 'b_logy', bLogy = val;
        case 'ssz_tag', sszTag = val;
        otherwise, error( sprintf( 'unrecognized key: %s', szKey ) )
    end
end

% Check arguments

[snrDbSortedVec, snrDbSortedInd] = sort( snrDbVec );

N_METHODS = length( sPerfRes );

if ( N_METHODS == 0 ), error( 'nothing to do' )
elseif ( N_METHODS == 1 )
    
    if length( figVec ) ~= ( length( errIndVec ) + 1 )
        error( 'N_METHODS = 1: unexpected length of figVec' )
    end
    
else
    
    if length( figVec ) ~= length( errIndVec )
        error( 'N_METHODS > 1: unexpected length of figVec' )
    end
    
end

if ~isempty( find( figVec <= 0 ) ), error( 'figVec should only have positive values' ), end
    
if bLogy
    if ( length( sszTag ) ~= N_METHODS )
        error( 'expected length of sszTag to equal N_METHODS' )
    end
end

% We plot the performance of one or multiple methods. If only plotting for
% one method, we create plots of performance and hyperparameter estimates.
% If plotting for 2 or more methods, only create performance plots.
%//////////////////////////////////////////////////////////////////////////


for errIndIter = 1:length( errIndVec )

    figure( figVec( errIndIter ) ), clf

    errInd = errIndVec( errIndIter );

    for methodIter = 1:N_METHODS

        perfMean = sPerfRes( methodIter ).mean( :, errInd );
        perfStd = sPerfRes( methodIter ).std( :, errInd );

        if ~bLogy
            errorbar( snrDbSortedVec, perfMean( snrDbSortedInd ), perfStd( snrDbSortedInd), ...
                char( sszPlotColour( methodIter ) ), 'LineWidth', 3, 'MarkerSize', 10 );
        else
            errorbar_semilogy( 'x', snrDbSortedVec, 'y', perfMean( snrDbSortedInd ), ...
                'e', perfStd( snrDbSortedInd ), 'linespec', char( sszPlotColour( methodIter ) ), ...
                'n_fig', figVec( errIndIter ), 'sz_tag', sszTag{ methodIter } );
        end
        
        if ( methodIter == 1 ), hold on, end;

    end
    
    if ~bLogy
        ylabel( char( sszMetricName( errInd ) ), 'Interpreter', 'none' );
    else
        ylabel( ['log( ', char( sszMetricName( errInd ) ), ' )' ], 'Interpreter', 'none' );
    end
    
    xlabel( 'SNR [dB]' );        
    axis tight
%     figfont

end
    
    
% Plot hyperparameter
%////////////////////

if ~isempty( sHypRes )
    
    if length( sHypRes ) == 1

        figure( figVec( end ) ), clf

        if ( size( sHypRes.mean, 2 ) ~= size( sHypRes.std, 2 ) ), error( 'inconsistent size' ), end;

        nHyperparam = size( sHypRes.mean, 2 );
        if nHyperparam > 9, error( 'cannot handle nHyperparam > 9' ), end;
        nWin = 100 + nHyperparam*10;

        for hyperparamIter = 1:nHyperparam
            subplot( nWin + hyperparamIter )
            hyperparamMean = sHypRes.mean( :, hyperparamIter );
            hyperparamStd = sHypRes.std( :, hyperparamIter );
            semilogy( snrDbSortedVec, hyperparamMean( snrDbSortedInd ), ...
                char( sszPlotColour( hyperparamIter ) ), 'LineWidth', 3, 'MarkerSize', 10 );
            axis tight, figfont
        end

    else
        warning( 'Cannot handle plotting more than 1 hyperparameter structure' )
    end

end

