% Feb 24, 2008. 
% Function to create figures for low/high SNR comparison of binary image with
% Gaussin psf.
% Args: vecSize
function tip07_hilow_binary( vecSize )

if ( nargin ~= 1 )
    error( 'missing arguments: run help tip07_hollow_binary' );
end

sObjSbl = load( 'sbl.mat', 'sRestore' );
sObjMap2 = load( 'map_revised.mat', 'sRestore' );
sObjLassoSure = load( 'lars_v2.mat', 'sRestore' );

sObjStompCfar = load( 'stomp_cfar.mat', 'sol' );

reconLimits = [ min( [ min( sObjSbl.sRestore.last(:) ), ...
                       min( sObjMap2.sRestore.last(:) ), ...
                       min( sObjLassoSure.sRestore.last(:) ) ] ), ...
                max( [ max( sObjSbl.sRestore.last(:) ), ...
                       max( sObjMap2.sRestore.last(:) ), ...
                       max( sObjLassoSure.sRestore.last(:) ) ] ) ];
            
nFig = 0;

nFig = incFigAndPlot( nFig, true, true, ...
    reconLimits, sObjSbl.sRestore.last, vecSize );
nFig = incFigAndPlot( nFig, true, true, ...
    reconLimits, sObjMap2.sRestore.last, vecSize );
nFig = incFigAndPlot( nFig, true, true, ...
    reconLimits, sObjLassoSure.sRestore.last, vecSize );

print -f1 -depsc2 tip07d6_hilow_binary_sbl.eps;
print -f2 -depsc2 tip07d6_hilow_binary_map2.eps;
print -f3 -depsc2 tip07d6_hilow_binary_lassos.eps;

nFig = incFigAndPlot( nFig, true, true, ...
    [], sObjStompCfar.sol, vecSize );

print -f4 -depsc2 tip07d6_hilow_binary_stomp_cfar.eps;

%////////////////////////////////////////////////////////////////
function nFig = incFigAndPlot( nFig, bUseLowresColormap, bFigfont, ...
    reconLimits, mat, vecSize )

% Check arguments
if ( length( mat(:) ) ~= prod( vecSize ) )
    error( 'length of mat != prod(vecSize)' );
end

nFig = nFig + 1;
figure( nFig ), clf;

imagesc( reshape( mat, vecSize ) );
axis on tight;

if ( bUseLowresColormap )
    tmp = lowresColormap;
    colormap( tmp );
end

set( gca, 'XTick', [] ); % Turn off x and y tick marks
set( gca, 'YTick', [] );

if ( ~isempty( reconLimits ) )
    if ( length( reconLimits ) ~= 2 )
        error( 'expected length of reconLimits to be 2' );
    end
    caxis( reconLimits );
end

colorbar;

if ( bFigfont )
    figfont;
end

%///////////////////////////
function M = lowresColormap

mSize = 8 * ones( 1, 4 );
    
M{1} = [ ones(mSize(1),1), linspace(1,0,mSize(1))', linspace(1,0,mSize(1))' ];
M{2} = [ linspace(1,0.5,mSize(2))', zeros(mSize(2),1), linspace(0.2,1,mSize(2))' ];
M{3} = [ linspace(0.5,0,mSize(3))', linspace(0.2,1,mSize(3))', zeros(mSize(3),1) ];
M{4} = [ zeros(mSize(4),1), ones(mSize(4),1), linspace(0,1,mSize(4))' ];

M = [ M{1}; M{2}; M{3}; M{4} ];


