
% Two delta prior on image, apply MAP 

function estParams = binaryd_map_optim2( zHat, alpha, bDebug, bPlot )

n = length( zHat );

wRange = [ 1e-2, .99 ];
wNumPts = 30;
thetaStarRange = [ 1e-2, 1.5 ];
thetaStarNumPts = 30;

vecW = linspace( wRange(1), wRange(2), wNumPts );
vecThetaStar = linspace( thetaStarRange(1), thetaStarRange(2), ...
    thetaStarNumPts );

for i = 1:wNumPts
    for j = 1:thetaStarNumPts
        vecCost( i, j ) = neg_q( [vecW(i), vecThetaStar(j)] );
    end
end

if ( nargin == 4 & bPlot == 1 )
    figure( 1 ), hist( zHat, 50 ), title( 'Histogram of zHat' );
    figure( 2 ), contour( vecThetaStar, vecW, vecCost, 15 );
    title( 'Contour of neg cost fcn' );
    %pause;
end

B_DEBUG = 0;
if ( nargin >= 3 & bDebug == 1 ), B_DEBUG = 1; end

% Locate the min of the grid as a starting point
minCost = min( vecCost(:) );
[wInd, thetaStarInd] = find( vecCost == minCost );
wHat0 = vecW( wInd(1) );
thetaStar0 = vecThetaStar( thetaStarInd(1) );
disp( sprintf( 'wHat0: %g, thetaStar0: %g', wHat0, thetaStar0 ) );

% Hmm, not possible to find gradient with thresholding operation?
options = optimset( 'TolX', 1e-4, 'TolFun', 1e-4, ...
    'Display', 'final' );

[xHat, fVal, exitFlag ] = fmincon( @neg_q, ...
    [wHat0, thetaStar0], ...
    [], [], [], [], ... % No linear equalities or inequalities
    [wRange(1), thetaStarRange(1)], [wRange(2), thetaStarRange(2)], ...
    [], ... % No non-linear constraints
    options );

if ( exitFlag <= 0 )
    error( 'fmincon failed in binaryd_optim' );
end

wHat = xHat(1);
thetaStarHat = xHat(2);

%warning off;
threshold = alpha^2/thetaStarHat * log( ( 1 - wHat )/wHat ) + thetaStarHat/2;
%warning on;

estParams = struct( 'w', wHat, 'thetaStar', thetaStarHat, ...
    'threshold', threshold, ...
    'wRange', wRange, 'thetaStarRange', thetaStarRange );

if B_DEBUG
    disp( sprintf( 'wHat: %g, thetaStar: %g, threshold: %g', ...
        wHat, thetaStarHat, threshold ) );    
end

    % Work with the NEGATIVE of q, as we'll be using fmincon
    
    function f = neg_q( x )
        w = x(1);
        thetaStar = x(2);
        
        %warning off;        
        threshold = alpha^2 / thetaStar * log( (1-w)/w ) + thetaStar/2;

        if ( thetaStar > 0 )
            thetaHatOptim = thetaStar * ( zHat > threshold );
        elseif ( thetaStar < 0 )
            thetaHatOptim = thetaStar * ( zHat < threshold );
        else
            thetaHatOptim = zeros( size( zHat ) );
        end

        numZeros = sum( abs( thetaHatOptim ) <= eps );
        f = -norm( thetaHatOptim - zHat, 2 )^2 / (2 * alpha^2) + ...
            log(1-w)*numZeros + log(w)*(n - numZeros);

        f = -f;
        %warning on;
    end

end