
% Assume two delta prior on image theta. Assume that the position of 
% both delta fcns are fixed, and we want to estimate the mixing parameter
% w.

function estParams = binaryd_optim1w( zHat, alpha, thetaStar, bDebug, ...
    bPlot )

wRange = [1e-2, 0.99];
wNumPts = 30;

vecW = linspace( wRange(1), wRange(2), wNumPts );
for i = 1:length( vecW )
    vecCost(i) = mml_cost_function( vecW(i) );
end

if ( nargin == 5 & bPlot == 1 )
    figure( 1 ), clf, hist( zHat, 50 ), title( 'Histogram of zHat' );
    figure( 2 ), clf, plot( vecW, vecCost, 'k' ), title( 'mml cost function' );
    %pause;
end

B_DEBUG = 0;
if ( nargin >= 4 & bDebug == 1 ), B_DEBUG = 1; end

% Pick the starting point as the index that maximizes mml_cost_function
% over the samples
[costMax, costMaxInd] = max( vecCost );
wHat0 = vecW( costMaxInd );

% Solve for the derivative = 0
options = optimset( 'MaxIter', 1e3, 'MaxFunEvals', 5*1e3, 'Display', 'final' );
[wHat, fVal, exitFlag] = fsolve( @mml_deriv_function, wHat0, options );

if ( exitFlag <= 0 )
    warning( 'fsolve of mml_deriv_function failed in binaryd_wfree_optim' );
    % Evaluate endpoints
    mmlValueStart = mml_cost_function( wRange(1) );
    mmlValueEnd = mml_cost_function( wRange(2) );
    if B_DEBUG
        disp( sprintf( 'mmlValueStart=%g, mmlValueEnd=%g', ...
            mmlValueStart, mmlValueEnd ) );
    end
    if ( mmlValueStart >= mmlValueEnd ), wHat = wRange(1);
    else, wHat = wRange(2);
    end
end

%warning off;
threshold = alpha^2/thetaStar * log( ( 1 - wHat )/wHat ) + thetaStar/2;
%warning on;

% Limit the threshold by the universal threshold
n = length( zHat );
tUniv = sqrt( 2*log(n) ) * alpha;
if ( ~isfinite( threshold ) | ( threshold > tUniv ) )
    if B_DEBUG, disp( sprintf( 'threshold=%g, tUniv=%g', threshold, tUniv ) ), end;
    threshold = tUniv;
end

estParams = struct( 'thetaStar', thetaStar, 'w', wHat, 'threshold', threshold, ...
    'wRange', wRange );

if B_DEBUG
    disp( sprintf( 'wHat: %g, threshold: %g', wHat, threshold ) );    
end

    % Nested functions to avoid global declarations. They are the
    % derivative and value of the marginalized ML fcn respectively.
    
    function f = mml_deriv_function( w )
        warning off; % Get rid of divide-by-zero warnings
        numer = normpdf( zHat, thetaStar, alpha ) - normpdf( zHat, 0, alpha );
        denom = normpdf( zHat, 0, alpha ) + w*numer;
        f = sum( numer ./ denom );
        warning on;
    end

    function f = mml_cost_function( w )
        warning off;
        f = sum( log( normpdf( zHat, 0, alpha ) + ...
            w*(normpdf( zHat, thetaStar, alpha ) - normpdf( zHat, 0, alpha )) ...
            ) );
        warning on;
    end
end


