
% Two delta prior on image, but optimize over w & thetaStar

function estParams = binaryd_optim2( zHat, alpha, bDebug, bPlot )

wRange = [ 1e-2, .99 ];
wNumPts = 30;
thetaStarRange = [ 1e-2 , 1.5 ];
thetaStarNumPts = 30;

vecW = linspace( wRange(1), wRange(2), wNumPts );
vecThetaStar = linspace( thetaStarRange(1), thetaStarRange(2), ...
    thetaStarNumPts );

for i = 1:wNumPts
    for j = 1:thetaStarNumPts
        vecCost( i, j ) = neg_mml_cost_function( [vecW(i), vecThetaStar(j)] );
    end
end

if ( nargin == 4 & bPlot == 1 )
    figure( 1 ), clf, hist( zHat, 50 ), title( 'Histogram of zHat' );
    figure( 2 ), clf, contour( vecThetaStar, vecW, vecCost, 15 );
    title( 'Contour of neg cost fcn' );
    %pause;
end

B_DEBUG = 0;
if ( nargin >= 3 & bDebug == 1 ), B_DEBUG = 1; end

% Locate the min of the grid as a starting point
minCost = min( vecCost(:) );
[wInd, thetaStarInd] = find( vecCost == minCost );
wHat0 = vecW( wInd(1) );
thetaStar0 = vecThetaStar( thetaStarInd(1) );
disp( sprintf( 'wHat0: %g, thetaStar0: %g', wHat0, thetaStar0 ) );

options = optimset( 'GradObj', 'on', 'DerivativeCheck', 'on', ...
    'TolX', 1e-4, 'TolFun', 1e-4, ...
    'Display', 'final' );

[xHat, fVal, exitFlag ] = fmincon( @neg_mml_cost_function, ...
    [wHat0, thetaStar0], ... % Initial starting point
    [], [], [], [], ... % No linear equalities or inequalities
    [wRange(1), thetaStarRange(1)], [wRange(2), thetaStarRange(2)], ...
    [], ... % No non-linear constraints
    options );

if ( exitFlag <= 0 )
    error( 'fmincon failed in binaryd_optim' );
end

wHat = xHat(1);
thetaStarHat = xHat(2);

%warning off;
threshold = alpha^2/thetaStarHat * log( ( 1 - wHat )/wHat ) + thetaStarHat/2;
%warning on;

estParams = struct( 'w', wHat, 'thetaStar', thetaStarHat, ...
    'threshold', threshold, ...
    'wRange', wRange, 'thetaStarRange', thetaStarRange );

if B_DEBUG
    disp( sprintf( 'wHat: %g, thetaStar: %g, threshold: %g', ...
        wHat, thetaStarHat, threshold ) );    
end

    % Work with the NEGATIVE of the cost function, as we'll be using
    % fmincon
    
    function [f, g] = neg_mml_cost_function( x )
        w = x(1);
        thetaStar = x(2);
        %warning off;
        numer1 = normpdf( zHat, thetaStar, alpha ) - normpdf( zHat, 0, alpha );
        denom = normpdf( zHat, 0, alpha ) + w*numer1;           
        f = -sum( log( denom ) );
        if ( nargout == 2 )
            % Compute the gradient             
            g(1) = -sum( numer1 ./ denom );
            numer2 = w * ...
                normpdf( zHat, thetaStar, alpha ) .* ( zHat - thetaStar );
            g(2) = -sum( numer2 ./ denom ) / alpha^2;
        end
        %warning on;
    end

end