% change_scale: change the scale of the bounding box
%#####################################################################

function sBoundingBox = change_scale( sBoundingBox, bCloseBoundingBox, ...
	vOptim, delta, paramLowerBound, paramUpperBound )

n = length( vOptim );

if ( length( paramLowerBound ) < n | length( paramUpperBound ) < n )
	error( 'param bounds not specified in change_scale' );
end

if delta < eps
	error( 'delta cannot be less than eps' );
end

% Iterate over the dimension of the hyperparameter vector
for i = 1:n    
    
    % Kludge: don't change the upper boundary for the 2nd hyperparameter. 
    % This applies to eb_better_nov05, and the hyperparameter of t, the 
    % threshold
    if ( i == 2 ), continue, end;
    
	if abs( vOptim(i) - paramLowerBound(i) ) < delta 

		% vOptim(i) is close to lower bound
		bCloseBoundingBox( i, 1 ) = 1;

		sBoundingBox.loBoundScale(i) = sBoundingBox.loBoundScale(i)/2;
		if ( sBoundingBox.loBoundScale(i) < sBoundingBox.loBoundScaleMin )
			error( sprintf( 'changing loBoundScale(%g) to less than tol', i ) );
		end

		warning( sprintf( 'changing loBoundScale(%g)=%g', i, ...
			sBoundingBox.loBoundScale(i) ) );

	elseif abs( vOptim(i) - paramUpperBound(i) ) < delta

		% vOptim(i) is close to upper bound               
		bCloseBoundingBox( i, 2 ) = 1;

		sBoundingBox.hiBoundScale(i) = sBoundingBox.hiBoundScale(i)*2;
		if ( sBoundingBox.hiBoundScale(i) > sBoundingBox.hiBoundScaleMax )
			error( sprintf( 'changing hiBoundScale(%g) to more than tol', ...
				i ) );
		end

		warning( sprintf( 'changing hiBoundScale(%g)=%g', i, ...
			sBoundingBox.hiBoundScale(i) ) );

	end

	% Because it's bad business to be increasing the bounding box, here's 
	% what we shall do. If the optimal value keeps on hitting the lower
	% bound and not the upper, we shall correspondingly shrink the upper
	% bound by the same amount. The same applies vice-versa. On the other hand,
	% if the optimal value has been close to both boundaries, don't attempt
	% to shrink or expand the boundaries to keep the same (log-scale) area.

	if ( bCloseBoundingBox( i, 1 ) == 1 & bCloseBoundingBox( i, 2 ) == 0 )
		sBoundingBox.hiBoundScale(i) = sBoundingBox.hiBoundScale(i)/2;
	elseif ( bCloseBoundingBox( i, 1 ) == 0 & bCloseBoundingBox( i, 2 ) == 1 )
		sBoundingBox.loBoundScale(i) = sBoundingBox.loBoundScale(i)*2;
	end

end
