% clev_ebayes: test the ebayes procedure with the cantilever psf
%-------------------------------------------------------------------------------
% [out, f] = clev_ebayes( sigmaFac )
%
% History:
%    23-feb-05: rendered obsolete by eb13
%
function [out, f] = clev_ebayes( sigmaFac )

% Parameters
DOUBLE_THRES = 1e-6;

par = psf_param3( 1, 3e4, 10 );
stMesh = psf_mesh( par );
numPts = stMesh.numXPts * stMesh.numYPts * stMesh.numZPts;

%%% XXX: assume for now that we only have a 2-d image
if ( stMesh.numZPts ~= 1 ), error( 'not using a 2-d image' ), end;

% Generate the output image
out = gen_img( par, stMesh, 'random', sigmaFac, [], 10 );

% Compute the variance in the image domain
h = real( get_h_op( par, stMesh, 1 ) );
facVar = sum( sum( h.^2 ) );
disp( sprintf( 'facVar = %g', facVar ) );
 
% Generate the ML estimator
f.fML = real( f_deconv( stMesh, out.h, out.gNoisy ) );

%%% We shall test out two alternatives using the ML estimator. 

% A: clean up gNoisy and then apply ML
vecGNoisy = reshape( real( out.gNoisy ), [numPts, 1] );
disp( ' === Method A: fCleanML ===' );
ebOut = ebayesthresh( vecGNoisy, 'laplace', [], 0, out.sigma1, 1, 'median', ...
	0 )

% XXX: hmm, are we specifying the dimensions of gClean correctly? I guess it
% doesn't matter as long as the number of pts in the x-dir and y-dir are the
% same.
gClean = reshape( ebOut.muhat, [stMesh.numXPts, stMesh.numYPts] );

% XXX: discard negative values
%gClean = ( gClean > -DOUBLE_THRES ) .* gClean;

f.fCleanML = real( f_deconv( stMesh, out.h, gClean ) );

% B: apply ML and then clean up the signal
vecfML = reshape( f.fML, [numPts, 1] );
disp( ' === Method B: fMLClean === ' );
ebOut = ebayesthresh( vecfML, 'laplace', [], 0, [], 1, 'median', 0 )
f.fMLClean = real( reshape( ebOut.muhat, [stMesh.numXPts, stMesh.numYPts] ) );

% DEBUG
%return

% C: apply ebayesthresh to A
vecfCleanML = reshape( f.fCleanML, [numPts, 1] );
disp( ' === Method C: fCleanMLClean === ' );
ebOut = ebayesthresh( vecfCleanML, 'laplace', [], 0, [], 1, 'median', 0 )
f.fCleanMLClean = real( reshape( ebOut.muhat, ...
	[stMesh.numXPts, stMesh.numYPts] ) );

% XXX: discard negative values
f.fCleanMLClean = (f.fCleanMLClean >= -DOUBLE_THRES) .* f.fCleanMLClean;

% D: method B but we compute the noise in the image domain
disp( ' === Method D: B + true sigma === ' );
ebOut = ebayesthresh( vecfML, 'laplace', [], 0, out.sigma1/sqrt(facVar), ...
	1, 'median', 0 )
f.fMLCleanD = real( reshape( ebOut.muhat, [stMesh.numXPts, stMesh.numYPts] ) );

% Test other methods here
%f.fBS = f_bstr_sd( par, stMesh, out.h, out.gNoisy );

%%% Now plot the images and compare them
create_int_plot( 1, 'gNoisy', [], out.X, out.Y, real(out.gNoisy), 1 );
create_int_plot( 2, 'gClean', [], out.X, out.Y, gClean, 1 );
%create_int_plot( 3, 'g', [], out.X, out.Y, real(out.g), 1 );
create_int_plot( 3, 'f', [], out.X, out.Y, real(out.f), 1, 1 );
create_int_plot( 4, 'fML', [], out.X, out.Y, f.fML, 1, 1 );
create_int_plot( 5, 'fCleanML', [], out.X, out.Y, f.fCleanML, 1, 1 );
create_int_plot( 6, 'fMLClean', [], out.X, out.Y, f.fMLClean, 1, 1 );
create_int_plot( 7, 'fCleanMLClean', [], out.X, out.Y, f.fCleanMLClean, 1, 1 );
create_int_plot( 8, 'fMLCleanD', [], out.X, out.Y, f.fMLCleanD, 1, 1 );
%create_int_plot( 9, 'fBS', [], out.X, out.Y, f.fBS, 1, 1 );

