% cmp_thres: compare the median and MAP thresholding functions
%

diary off, delete 'cmp_thres.log', diary 'cmp_thres.log';

numTrials = 100;
n = 1e3;

% Params for prior
a = linspace( 0.1, 2, 20 );
w = 0.1;

% Params for Johnstone's data
k = 5;
mu = linspace( 3, 7, 9 );
%mu = [3];

sigma = 1;

aSseMed = [];
aSseMap = [];

MAP_GRADIENT_SEARCH_FAILURES_MAX = -1;

%for aTest = a
for muTest = mu

	sseMed = [];
	sseMap = [];

	%disp( sprintf( 'aTest = %g', aTest ) );
	disp( sprintf( 'muTest = %g', muTest ) );

	numMapGradientSearchFailures = 0; % Initialize

	for indTrial = 1:numTrials

		% Use the prior
		%[fclean, fnoisy] = gen_laplace_data( n, aTest, w, sigma );

		% Use Johnstone's data
		[fclean, fnoisy] = gen_john_data( n, k, muTest, sigma );

		%estMed = laplace_posterior_med( fnoisy(:), aTest, w );
		%estMap = laplace_pseudo_map( fnoisy(:), aTest, w, sigma );
		sOptions = struct( 'bSkipInitialGridSearch', 0, 'nDebugFlag', 0 );
		[estParamsMed, estMed] = embayfunc( fnoisy(:), 'laplace', [], ...
			sigma, 'median', sOptions );
		%estMap = laplace_map_thres0( fnoisy(:), ...
		%	estParamsMed.a, estParamsMed.w, sigma );

		if ( numMapGradientSearchFailures < MAP_GRADIENT_SEARCH_FAILURES_MAX )

			% Do a gradient search to find the unknown parameters
			[estParamsMap, estMap] = mapfunc( fnoisy(:), 'laplace', [], ...
				sigma, 'map', 0 );
			
			% Check to see if a gradient search failure occurred
			if ( estParamsMap.exitFlag <= 0 )
				numMapGradientSearchFailures = ...
					numMapGradientSearchFailures + 1;
			end

		else

			% Skip the gradient search
			[estParamsMap, estMap] = mapfunc( fnoisy(:), 'laplace', [], ...
				sigma, 'map_no_grad', 0 );

		end

		sseMed = [ sseMed, sum( (fclean(:) - estMed(:)).^2 ) ];
		sseMap = [ sseMap, sum( (fclean(:) - estMap(:)).^2 ) ];
	end

	aSseMed = [ aSseMed, mean( sseMed ) ];
	aSseMap = [ aSseMap, mean( sseMap ) ];

end

figure( 1 ), clf; 

% Using prior
%plot( a, aSseMed, 'bx' ), hold on, plot( a, aSseMap, 'ro' );
%title( 'Average of SSE over 100 trials' );
%xlabel( 'a' );
%ylabel( sprintf( 'w = %g', w ) );

% Using Johnstone's data
plot( mu, aSseMed, 'bx' ), hold on, plot( mu, aSseMap, 'ro' );
title( 'Average of SSE over 100 trials' );
xlabel( '\mu' );
ylabel( sprintf( 'k = %g', k ) );

%legend( 'Median thresholding', 'MAP thresholding', 0 );
%legend( 'Median thresholding', 'MAP thresholding using median threshold', 0 );


diary off;
