
% compute_ssim: From Wang et al., IEEE Trans. of Image Proc., (13) 4, 2004.

function s = compute_ssim( varargin )

x = [];
y = [];
C = [];
w = [];

% Process input arguments

lenVarargin = length( varargin );
numArgsIn = floor( lenVarargin/2 );

if 2*numArgsIn ~= lenVarargin, error( 'even ip args expected' ), end

for argInInd = 1:numArgsIn
    
    szKey = lower( varargin{2*argInInd-1} );    
    switch szKey
        case 'x', x = varargin{2*argInInd}; x = x(:);
		case 'y', y = varargin{2*argInInd}; y = y(:);
		case 'c', C = varargin{2*argInInd};
		case 'w', w = varargin{2*argInInd};
        otherwise
            error( sprintf( 'unrecognized option %s', szKey ) );
    end
end

% Check arguments
if length( x ) ~= length( y ), error( 'lengths of x and y unequal' ), end;
if length( C ) ~= 3 | length( w ) ~= 3, error( 'lengths C,w ~= 3' ), end;

% Compute global ssim

mu_x = mean( x );
mu_y = mean( y );
l_xy = ( 2*mu_x*mu_y + C(1) ) / ( mu_x^2 + mu_y^2 + C(1) );

sigma_x = std( x - mu_x );
sigma_y = std( y - mu_y );
c_xy = ( 2*sigma_x*sigma_y + C(2) ) / ( sigma_x^2 + sigma_y^2 + C(2) );

if ( abs(sigma_x) < eps ) | ( abs(sigma_y) < eps )
    warning( 'sigma_x or sigma_y is zero => R = nan' );
    R = nan;
    s = nan;
else
    R = corrcoef( ( x - mu_x ) ./ sigma_x, ( y - mu_y ) ./ sigma_y );
    s_xy = ( R(1,2) + C(3) ) / ( sigma_x * sigma_y + C(3) );
    ssimGlobal = l_xy.^w(1) * c_xy.^w(2) * s_xy.^w(3);
    s = ssimGlobal;
end

