% convert_tsigma2w: From (a,t,sigma), obtain w
%
% Usage:
%    w = convert_tsigma2w( a, t, sigma, szThresRule )
%

function w = convert_tsigma2w( a, t, sigma, szThresRule )

%if ( (a <= 0) | (t <= 0) | (sigma <= 0) )
%	warning( 'convert_tsigma2w: illegal parameter values' );
%end

switch ( lower(szThresRule) )

case 'median'
	tTilde = t/sigma;
	aTilde = a*sigma;

	num = 1/sqrt(2*pi) * exp( -tTilde^2/2 );

	denom = 0.5 * aTilde * exp( aTilde^2/2 ) * ...
		( exp( -aTilde*tTilde )*normcdf( tTilde - aTilde ) - ...
	  	  exp( aTilde*tTilde  )*normcdf( -tTilde - aTilde ) ) + num;

	% XXX: DEBUG
	%disp( sprintf( 'num: %g, denom: %g', num, denom ) );

	w = num / denom;

	% Sanity
	if ( ( w < 0 ) | ( w > 1 ) )
		disp( 'Illegal value of w!' );
		error( 'Aborting' );
	end

case 'map'
	if ( t >= a*sigma^2 )
		tmp = (t / sigma - a*sigma)^2;
		w = 1 / ( 1 + exp( tmp/2 ) );
	else
		disp( 't >= a*sigma^2' );
		error( 'Aborting' );
	end

otherwise
	error( 'Undefined thresholding function' );
end


