% convert_wsigma2t: From (a,w,sigma), obtain the t
%
% Usage:
%    t = convert_wsigma2t( a, w, sigma, szThresRule )
%

function t = convert_wsigma2t( a, w, sigma, szThresRule )

global localA localW localSigma

localA = a;
localW = w;
localSigma = sigma;

B_DEBUG = 0;

switch ( lower(szThresRule))

case 'median'
    options = optimset( 'FunValCheck', 'on', 'TolX', 1e-6 );
	[t, fEqnVal, exitFlag] = fzero( @fEqn, sigma, options );

    % Check result of fzero
    if ( exitFlag <= 0 ), error( 'convert_wsigma2t: fzero failed' ), end
    if ( t <= 0 )
        disp( [sprintf( 'fzero returned t=%g, fEqn(t)=%g!\n', t, fEqnVal ), ...
               sprintf( 'input parameters a=%g, w=%g, sigma=%g', a, w, sigma )] ...
               );
    end
        
    
    numNewtonIters = 10;
    
    while ( ( t <= 0 ) & ( numNewtonIters > 0 ) )
        tBetter = refine_js_threshold( 'a', a, 'w', w, 'sigma', sigma, 't0', t );
        t = tBetter;
        fEqnVal = fEqn( t );
        numNewtonIters = numNewtonIters - 1;
    end
    
%     % Check the value of fEqn( t ), which should be 0
%     if ( abs( fEqnVal ) > 1e3*eps )
% 		warning( 'convert_wsigma2t: fEqn not satisfied' );
% 		warning( sprintf( 'fEqnVal=%g', fEqnVal ) );
%     end

    fprintf( 'convert_wsigma2t RESULTS...' );
    if ( t > 0 )
        disp( sprintf( 'Ok: t=%g, fEqn(t)=%g', t, fEqnVal ) );
    else
        disp( sprintf( 'FAILED: t=%g, fEqn(t)=%g', t, fEqnVal ) );
    end
    
    if B_DEBUG
        xVec = linspace( -abs(t), abs(t), 100 );
        yVec = fEqn( xVec );
        figure( 1 ), plot( xVec, yVec, 'r' );
    end
    
case 'map'
	if ( ( w > 0 ) & ( w <= 0.5 ) )
		t = a*sigma^2 + sqrt( 2*sigma^2*log( (1-w)/w ) );
	elseif ( ( w > 0.5 ) & ( w <= 1 ) )
		t = a*sigma^2;
	else
		warning( 'convert_wsigma2t: illegal w' );
		t = -1;
	end

otherwise
	error( 'Undefined thresholding function' );

end

%%% solution of fEqn = 0 is t 
function c = fEqn( t )

global localA localW localSigma

a = localA;
w = localW;
sdev = localSigma;
c = w*a*exp( -a*t + (a*sdev)^2/2 ).*( 1 - erf( (-t/sdev + a*sdev)/sqrt(2) ) ) ...
	- 2*fLaplacianGauss( -t, a, w, sdev );



