% eb13: Implements the heurestic methods based on EB 
%
% Usage:
%    sRecon = eb13( hForwardOp, imgNoisy, noiseSigma )
%
%    sRecon - structure containing the restroncuted images (eb1,eb2,eb3)
%    hForwardOp - forward operator in convolution form
%    imgNoisy - noisy image
%    noiseSigma - noise variance
%

function sRecon = eb13( hForwardOp, imgNoisy, noiseSigma )

sClock = clock;
szLogFilename = sprintf( 'eb13_%02d%02d%02d.log', sClock(3), ...
	sClock(4), sClock(5) );
% DEBUG
disp( sprintf( 'Using log file: %s', szLogFilename ) );

% Start diary
diary off, delete( szLogFilename ), diary( szLogFilename );

sOptions = struct( 'bSkipInitialGridSearch', 0, 'nDebugFlag', 0 );

%%% EB1: clean up the noisy image and apply the pseudoinverse operator
banner( 'EB1' );
[estParams, thetaHat] = embayfunc( real( imgNoisy(:) ), 'laplace', [], ...
	noiseSigma, 'median', sOptions );
imgClean = reshape( thetaHat, size(imgNoisy) );
imgEb1 = pseudoinv_eqdim( hForwardOp, imgClean );

%%% EB2: apply the pseudoinverse operator and then clean up the image
banner( 'EB2' );
imgPseudoInv = pseudoinv_eqdim( hForwardOp, imgNoisy );
[estParams, thetaHat] = embayfunc( imgPseudoInv(:), 'laplace', [], ...
	[], 'median', sOptions );
imgEb2 = reshape( thetaHat, size(imgNoisy) ); 

%%% EB3: clean up EB1
banner( 'EB3' );
[estParams, thetaHat] = embayfunc( imgEb1(:), 'laplace', [], ...
	[], 'median', sOptions );
imgEb3 = reshape( thetaHat, size(imgNoisy) );

sRecon = struct( 'eb1', imgEb1, 'eb2', imgEb2, 'eb3', imgEb3, ...
	'pinv', imgPseudoInv, 'imgc', imgClean );

diary off;
