% eb_check: check the results in the Johnstone & Silverman article
%-------------------------------------------------------------------------------
% Want to duplicate some of the simulation results in "Needles and Straw in
% Haystacks: etc."
%

numTrials = 100;
vecL2a = [];
vecL2b = [];
vecL1a = [];
vecL1b = [];

kVals = 500;
muVals = 3;
vecLen = 1000;

tic; % Start timing

for ind = 1:numTrials

	% Conduct experiment
	[mu, z] = gen_john_data( numTrials, kVals, muVals, 1 );

	% Call the supplied function translated from Johnstone's R code
	f1 = ebayesthresh( z, 'laplace', [], 0, 1, 1, 'median', 0 );

	% Our home-cooked version
	sOptions = struct( 'bSkipInitialGridSearch', 0, 'nDebugFlag', 0 );
	f2 = embayfunc( z, 'laplace', [], 1, 'pseudo-map', sOptions );
	 
	% Count errors
	L2 = sqrt( sum( (f1.muhat(:) - mu(:)).^2 ) );
	vecL2a = [vecL2a, L2];

	L2 = sqrt( sum( (f2(:) - mu(:)).^2 ) );
	vecL2b = [vecL2b, L2];

	L1 = sum( abs(f1.muhat(:) - mu(:)) );
	vecL1a = [vecL1a, L1];

	L1 = sum( abs(f2(:) - mu(:)) );
	vecL1b = [vecL1b, L1];
end

toc; % Stop timing

disp( sprintf( ' ===< nnz: %g, val: %g, numTrials: %g >=== ', ...
	kVals, muVals, numTrials ) );
disp( sprintf( 'sse(a): %g', mean( vecL2a.^2 ) ) );
disp( sprintf( 'sse(b): %g', mean( vecL2b.^2 ) ) );
disp( sprintf( 'L2a : %g', mean(vecL2a) ) );
disp( sprintf( 'L2b : %g', mean(vecL2b) ) );
disp( sprintf( 'L1a : %g', mean(vecL1a) ) );
disp( sprintf( 'L1b : %g', mean(vecL1b) ) );


