% eb_log_lik: computes log likelihood for the EB method
%-------------------------------------------------------------------------------
% Assume the framework of the Johnstone paper. 
%
% Usage:
%    loglik = eb_log_lik( w, a, vecX )
%
% Inputs:
%    w - weight
%    a - scale factor for non-zero density
%    vecX - vectors of observations with noise variance normalized to 1
% 
% Outputs:
%    loglik - log likelihood
%
function logLik = eb_log_lik( w, a, vecX )

vecX = vecX(:);
p1 = normpdf( vecX );
p2 = 1/4*a*exp( a^2/2 ) * ( exp( a*vecX) .* (1 - erf( (vecX+a)/sqrt(2) )) + ...
	exp( -a*vecX) .* (1 + erf( (vecX-a)/sqrt(2) )) );
v = (1-w)*p1 + w*p2;

% Lower bound the entries of v
i = find( v < 1e-6 );
if any( i ), v(i) = 1e-6; end

logLik = sum( log( v ) );

