% ebayestest: tests the EbayesThresh algorithm using uniform non-zero values
%-------------------------------------------------------------------------------
% [f, sse] = ebayestest( n, k, muVal, bCorrNoise, bPlot )
%
function [f, sse] = ebayestest( n, k, muVal, bCorrNoise, bArgPlot )

if ( muVal == 0 ), error( 'muVal must be non-zero' ), end;
if ( k >= n ), error( 'must have n > k' ), end;

bPlot = 0;
if ( ( nargin == 5 ) & ( bArgPlot == 1 ) ), bPlot = 1; end

% Instead of i.i.d. noise, introduce a correlation in the noise
if ( ( nargin >= 4 ) & ( bCorrNoise == 1 ) )
	A = diag( ones( n, 1 ) ) + diag( 0.3*ones( n-1, 1 ), 1 ) + ...
		diag( -0.3*ones( n-1, 1 ), -1 );
	noise = A * randn( n, 1 );
else
	noise = randn( n, 1 );
end

mu = zeros( size( noise ) );

% Populate mu with k non-zero elements, each with value muVal
numNonzeroElems = 0;
while ( numNonzeroElems < k )
	muPos = 1 + floor( rand * n );
	% Check if muPos is valid
	if ( ( muPos < 1 ) | ( muPos > n ) ), error( 'invalid muPos' ), end;
	% Check if mu(muPos) has been already filled; if not, fill it
	if ( mu(muPos) == 0 )
		mu(muPos) = muVal;
		numNonzeroElems = numNonzeroElems + 1;
	end
end

% Sanity: check that there are indeed k non-zero elements
chk = round( sum( mu ) / muVal );
if ( chk ~= k ), error( 'mu does not have k nnz elems' ), end;
 
z = mu + noise;
[estParams, f.muhat] = embayfunc( z, 'laplace', [], 1, 'median', struct( 'bSkipInitialGridSearch', 0, ...
    'nDebugFlag', 0 ) );
f.x = z;

% Compute sum of squared error (SSE)
sse = sum( ( f.muhat - mu ).^2 );

% Make plot
if ( bPlot == 1 )
	close all;
	figure( 1 ), subplot( 121 ), plot( f.x, 'rx' ); 
	subplot( 122 ), plot( f.muhat, 'bo' ), hold on;
	nnzInd = find( abs(mu) > eps );
	plot( nnzInd, mu(nnzInd), 'gd' );
	plotMax = max( [max( abs(f.muhat) ), abs(muVal)] ) * 1.1;
	ax = axis;
	axis( [ ax(1), ax(2), -plotMax, plotMax ] );
end

