% ebayestest2: tests the EbayesThresh algorithm using non-uniform non-zero vals
%-------------------------------------------------------------------------------
% [f, sse] = ebayestest2( n, k, vecMuVal, sNzAssign, bPlot )
%
function [f, sse] = ebayestest2( n, k, vecMuVal, sNzAssign, bArgPlot )

if ( k >= n ), error( 'must have n > k' ), end;

bPlot = 0;
if ( ( nargin == 5 ) & ( bArgPlot == 1 ) ), bPlot = 1; end

noise = randn( n, 1 );
mu = zeros( size( noise ) );
numMuVals = length( vecMuVal );

% Populate mu with k non-zero elements, with each non-zero element having
% a random value in vecMuVal.
numNonzeroElems = 0;

while ( numNonzeroElems < k )
	muPos = 1 + floor( rand * n );
	% Check if muPos is valid
	if ( ( muPos < 1 ) | ( muPos > n ) ), error( 'invalid muPos' ), end;
	% Check if mu(muPos) has been already filled; if not, fill it according
	% to the value of sNzAssign
	if ( mu(muPos) == 0 )
		switch ( lower( sNzAssign ) )
		case 'random'
			valInd = 1 + floor( rand*numMuVals );
		case 'iter'
			valInd = 1 + mod( numNonzeroElems, numMuVals );
		otherwise
			error( 'invalid sNzAssign' );
		end 
		mu(muPos) = vecMuVal( valInd );
		numNonzeroElems = numNonzeroElems + 1;
	end
end

% Sanity: check that there are indeed k non-zero elements
chk = length( find( abs(mu) > eps ) );
if ( chk ~= k ), error( 'mu does not have k nnz elems' ), end;
 
z = mu + noise;
f = ebayesthresh( z, 'laplace', [], 0, 1, 1, 'median', 0 );

% Compute sum of squared error (SSE)
sse = sum( ( f.muhat - mu ).^2 );

% Make plot
if ( bPlot == 1 )
	close all;
	figure( 1 ), subplot( 121 ), plot( f.x, 'rx' ); 
	subplot( 122 ), plot( f.muhat, 'bo' ), hold on;
	nnzInd = find( abs(mu) > eps );
	plot( nnzInd, mu(nnzInd), 'gd' );
	plotMax = max( [max( abs(f.muhat) ), max( abs(vecMuVal) )] ) * 1.1;
	ax = axis;
	axis( [ ax(1), ax(2), -plotMax, plotMax ] );
end

