
% emgauss_restore: Implements the EM algorithm for Gaussian statistics.
% 
% Based on em_restore

function [sRestore, sRestoreOptions] = emgauss_restore( varargin )

global sBoundingBox bCloseBoundingBox sRestoreOptions

nConvMethod = 1; % 1 => use DFTs, 2 => use convolution

% Default restore values
sRestoreOptions = struct( 'FIXED_AVAL', 1, ...
    'FIXED_WVAL', 1e-2, ...
    'FIXED_TVAL', -1, ...
    'FIXED_SPARSE_IND', 'w', ...
    'STOPPING_COND', 'fixed', ...
    'STOPPING_COND_TOL', 1e-2, ...
    'STOPPING_COND_WIN', 1e2, ...
    'STOPPING_COND_LAST_FAC', 1, ...
    'THETASTAR_FIXED', 1, ...
    'SOFT_THRES_T', 1, ...
    'GRADIENT_SEARCH_FAILURES_MAX', 1, ...
    'EM_SKIP_INITIAL_GRID_THRES', 5, ...   
    'NUM_EM_ITERS', 0, ...      
    'THETA0_SELECTOR', 'obs', ...
    'THETA0', [], ...
    'numGradientSearchFailures', 0, ...
	'nConvMethod', nConvMethod, ...	
    'nHparamRestInterval', 1e3, ...
	'bSeparateHparamEst', 1, ...
	'gSpecial', 1, ...
	'numThresWarning', 0, ...
	'bUpperBoundThres', 0, ...
    'lambda', 0, ...
    'hard_soft_frac', 0 ...
    );

% Process input arguments

lenVarargin = length( varargin );
numArgsIn = floor( lenVarargin/2 );

if 2*numArgsIn ~= lenVarargin, error( 'even ip args expected' ), end

for argInInd = 1:numArgsIn
    
    szKey = lower( varargin{ 2*argInInd-1 } );    
    ip = varargin{ 2*argInInd };
    switch szKey
        case 'forward_op', hForwardOp = ip;
        case 'noisy_output', imgNoisy = ip;
        case 'noise_sigma', noiseSigma = ip;
        case 'theta_true', thetaTrue = ip;
        case 'm_selector', szMSelector = ip;       
        case 'num_max_grad_search_failures', numMaxGradientSearchFailures = ip;
                                 
        case 'a_val', sRestoreOptions.FIXED_AVAL = ip;
        case 'w_val', sRestoreOptions.FIXED_WVAL = ip;
        case 't_val', sRestoreOptions.FIXED_TVAL = ip;
        case 'sparse_ind', sRestoreOptions.FIXED_SPARSE_IND = ip;
        case 'stopping_cond', sRestoreOptions.STOPPING_COND = ip;
        case 'stopping_cond_tol', sRestoreOptions.STOPPING_COND_TOL = ip;
        case 'stopping_cond_win', sRestoreOptions.STOPPING_COND_WIN = ip;
        case 'stopping_cond_last_fac', sRestoreOptions.STOPPING_COND_LAST_FAC = ip;
        case 'thetastar', sRestoreOptions.THETASTAR_FIXED = ip;
        case 'soft_thres_t', sRestoreOptions.SOFT_THRES_T = ip;
        case 'num_em_iters', sRestoreOptions.NUM_EM_ITERS = ip;
        case 'b_save_history', sRestoreOptions.B_SAVE_HISTORY = ip;
        case 'theta0_selector', sRestoreOptions.THETA0_SELECTOR = ip;
        case 'theta0', sRestoreOptions.THETA0 = ip;
		case 'sboundingbox', sBoundingBox = ip;
		case 'bcloseboundingbox', bCloseBoundingBox = ip;
        case 'hparam_rest_interval', sRestoreOptions.nHparamRestInterval = ip;
        case 'bseparate_hparam_est', sRestoreOptions.bSeparateHparamEst = ip;
		case 'gspecial', sRestoreOptions.gSpecial = ip;
        case 'lambda', sRestoreOptions.lambda = ip;
        case 'hard_soft_frac', sRestoreOptions.hard_soft_frac = ip;
            
        otherwise
            error( sprintf( 'unrecognized option %s', szKey ) );
    end
end

% WTF? Convert thetaTrue to double. It seems that sometimes we get a logical
% array!
thetaTrue = double( thetaTrue );

n = length( imgNoisy(:) );

if ndims( thetaTrue ) == 2
    B_USE_FFT2 = 1;
    disp( 'Setting B_USE_FFT2=1' );
else
    B_USE_FFT2 = 0;
    disp( 'Setting B_USE_FFT2=0' );
end

DISP_UPDATE_FREQ = 100;

B_DISP_PROGRESS_MSGS = 0;

% Use DFTs
if B_USE_FFT2, imgNoisyDft = fft2( imgNoisy );
else, imgNoisyDft = fftn( imgNoisy );
end

% Must zero-pad the psf
if B_USE_FFT2, hForwardOpDft = fft2( hForwardOp, size( imgNoisy,1 ), size( imgNoisy,2 ) );
else, hForwardOpDft = fftn( hForwardOp, size( imgNoisy ) );
end

sResH = emsep_comp_alpha( hForwardOpDft, noiseSigma );

disp( sprintf( '|H| = %g', sResH.norm2 ) );
disp( sprintf( 'cond(H) = %g', sResH.cond ) );

sRestoreOptions.daub1NormConst = 1.1 * sResH.norm2;
sRestoreOptions.daub2NormConst = 1.1 * (sResH.norm2)^2;

if strcmp( szMSelector, 'daub1' ) == 1 | ...
   strcmp( szMSelector, 'daub_hard_soft' ) == 1
    
    if abs( sRestoreOptions.daub1NormConst ) < 1e6 * eps
        error( 'daub1NormConst = 0' );
    end
    
    % Normalize hForwardOp
    hForwardOp = hForwardOp/sRestoreOptions.daub1NormConst;
    
    % Re-compute hForwardOpDft
    warning( 'daub1: normalizing H' );
    
    if B_USE_FFT2
        hForwardOpDft = fft2( hForwardOp, size( imgNoisy,1 ), size( imgNoisy,2 ) );
    else
        hForwardOpDft = fftn( hForwardOp, size( imgNoisy ) );
    end
    
    sResH = emsep_comp_alpha( hForwardOpDft, noiseSigma );  
    
    disp( sprintf( 'new |H| = %g', sResH.norm2 ) );
    disp( sprintf( 'cond(H) = %g', sResH.cond ) );
    
end

% Initialize Gblur object

if nConvMethod == 1
	G = Gblur( ones( size( imgNoisy ) ), ...
        'psf', hForwardOp, ...
        'type', 'conv,fft', ...
        'nDim', ndims( thetaTrue ) );
elseif nConvMethod == 2
	G = Gblur( ones( size( imgNoisy ) ), ...
        'psf', hForwardOp, ...
        'type', 'conv,same', ...
        'nDim', ndims( thetaTrue ) );
else
    error( 'illegal nConvMethod' );
end

% Set initial value of theta

switch sRestoreOptions.THETA0_SELECTOR 
    case 'deconv'
        % Apply the pseudo-inverse to the noise observation
        deconvSoln = pseudoinv_eqdim( hForwardOp, imgNoisy );
        thetaHat0 = deconvSoln;
    case 'zero'
        thetaHat0 = zeros( size( imgNoisy ) );
    case 'checker'
        thetaHat0 = create_checker( size( imgNoisy ), 0.5, 1 );
    case 'custom'
        thetaHat0 = sRestoreOptions.THETA0;
    case 'true'
        thetaHat0 = thetaTrue;
    case 'obs'
        thetaHat0 = imgNoisy;
    case 'Hty'
        thetaHat0 = G' * imgNoisy;
	case 'Hty2'
        thetaHat0 = G' * imgNoisy;
        
        % Only works with Matlab 7
        %thetaHat0AbsSorted = sort( abs( thetaHat0(:) ), 'descend' );
        
        thetaHat0AbsSorted = sort( abs( thetaHat0(:) ) ); % Default sort is in ascending order
        thetaHat0AbsSorted = thetaHat0AbsSorted(end:-1:1); % Reverse order
        
        absPosInd = find( thetaHat0AbsSorted > eps );
        k = floor( 0.8 * length( absPosInd ) );
        thresh0 = thetaHat0AbsSorted( k );
        disp( sprintf( 'thresh0=%g', thresh0 ) );
        
        if 0, % SANITY
            figure( 2 ), clf, plot( thetaHat0AbsSorted(:), 'k.' );
            title( 'Sorted |thetaHat0| values' );
            disp( 'Pausing...' ), pause;
        end
        
        % SANITY
        if ( abs(thresh0) < eps ), error( 'thresh0 < 0' ), end;
		thetaHat0 = ( thetaHat0 - sign(thetaHat0)*thresh0 ) .* ( abs(thetaHat0) > thresh0 );
    otherwise
        error( 'invalid selection for theta0' );
end

% Adjust initial value for daub1/daub_hard_soft method
if strcmp( szMSelector, 'daub1' ) == 1 | ...
   strcmp( szMSelector, 'daub_hard_soft' ) == 1
    thetaHat0 = thetaHat0 * sRestoreOptions.daub1NormConst;
end

% Extract upper bound for alpha
alphaUpperBound = sResH.alphaUpperBound;
alpha = sqrt(0.9)*alphaUpperBound;
sRestoreOptions.alpha = alpha;

% Set tau. As well, check that sRestoreOptions.lambda is strictly positive
% for daub1 and daub2.

switch ( szMSelector )
    case 'soft_fixed'
        sRestoreOptions.tau = 1;  
        if sResH.norm2 >= (1-1e3*eps)
            error( '|H|>= 1, tau cannot be 1' );
        end
        
    case { 'daub1', 'daub_hard_soft' }
        sRestoreOptions.tau = 1;
        if ( sRestoreOptions.lambda <= 0 ), error( 'illegal lambda' ), end;
                    
    case 'daub2'
        sRestoreOptions.tau = 1/sRestoreOptions.daub2NormConst;
        disp( sprintf( 'daub2: tau=%g', sRestoreOptions.tau ) );
        if ( sRestoreOptions.lambda <= 0 ), error( 'illegal lambda' ), end;

    case 'id'
        % Standard Landweber iterations. The parameter tau must satisfy:
        % 0 < tau < 2/rho(H'H). Use the fact that rho(H'H) = |H|_2^2.
        sRestoreOptions.tau = 1 / (sResH.norm2)^2;
        
    otherwise
        sRestoreOptions.tau = (alpha/noiseSigma)^2;
end

% Set mData

switch ( szMSelector )
    case 'map1'
        mData = [ abs(thetaHat0(:)) > eps, thetaHat0(:) ];
        
    case 'map2'
        mData = thetaHat0(:);
        
    otherwise
        mData = [];
end
        
thetaHatPrev = thetaHat0;

if 0, % SANITY
	if ( ndims( thetaHat0 ) == 2 )
		figure( 1 ), clf, imagesc( thetaHat0 ), colorbar;
		title( 'initial image' );
	else
		disp( 'Not 2-d problem, skipping plot of thetaHat0' );
    end	
end

disp( sprintf( 'nnz=%g', sum( abs(thetaHat0(:))>eps ) ) );

% History of hyperparameter estimates and their range
hyperParamHist = [];
hyperParamRangeHist = [];

% Metrics for convergence of iterations

thetaMaxDiffHist = []; % l_inf norm of difference of 2 consecutive estimates
thetaSumDiffHist = []; % l_1 norm of difference ...

% Metrics for accuracy of reconstruction. Note that depending on the method
% used, some of these might not be computed.

jointProbHist = []; % Joint probability p(y,theta)
jointProbReal = -1;

errorNorm1Hist = []; % l_1 norm of difference
errorNorm2Hist = []; % l_2 norm ...
costFuncHist = []; 

emCounter = 0; % Loop counter
stoppingCondWin = sRestoreOptions.STOPPING_COND_WIN;
bStopIterations = 0;


tCpuTimeStart = cputime;

while 1

	if B_DISP_PROGRESS_MSGS, disp( 'Gathering progress stats' ), end;

    % Compute the l_1 and l_2 error norm
    thetaHatDelta = thetaHatPrev(:) - thetaTrue(:);
    errorNorm1Hist = [ errorNorm1Hist, norm( thetaHatDelta, 1 ) ];
    errorNorm2Hist = [ errorNorm2Hist, norm( thetaHatDelta, 2 ) ];

	% Compute the marginalized log probability, log p(y,theta)    
	if exist( 'hParamEst', 'var' )  
        
       	bComputeLogPtheta = 1; % Assume that we will compute the joint prob.

       	switch szMSelector
           	case { 'eb_better', 'map_prior', 'eb_better_nov05' }
                logPcond = logp_ytheta_cond_gauss( imgNoisy, G, thetaHatPrev, noiseSigma );
               	logPtheta = logp_hyper_john_unif( thetaHatPrev, hParamEst.aRange );
                logPjoint = logPcond + logPtheta;
                
			case { 'binaryd', 'binaryd_map' }
                logPcond = logp_ytheta_cond_gauss( imgNoisy, G, thetaHatPrev, noiseSigma );
               	logPtheta = logp_hyper_binaryd_unif( thetaHatPrev, hParamEst.thetaStarRange );
                logPjoint = logPcond + logPtheta;
                
           	case { 'eb_fix_w', 'eb_fix_w_nov05' }
                logPcond = logp_ytheta_cond_gauss( imgNoisy, G, thetaHatPrev, noiseSigma );
               	logPtheta = logp_hyper_laplace_unif( thetaHatPrev, hParamEst.aRange );
                logPjoint = logPcond + logPtheta;
                
            otherwise
                % The default action is to not compute the marginalized
                % probability
               	bComputeLogPtheta = 0;
        end
        
        if bComputeLogPtheta
            jointProbHist = [jointProbHist, logPjoint ];
        end

	end

    % Compute the cost function    
    bComputeCostFunc = 1;
    
    switch ( szMSelector )
        case 'soft_fixed'
            logPcond = logp_ytheta_cond_gauss( imgNoisy, G, thetaHatPrev, noiseSigma );
            cost = -2*noiseSigma^2 * logPcond + 2*sRestoreOptions.SOFT_THRES_T*norm(thetaHatPrev(:),1);
            
        case 'daub1'
            residual = imgNoisy - G * thetaHatPrev;
            cost = norm( residual(:), 2 )^2 + ...
                sRestoreOptions.lambda/sRestoreOptions.daub1NormConst*norm( thetaHatPrev(:), 1 );
            
        case 'daub2'
            residual = imgNoisy - G * thetaHatPrev;
            cost = norm( residual(:), 2 )^2 + sRestoreOptions.lambda*norm( thetaHatPrev(:), 1 );            
            
        otherwise
            % The default action is to not compute the cost function
            bComputeCostFunc = 0;
    end
            
    if bComputeCostFunc
        costFuncHist = [costFuncHist, cost];
    end

    % Save hyperparameter estimates and their initial range
    if exist( 'hParamEst', 'var' )
        
        switch ( szMSelector )
            case 'map_prior'
                % hyperparameter = (a, t, w)
                try
                    hyperParam = [ hParamEst.a, hParamEst.t, hParamEst. w ];
                    hyperParamRange = [ hParamEst.aRange(:)', hParamEst.tRange(:)' ];
                catch
                    error( 'map_prior: cannot save hyperparams' );
                end

            case 'eb_better'
                % hyperparameter = ( x(1) -> a, x(2) -> t, w )
                try
                    hyperParam = [ hParamEst.x(1), hParamEst.x(2), ...
                        convert_tsigma2w( hParamEst.x(1), hParamEst.x(2), alpha, 'median' ) ];
                    hyperParamRange = [ hParamEst.aRange(:)', hParamEst.tRange(:)' ];
                catch
                    error( 'eb_better: cannot save hyperparams' );
                end

            case 'eb_better_nov05'
                % hyperparameter = ( voptim )
                try
                    hyperParam = hParamEst.voptim(:)';
                    hyperParamRange = [ hParamEst.aRange(:)', hParamEst.tRange(:)' ];
                catch
                    error( 'eb_better_nov05: cannot save hyperparams' );
                end
            
			case 'eb_complete'
				% hyperparameter = ( voptim )
				try
					hyperParam = hParamEst.voptim(:)';
					hyperParamRange = hParamEst.hyperParamRange(:)';
				catch
					error( 'eb_complete: cannot save hyperparams' );
				end
				 
            case 'eb_fix_w'
                % hyperparameter = ( x -> a, t, w )
                try
                    hyperParam = [ hParamEst.x, ...
                        convert_wsigma2t( hParamEst.x, hParamEst.w, alpha, 'median' ), ...
                        hParamEst.w ];
                    hyperParamRange = hParamEst.aRange(:)';
                catch
                    error( 'eb_fix_w: cannot save hyperparams' );
                end

            case 'eb_fix_w_nov05'
                % hyperparameter = ( voptim, t, w )
                try
                    hyperParam = [hParamEst.voptim, ...
                        convert_wsigma2t( hParamEst.voptim, hParamEst.w, alpha, 'median' ), ...
                        hParamEst.w ];
                    hyperParamRange = hParamEst.aRange(:)';
                catch
                    error( 'eb_fix_w_nov05: cannot save hyperparams' );
                end
                
            case { 'binaryd_wfree', 'binaryd', 'binaryd_map' }
                % hyperparameter = ( thetaStar, w )
                try
                    hyperParam = [ hParamEst.thetaStar, hParamEst.w ];
                    hyperParamRange = [];
                catch
                    error( 'binaryd*: cannot save hyperparams' );
                end
                
            case { 'exp1', 'exp2' }
                % hyperparameter = ( lambda, sigma )
                try
                    hyperParam = hParamEst.voptim(:)';
                    hyperParamRange = [ hParamEst.vLowerBound(:)', hParamEst.vUpperBound(:)' ];
                catch
                    error( 'exp1/2: cannot save hyperparams' );
                end              

			case { 'map1', 'map2' }
				% hyperparameter = ( a, w )
				try
					hyperParam = [ hParamEst.a, hParamEst.w, hParamEst.thres ];
					hyperParamRange = [];
				catch
					error( 'map*: cannot save hyperparams' );
				end

            otherwise
                % Default action: do not save any hyperparameter estimates
                hyperParam = [];
                hyperParamRange = [];
        end
      
        % If hyperParam and hyperParamRange are non-empty, save them in the
        % history
        if ~isempty( hyperParam )
            hyperParamHist = [hyperParamHist; hyperParam];
        end
        
        if ~isempty( hyperParamRange )
            hyperParamRangeHist = [hyperParamRangeHist; hyperParamRange];
        end
        
    end

    % Increment loop counter
    emCounter = emCounter + 1;
    if ( mod( emCounter, DISP_UPDATE_FREQ ) == 0 )
        disp( sprintf( 'EM iter %g', emCounter ) );
    end
 
	% For the methods where the hyperparameter estimation is independent of
	% the EM iterations. This should ONLY be called for map1 and map2.
    
	if sRestoreOptions.bSeparateHparamEst
        if ( sRestoreOptions.nHparamRestInterval <= 0 ) | ...
    		 ( ( sRestoreOptions.nHparamRestInterval > 0 ) & ...
             ( mod( emCounter, sRestoreOptions.nHparamRestInterval ) == 1 ) )
         
             try
                 hParamEst = hparam_est( mData, szMSelector );
             catch
                 error( sprintf( 'Caught error [%s] in hparam_est, aborting!', lasterr ) );
             end
             
             % Check return code in hParamEst. If 0, then estimation of the
             % hyperparameters was ok. Otherwise, stop iterations.
             
             if hParamEst.nRc < 0
                 warning( 'hParamEst.nRc < 0, stopping iterations' );
                 bStopIterations = 1;
             end
                         
		end
    end
    
    if bStopIterations, break, end
    
    %
    % E step
    %

	if B_DISP_PROGRESS_MSGS, disp( 'Doing E-step' ), end;

    % Faster way -- directly code here
    zHatNext = thetaHatPrev + sRestoreOptions.tau * ( G' * ( imgNoisy - G*thetaHatPrev ) );    
	
	if 0, % DEBUG: Quit after the first E-step and return the first zHatNext
		sRestore = struct( 'zHatNext', zHatNext );
		warning( 'DEBUG...quitting after first E-step' );
		return;
	end
	
	if 0, % DEBUG: plot time series and histogram of z values
		figure( 1 ), clf, hist( zHatNext(:), 50 );
		figure( 2 ), clf, plot( zHatNext(:), 'mx' );
		disp( 'Press key to continue' ), pause;
    end

    %
    % M step
    %

	if B_DISP_PROGRESS_MSGS, disp( 'Doing M-step' ), end;

    sRestoreOptions.nEmIter = emCounter;
    
    if exist( 'hParamEst', 'var' ), hParamEstPrev = hParamEst;
    else, hParamEstPrev = [];
    end
    
    [mData, hParamEst] = em_restore_m( zHatNext, lower(szMSelector), hParamEstPrev );
    
    switch ( szMSelector )
        case 'map1'
            estI = mData( :, 1 );
            estThetaTilde = mData( :, 2 );
            thetaHatNext = estI .* estThetaTilde;
            thetaHatNext = reshape( thetaHatNext, size( zHatNext ) );
        otherwise
            thetaHatNext = reshape( mData, size( zHatNext ) );
    end

	if B_DISP_PROGRESS_MSGS, disp( 'Post M-step' ), end;
	   
    % Store the difference in the estimates between iterations. The
    % stopping condition is based on how "small" the difference between
    % consecutive reconstructions are.    
    thetaHatDiff = thetaHatNext(:) - thetaHatPrev(:);    
    % The l_infinity norm of the difference
    thetaMaxDiffHist = [thetaMaxDiffHist; max( abs( thetaHatDiff ) )];
    % The l_1 norm of the difference
    thetaSumDiffHist = [thetaSumDiffHist; sum( abs( thetaHatDiff ) )];

    % Check stopping condition
    
    switch ( sRestoreOptions.STOPPING_COND )
        case 'fixed'
            % Perform a fixed number of iterations
            if ( emCounter >= sRestoreOptions.NUM_EM_ITERS )
                bStopIterations = 1;
                warning( 'stopping cond=fixed, exceeded NUM_EM_ITERS' );
            end
                        
        case 'conv_win'
            % Find the max value in the last STOPPING_COND_WINDOW iterations, 
            % and if it is less than a certain tolerance, quit.
            histLen = length( thetaMaxDiffHist );           
            if ( histLen >= stoppingCondWin )
                maxChange = max( thetaMaxDiffHist( max(1,histLen-stoppingCondWin): histLen ) );
                if ( maxChange < sRestoreOptions.STOPPING_COND_TOL )
					bStopIterations = 1;
                else
                    disp( sprintf( 'stop cond: maxChange=%g, tol=%g', ...
                        maxChange, sRestoreOptions.STOPPING_COND_TOL ) );
				end
            end
                                
            % Never exceed NUM_EM_ITERS
            if ( emCounter > sRestoreOptions.NUM_EM_ITERS )
                warning( 'stopping cond=conv_win, but exceeded NUM_EM_ITERS' );
                bStopIterations = 1;
            end
            
        case 'conv_last'
            % Find the last time T that we saw a value exceed
            % STOPPING_COND_TOL. We don't want to see another value exceed
            % STOPPING_COND_TOL for the next STOPPING_COND_LAST_FAC*T samples. 
			% More rigorous stopping condition than conv_win.
            indLastViolation = ...
				find( thetaMaxDiffHist > sRestoreOptions.STOPPING_COND_TOL, 1, 'last' );
            if isempty( indLastViolation )
                indLastViolation = -1;
            end
            
            stopSpacing = emCounter - indLastViolation + 1;
            stopThres = sRestoreOptions.STOPPING_COND_LAST_FAC * indLastViolation;
            if ( stopSpacing > stopThres )
                bStopIterations = 1;
            else
                disp( sprintf( 'stop cond: spacing=%g, thres=%g', ...
                    stopSpacing, stopThres ) );
            end
            
            % Never exceed NUM_EM_ITERS
            if ( emCounter > sRestoreOptions.NUM_EM_ITERS )
                warning( 'stopping cond=conv_last, but exceeded NUM_EM_ITERS' );
                bStopIterations = 1;
            end
                
        otherwise
            error( 'unrecognized stopping condition' );
    end    

    % Update state variable
    thetaHatPrev = thetaHatNext;
    
end % while

tCpuTimeStop = cputime;

% Compute the ACTUAL joint probability of y and theta, i.e. use
% thetaTrue. XXX: assume that we are using the Gblur method
logPcond = logp_ytheta_cond_gauss( imgNoisy, G, thetaHatPrev, noiseSigma );
bComputeLogPtheta = 1;
 
switch ( szMSelector )
   	case { 'eb_better', 'map_prior', 'eb_better_nov05' }
       	logPtheta = logp_hyper_john_unif( thetaTrue, hParamEst.aRange );        
   	case { 'binaryd', 'binaryd_map' }
       	logPtheta = logp_hyper_binaryd_unif( thetaHatNext, ...
           	hParamEst.thetaStarRange );
	case { 'eb_fix_w', 'eb_fix_w_nov05' }
		logPtheta = logp_hyper_laplace_unif( thetaHatNext, ...
			hParamEst.aRange );
    otherwise
        % Default action: do not compute the actual joint prob of y and
        % theta.
       	bComputeLogPtheta = 0;
end

if bComputeLogPtheta, jointProbReal = logPcond + logPtheta; end;

% Re-normalize the output for daub1 method. XXX: Note that we are faced
% with the inconsistency that thetaHat0 (the init element in sRestoreElems)
% is the NORMALIZED version whereas thetaHatPrev (the last element) is the
% UN-NORMALIZED version. Caveat emptor!

if strcmp( szMSelector, 'daub1' ) == 1 | ...
   strcmp( szMSelector, 'daub_hard_soft' ) == 1
    thetaHatPrev = thetaHatPrev / sRestoreOptions.daub1NormConst;
end

% Defines the structure of sRestore
sRestoreElems = { 'last', 'thetaHatPrev'; ...
    'init', 'thetaHat0'; ...
    'hyperParamHist', 'hyperParamHist'; ...
    'hyperParamRangeHist', 'hyperParamRangeHist'; ...  
    'thetaMaxDiff', 'thetaMaxDiffHist'; ...
    'thetaSumDiff', 'thetaSumDiffHist'; ...
    'errorNorm1', 'errorNorm1Hist'; ...
    'errorNorm2', 'errorNorm2Hist'; ...
    'jointProb', 'jointProbHist'; ...
    'jointProbReal', 'jointProbReal'; ...
    'cost', 'costFuncHist'; ...
    'hProp', 'sResH'; ...
    'cputime', '[tCpuTimeStart, tCpuTimeStop]' ...
    };

for elemInd = 1:size( sRestoreElems, 1 )
    szCmd = sprintf( 'sRestore.%s = %s;', ...
        char( sRestoreElems(elemInd,1) ), ...       
        char( sRestoreElems(elemInd,2) ) );
    try
        eval( szCmd );
    catch
        warning( sprintf( 'Could not execute: %s', szCmd ) );
    end
end

sRestoreOptions.sBoundingBox = sBoundingBox;
sRestoreOptions.bCloseBoundingBox = bCloseBoundingBox;



