% We assume that H is approximately circulant, so can work with DFTs, i.e.
% H = Q Gamma Q', where Gamma = diagonal matrix with DFT coefficients 
% ordered in a lexicographic fashion.
%
% Inputs: hForwardOpDft, noiseSigma, bPlot
% Outputs: sRes

function sRes = emsep_comp_alpha( hForwardOpDft, noiseSigma, bPlot )

n = length( hForwardOpDft(:) );

if ( ( nargin == 3 ) & ( bPlot == 1 ) ), B_PLOT = 1;
else, B_PLOT = 0;
end

nWin = 1;

% Gamma' * Gamma = Gamma * Gamma', as Gamma is diagonal. Note that for H
% being a convolution matrix, H is normal, i.e. HH' = H'H

GHG = conj( hForwardOpDft ) .* hForwardOpDft;

% Compute the condition number of H

% This only works with Matlab 7
%singularValues = sqrt( sort( GHG(:), 'descend' ) );

GHGascend = sort( GHG(:) ); % Default sort is in ascending order
singularValues = sqrt( GHGascend(end:-1:1) ); % Reverse vector

if true % This only works with Matlab 7
    indLastNz = find( singularValues > eps, 1, 'last' );
else
    nonZeroInd = find( singularValues > eps );
    indLastNz = nonZeroInd(end);
end

if B_PLOT
    figure( nWin ), clf, nWin = nWin + 1;
    semilogy( singularValues, 'bx' );
    ylabel( 'singular values of H' );
end
    
% Compute the upper bound for alpha

singularValueMax = singularValues( 1 );
singularValueNzMin = singularValues( indLastNz );
singularValueMin = singularValues( end );

disp( 'singular values of H' );
disp( sprintf( 'max: %g, non-zero min: %g, min: %g', ...
    singularValueMax, singularValueNzMin, singularValueMin ) );

alphaUb = noiseSigma / singularValueMax;

sRes = struct( 'singularValues', singularValues, ...
    'alphaUpperBound', alphaUb, ...   
    'cond', singularValueMax/singularValueMin, ...
    'norm2', singularValueMax );



