
%
% Threshold function for exponential prior of the form:
%    p(x) = lambda * exp( -lambda*x ) * u( x )
%

function est = exp_thres( y, lambda, noiseSigma )

yTilde = y / noiseSigma;
lambdaTilde = lambda * noiseSigma;
tmp1 = 1 - 0.5*normcdf( yTilde - lambdaTilde, 0, 1 );
tmp2 = norminv( tmp1, 0, 1 );
est = noiseSigma * ( yTilde - lambdaTilde + tmp2 );

% Check that the estimates are non-negative. Due to numerical computation, we 
% could have est=inf for some values.

global obj

n = length( est );
numZeroEst = 0;

for i = 1:n
	if ( ~isfinite( est(i) ) | ( est(i) < 0 ) )
		% Re-compute values of est
		xi = -( y(i)/noiseSigma - lambdaTilde );
		% We expect xi to be a large positive value
		if ( xi < 3 ), error( 'unxpected value of xi' ), end;
		obj = (1/2) * gaussianTail( xi );
		disp( sprintf( 'xi=%g, obj=%g', xi, obj ) );
		options = optimset( 'TolX', 1e-3*xi, 'TolFun', 1e-6*obj );
		[x, fval] = fsolve( @gaussianTailObj, xi, options );
		if ( isfinite( x ) & ( x > xi ) )
			est(i) = x - xi;
		else
			% Comment this out
			%disp( sprintf( 'obtained x=%g, fsolve failed', x ) );
			%disp( 'setting est(i)=0' );
			est(i) = 0;
			numZeroEst = numZeroEst + 1;
		end
	end
end

if numZeroEst > 0
	disp( sprintf( 'exp_thres: forced est(i)=0 %g times', numZeroEst ) );
end

% g = 1 - Phi( x ) for x large
function g = gaussianTail( x )
g = 1./( (x.^2 + 2) * sqrt(2*pi) ) .* x .* exp( -x.^2/2 ) .* ...
	( x.^6 + 6*x.^4 + 14*x.^2 - 28 ) ./ ...
	( x.^6 + 5*x.^4 - 20*x.^2 - 4 );

% g = 1 - Phi( x ) - obj
function g = gaussianTailObj( x )
global obj
g = gaussianTail( x ) - obj;
