% gen_laplace_data: Generates data from a Laplacian prior
%
% Usage:
%    [fClean, fNoisy] = gen_laplace_data( n, a, w, sigma )
%
function [fClean, fNoisy] = gen_laplace_data( n, a, w, sigma )

fClean = zeros( n, 1 );

nnz = round( n*w );
for i = 1:nnz
    fClean(i) = laplacian_transform( rand, a );
end

fClean = fClean( randperm(n) );
noise = randn( size( fClean ) ) * sigma;
fNoisy = fClean + noise;

function x = laplacian_transform( y, a )
if ( ( y > 0 ) & ( y <= 1/2 ) )
	x = log( 2*y ) / a;
elseif ( ( y > 1/2 ) & ( y < 1 ) )
	x = (-1/a) * log( 2*(1-y) );
else
	error( 'illegal value of y' );
end

