% hparam_est

function hParamEst = hparam_est( mData, szMethod )
%#################################################

switch lower( szMethod )
	case 'map1'
		% Use the iid prior for I, \tilde{theta}
        M = size( mData, 1 );
        if size( mData, 2 ) ~= 2, error( 'unexpected mData for map1 in hparam_est' ), end;
        
        estI = mData( :, 1 );
        estThetaTilde = mData( :, 2 );
        estI0 = sum( abs( estI ) > eps );
        estThetaTilde1 = sum( abs( estThetaTilde ) );
        % Sanity on estThetaTilde1, since we are dividing by it
        if ( estThetaTilde1 < eps )
            warning( sprintf( 'estThetaTilde1=%g for map1!', estThetaTilde1 ) );
            hParamEst = struct( 'nRc', -1 );
            return;
        end
        
        aHat = M / estThetaTilde1;
        wHat = estI0 / M;        
		hParamEst = struct( 'a', aHat, 'w', wHat, 'nRc', 0 );
		fprintf( '\thparam_est (map1)\n\testI0=%g, estThetaTilde1=%g, M=%g => a=%g, w=%g\n', ...
            estI0, estThetaTilde1, M, aHat, wHat );
		
	case 'map2'
		% Use the non-iid prior for I, \tilde{theta}. Compare with
		% estimation equations for map1.
		M = size( mData, 1 );
        if size( mData, 2 ) ~= 1, error( 'unexpected mData for map2 in hparam_est' ), end;
        
		thetaHat0 = sum( abs( mData ) > eps );
		thetaHat1 = sum( abs( mData ) );        
        % Sanity on thetaHat1, since we are dividing by it
        if ( thetaHat1 < eps )
            warning( sprintf( 'thetaHat1=%g for map2!', thetaHat1 ) );
            hParamEst = struct( 'nRc', -1 );
            return;
        end
        
        aHat = thetaHat0 / thetaHat1;
		wHat = thetaHat0 / M;		
		hParamEst = struct( 'a', aHat, 'w', wHat, 'nRc', 0 );
		fprintf( '\thparam_est (map2)\n\tthetaHat0=%g, thetaHat1=%g, M=%g => a=%g, w=%g\n', ...
                thetaHat0, thetaHat1, M, aHat, wHat );

    otherwise
        % Error: should not be called
        error( sprintf( 'method %s should not call hparam_est!', szMethod ) );
end