% im2BlockCirc: converts an impulse response h[n,m] to a block-circulant matrix
%-------------------------------------------------------------------------------
% Given a 2-d impulse response h[n,m], convert it to a block-circulant matrix
% so that g = h * f can be written in matrix-vector form as y = Ax.
%
% NOTE: it is impractical to use this approach for even decently sized
% h[n,m].
%
% A = im2BlockCirc( h )
%
function A = im2BlockCirc( h )
[N, M] = size( h );
ASize = N*M;
A = zeros( ASize, ASize );

fnN = inline( 'mod(i-1,N)', 'i', 'N' );
fnM = inline( 'floor( (i-1)/N )', 'i', 'N' );

for i = 1:ASize
	for j = 1:ASize
		nIndex = mod( fnN(i,N) - fnN(j,N), N );
		mIndex = mod( fnM(i,N) - fnM(j,N), M );
		% DEBUG
		%disp( sprintf( 'nIndex=%g, mIndex=%g', nIndex, mIndex ) );
		A( i, j ) = h( 1 + nIndex, 1+ mIndex );
	end
end
