
% l1_restore: Use ASPIRE to perform l1 restoration. Major portions copied
% from em_restore.m.

if ~exist( 'sRestoreOptions', 'var' )
    error( 'missing variable sRestoreOptions' );
end

if ~exist( 'szInitialSelector', 'var' ) | ~exist( 'yi', 'var' )
    error( 'missing variable szInitialSelector or yi' );
end

% Initialize
imgNoisy = yi;
hForwardOpDft = fftn( h );
noiseSigma = estd;

switch ( lower(szInitialSelector) )
case 'zero'
	thetaHat0 = zeros( size( imgNoisy ) );
case 'obs'
	thetaHat0 = imgNoisy;
otherwise
	error( 'invalid szInitialSelector' );
end

thetaHatPrev = thetaHat0;
thetaMaxDiffHist = [];
thetaSumDiffHist = [];

f.l2b_n = 0;
f.niter = sRestoreOptions.NUM_EM_ITERS;
rx_delta = 1e-2;

logPcond = logp_ytheta_cond_gauss( imgNoisy, thetaHat0, hForwardOpDft, ...
    noiseSigma );
cost = -2*noiseSigma^2 * logPcond + ...
    (2/sqrt(3)) * (2^f.l2b_n) * rx_delta * norm(thetaHat0(:),1);
costFuncHist = [cost];

close all;

Rn = Robject(mask, 'type_denom', 'matlab', ...
    'offsets', 0, ... % trick for identity
    'potential', 'hyper3', 'beta', 2^f.l2b_n, 'delta', rx_delta);
xnpls = pwls_sps_os(thetaHat0(:), imgNoisy(:), [], G, Rn, ...
    f.niter, [-inf, inf], [], [], 1);
xnpls = embed(xnpls, mask);
%disp( 'size of xnpls:' ), size( xnpls )

for i = 1:sRestoreOptions.NUM_EM_ITERS
        
    if ( i > 1 )
        thetaHatNext = xnpls( :, :, i );
        thetaHatPrev = xnpls( :, :, i-1 );

    else
        % i == 1
        thetaHatNext = xnpls( :, :, i );
        thetaHatPrev = thetaHat0;
    end

    % Form the difference between two consecutive reconstructions
    thetaHatDiff = thetaHatNext - thetaHatPrev;
    % The l_infinity norm of the difference
    thetaMaxDiffHist = [thetaMaxDiffHist, max( abs( thetaHatDiff(:) ) )];
    % The l_1 norm of the difference
    thetaSumDiffHist = [thetaSumDiffHist, sum( abs( thetaHatDiff(:) ) )];
    
    logPcond = logp_ytheta_cond_gauss( imgNoisy, thetaHatNext, hForwardOpDft, ...
        noiseSigma );
    cost = -2*noiseSigma^2 * logPcond + ...
        (2/sqrt(3)) * (2^f.l2b_n) * (1e-2) * norm(thetaHatNext(:),1);
    costFuncHist = [costFuncHist, cost];
end
            
% Figure out which is the "last" iteration according to the same
% criterion that we use in em_restore.
switch ( lower(sRestoreOptions.STOPPING_COND) )
    case 'fixed'
        lastImage = xnpls( :, :, end );
    case 'conv'
        % Find the max value in the last 50 iterations, and if it is
        % less than a certain tolerance, quit.
        histLen = length( thetaMaxDiffHist );
        if ( histLen ~= sRestoreOptions.NUM_EM_ITERS )
            error( 'histLen not equal to NUM_EM_ITERS!' );
        end
        lastImage = xnpls( :, :, end ); % Initialize
        bConvReached = 0;

        for histInd = 51:histLen
            maxChange = max( thetaMaxDiffHist( max(1,histInd-50): histInd ) );
            if ( maxChange < sRestoreOptions.STOPPING_COND_TOL )
                disp( sprintf( 'Convergence reached at histInd=%g, max(l_inf)=%g', ...
                    histInd, maxChange ) );
                lastImage = xnpls( :, :, histInd );
                bConvReached = 1;
                break;
            end
        end

        if ( ~bConvReached )
            warning( 'stopping cond=conv, exceeded NUM_EM_ITERS' );
        end
    otherwise
        error( 'unrecognized stopping condition' );
end

if 1 % Make plot
    figure( 1 ), semilogy( thetaMaxDiffHist, 'bx' ), title( 'l_{\infty} norm' );
    figure( 2 ), semilogy( thetaSumDiffHist, 'r^' ), title( 'l_1 norm' );
    figure( 3 ), imagesc( lastImage ), colormap gray, cbar;
end

reconElems = { 'last', 'lastImage'; ...
    'init', 'thetaHat0'; ...
    'thetaMaxDiff', 'thetaMaxDiffHist'; ...
    'thetaSumDiff', 'thetaSumDiffHist'; ...
    'cost', 'costFuncHist' };

clear sRecon;

for i = 1:size( reconElems, 1 )
    szCmd = sprintf( 'sRecon.%s = %s;', char( reconElems(i,1) ), ...       
        char( reconElems(i,2) ) );
    try
        eval( szCmd );
    catch
        warning( sprintf( 'Could not execute: %s', szCmd ) );
    end
end

%..........................................................................
%                                OBSOLETE
%##########################################################################

%i = 1; % Loop counter

while 0 % Hmm, for some reason, this doesn't work very well. It could be
        % that the pwls_sps_os function requires more than one iteration
        % to properly initialize. -- MT
    
    switch ( lower(sRestoreOptions.STOPPING_COND) )
        case 'fixed'
            % Perform a fixed number of iterations
            if ( i > sRestoreOptions.NUM_EM_ITERS ), break, end
        case 'conv'
            % Find the max value in the last 50 iterations, and if it is
            % less than a certain tolerance, quit.
            histLen = length( thetaMaxDiffHist );
            if ( histLen >= 1 )
                maxChange = max( thetaMaxDiffHist( max(1,histLen-50): histLen ) );
                if ( maxChange < sRestoreOptions.STOPPING_COND_TOL ), break, end
            end
                                
            if ( i > sRestoreOptions.NUM_EM_ITERS )
                warning( 'stopping cond=conv, exceeded NUM_EM_ITERS' );
                break;
            end
        otherwise
            error( 'unrecognized stopping condition' );
    end
    
    if 0 % DEBUG
        disp( sprintf( 'i=%g', i ) );
    end
    
    % Straight copy from restore_sparse.m with the relevant variables
    % substituted out
    
    f.l2b_n = 2;
    f.niter = 1;
    Rn = Robject(mask, 'type_denom', 'matlab', ...
        'offsets', 0, ... % trick for identity
        'potential', 'hyper3', 'beta', 2^f.l2b_n, 'delta', 0.01);
    xnpls = pwls_sps_os(thetaHatPrev(:), yi(:), [], G, Rn, ...
        f.niter, inf, [], [], 1);
    xnpls = embed(xnpls, mask);
    %thetaHatNext = max(xnpls(:,:,end), 0);
    thetaHatNext = xnpls( :, :, end );  
    
    % Store the difference in the estimates between iterations
    thetaHatDiff = thetaHatNext(:) - thetaHatPrev(:);
    % The l_infinity norm of the difference
    thetaMaxDiffHist = [thetaMaxDiffHist; max( abs( thetaHatDiff ) )];
    % The l_1 norm of the difference
    thetaSumDiffHist = [thetaSumDiffHist; sum( abs( thetaHatDiff ) )];

	% Update thetaHat
	thetaHatPrev = thetaHatNext;

    % Increment loop counter
    i = i + 1;
    
end % while