% laplace_map_thres0: Implements the MAP thresholding function using
% (a, w, sigma).
%
% Usage:
%    est = laplace_map_thres0( y, a, w, sigma )
%

function est = laplace_map_thres0( y, a, w, sigma )

if ( w <= 0.5 )
	t = a*sigma^2 + sqrt( 2*sigma^2*log( (1-w)/w ) );
else
	% w > 0.5
	t = a*sigma^2;
end

% Sanity: check that t > 0
if ( t <= 0 ), error( 'negative threshold in laplace_pseudo_map' ), end;

yMinus = y - a*sigma^2;
yPlus = y + a*sigma^2;
est = yMinus .* ( y > t ) + yPlus .* ( y < -t );
