% laplace_map_thres1: Implements the MAP thresholding function using
% (tMedian, w, sigma ). NB. tMedian is an arbitrary threshold.
%
% Usage:
%    est = laplace_map_thres1( y, a, tMedian, w, sigma )
%
function est = laplace_map_thres1( y, a, tMedian, w, sigma )

if ( w <= 0.5 )
	tMap = a*sigma^2 + sqrt( 2*sigma^2*log( (1-w)/w ) );
else
	% w > 0.5
	tMap = a*sigma^2;
end

disp( sprintf( 'tMedian = %g, tMap = %g', tMedian, tMap ) );

% Sanity: check that tMedian > 0 and tMap > 0
if ( ( tMedian <= 0 ) | ( tMap <= 0 ) )
	error( 'negative threshold in laplace_pseudo_map' )
end

if ( w <= 0.5 )
	% Check that w is lower bounded if w <= 0.5
	wLowerBound = 1/( 1 + exp( 0.5*(tMedian/sigma)^2 ) );
	if ( w > wLowerBound )
		% Let alpha = a*sigma^2
		alpha = tMedian - sigma * sqrt( 2*log( (1-w)/w ) );
		if ( alpha <= 0 ), error( 'alpha < 0' ), end;
	else
		warning( 'w does not satisfy the lower bound' );
		% Set alpha = 0
		alpha = 0;
	end
else
	% w > 0.5
	alpha = tMedian;
end

yMinus = y - alpha;
yPlus = y + alpha;
est = yMinus .* ( y > tMedian ) + yPlus .* ( y < -tMedian );
