% laplace_map_thres2: Implements the MAP thresholding function using
% (tMedian, a, sigma). NB. tMedian is an arbitrary threshold.
%
% Usage:
%    est = laplace_map_thres2( y, a, tMedian, w, sigma )
%
function est = laplace_map_thres2( y, a, tMedian, w, sigma )

if ( w <= 0.5 )
	tMap = a*sigma^2 + sqrt( 2*sigma^2*log( (1-w)/w ) );
else
	% w > 0.5
	tMap = a*sigma^2;
end

disp( sprintf( 'tMedian = %g, tMap = %g', tMedian, tMap ) );

% Sanity: check that tMedian > 0 and tMap > 0
if ( ( tMedian <= 0 ) | ( tMap <= 0 ) )
	error( 'negative threshold in laplace_pseudo_map' )
end

alpha = a*sigma^2;

if ( alpha < tMedian )
	% Means that w <= 0.5, so we can solve for w
	wTilde = 1/( 1 + exp( 0.5*(tMedian/sigma - a*sigma)^2 ) );
	disp( sprintf( 'wML = %g vs. w(tMedian,a,sigma) = %g', w, wTilde ) );
else
	% Means that w > 0.5. Set alpha = tMedian
	disp( sprintf( 'Inconsistent: a*sigma^2 = %g, tMedian = %g', ...
		alpha, tMedian ) );
	alpha = tMedian;
end
	
yMinus = y - alpha;
yPlus = y + alpha;
est = yMinus .* ( y > tMedian ) + yPlus .* ( y < -tMedian );
