% laplace_posterior_med: Apply the median thresholding rule
%
% Usage:
%    est = laplace_posterior_med( y, a, aux, sdev, szAuxSel )
%
function est = laplace_posterior_med( y, a, aux, sdev, szAuxSel )

n = max( size( y ) );
est = zeros( size(y) );

AUX_T = 1;
AUX_W = 2;

if ( nargin < 5 )    
    nAuxSel = AUX_T;
else
    switch ( lower( szAuxSel ) )
        case 't'
            nAuxSel = AUX_T;
        case 'w'
            nAuxSel = AUX_W;
        otherwise
            error( 'invalid szAuxSel' );
    end
end
            

if ( nAuxSel == AUX_T )
    t = aux;
    w = convert_tsigma2w( a, t, sdev, 'median' );
    % DEBUG
    disp( sprintf( 'DEBUG: a=%g, t=%g, sdev=%g, w=%g', a, t, sdev, w ) );
elseif ( nAuxSel == AUX_W )
    w = aux;
    % Don't need threshold really
    %t = convert_wsigma2t( a, w, sdev, 'median' );
end

% Flip the sign of positive element in y. Thus, all the elements of yNeg 
% should be non-positive. Keep the elements that are negative unchanged.
yNeg = y .* -(y > 0) + y .* (y < 0);

% Vectorize
tmp = 1/(a*w)*exp( -a*yNeg-(a*sdev)^2/2 ).*fLaplacianGauss( yNeg, a, w, sdev );
s = yNeg + a*sdev^2 + sdev*norminv( tmp );

% Assemble back. indNeg contains values of s that are valid; otherwise, the
% thresholded value is zero. Refer back to the sign of y_i; if it was
% originally positive, the corresponding thresholded value should also be
% positive. So flip the sign of the thresholded value.
indNeg = find( s < 0 );
est(indNeg) = (y(indNeg) < 0) .* s(indNeg) + (y(indNeg) > 0) .* (-s(indNeg));

% XXX: obsolete
% for i = 1:n
% 	yy = y(i);
% 
% 	% If the observation is zero, because the posterior median function is 
% 	% anti-symmetric, the output is 0.
% 	if ( yy == 0 ), est(i) = 0; continue, end;
% 
% 	% Here deal with non-zero values of the observation
% 
% 	bPos = 0;
% 	if ( yy > 0 ), yy = -yy; bPos = 1; end;
% 
% 	tmp = 1/(a*w)*exp( -a*yy-(a*sdev)^2/2 )*fLaplacianGauss( yy, a, w, sdev );
% 	s = yy + a*sdev^2 + sdev*norminv( tmp );
% 
% 	% If s isn't negative, then the posterior median is 0
% 
% 	if ( s < 0 )
% 		if ( bPos == 0 ), est(i) = s; 
% 		else
% 			% Need to flip back sign
% 			est(i) = -s;
% 		end
% 	end
% end



