% neg_laplace_log_lik_atsigma
%
% Usage:
%    f = neg_laplace_log_lik_atsigma( params )
%
%    f - negative of log likelihood of laplacian prior + gaussian noise
%    params - vector containing [a,t,sdev]
%    
% Globals:
%    y - observations
%

function f = neg_laplace_log_lik_atsigma( params )

global y

a = params(1);
t = params(2);
sdev = params(3);

% Check param values
%if ( ( a <= 0 ) | ( t <= 0 ) | ( sdev <= 0 ) )
%	warning( 'parameter (a,t,sdev) not within legal range' );
%end

try
	w = convert_tsigma2w( a, t, sdev, 'median' );
	vecS = (1-w) * normpdf( y, 0, sdev ) + a*w/4*exp((a*sdev)^2/2) * ...
		( exp(a*y) .* (1 - erf( (y/sdev+a*sdev)/sqrt(2) )) + ...
	  	exp(-a*y) .* (1 + erf( (y/sdev-a*sdev)/sqrt(2) )) );
	f = sum( log( vecS ) );
	f = -f;
catch
	disp( sprintf( 'a: %g, t: %g, sdev: %g', a, t, sdev ) );
	warning( 'Press any key to continue...' );
	%pause;
	f = inf;
end


