% neg_laplace_log_lik_map_awsigma
%
% f = neg_laplaceE_log_lik_map_awsigma( params )
%

function f = neg_laplace_log_lik_map_awsigma( params )

global y bMapCmpUniv

a = params(1);
w = params(2);
sigma = params(3);

n = length( y );

% Screen out w not in [0,1], sigma < 0, a < 0
if ( ( w < 0 ) | ( w > 1 ) | ( sigma <= 0 ) | ( a <= 0 ) )
	f = inf; % Impossible
end

% Find the MAP threshold
thresMap = convert_wsigma2t( a, w, sigma, 'map' );
thresUniv = sqrt( 2*log(n) ) * sigma;

% Check to see if the MAP threshold exceeds the universal threshold?
%bCmpUniv = 1;
% Make this a global variable

if ( thresMap < 0 ) 
	
	% Reject this threshold
	f = inf;
	% DEBUG
	%disp( 'thresMap < 0' );
	%disp( sprintf( 'a: %g, w: %g, sigma: %g', a, w, sigma ) );
	
elseif ( ( bMapCmpUniv == 1 ) & ( thresMap > thresUniv ) )

	% Reject this threshold
	f = inf;
	% DEBUG
	%disp( 'thresMap > thresUniv' );
	%disp( sprintf( 'a: %g, w: %g, sigma: %g', a, w, sigma ) );

else

	% Compute the estimates of \tilde{x} and I
	if ( w < 0.5 )
		IHat = ( abs(y) >= thresMap ); 
	else
		IHat = ones( size(y) );
	end
	xTildeHat = ( y - sign(y)*a*sigma^2 ) .* ( abs(y) >= thresMap );

	xHat = IHat .* xTildeHat;
	tmp = sum( (xHat(:) - y(:)).^2 );

	f = n*( -log(sigma) + log(a) + log(1-w) ) + ...
		log( w/(1-w) )*sum( IHat ) - a * sum( abs( xTildeHat ) ) - ...
		tmp / (2*sigma^2);

	f = -f;
end
