% phi_deriv: Computes the gradient and Hessian of the marginalized log lik
%
% Usage:
%    [phiGrad, phiHess] = phi_deriv( y, a, t, sigma, szOption )
%
% OBSOLETE: do not use!
%

function [phiGrad, phiHess] = phi_deriv( y, a, t, sigma, szOption )

% Initialize
n = length( y(:) );
phiGrad = zeros( 2, 1 );
phiHess = zeros( 2, 2 );

gY = fn_g( y, a, t, sigma );
fY = f1( y, sigma );
betaY = gY ./ fY - 1;

for i=1:n
	dgdaY(i) = dg_da( y(i), a, t, sigma ); 
	dgda2Y(i) = dg_da2( y(i), a, t, sigma );
end

% Make everything column vectors
gY = gY(:);
fY = fY(:);
betaY = betaY(:);
dgdaY = dgdaY(:);
dgda2Y = dgda2Y(:);

w = fn_w( a, t, sigma );
dwda = dw_da( a, t, sigma );
dwdt = dw_dt( a, t, sigma );

switch ( lower( szOption ) )
case {'grad','gradhess'}
	% First find dphi_da
	tmpNumer = dwda * betaY + w * dgdaY .* (1./fY);
	tmpDenom = 1 + w*betaY;
	dphi_da = sum( tmpNumer ./ tmpDenom );

	% Next, find dphi_dt
	tmpNumer = betaY;
	dphi_dt = dwdt * sum( tmpNumer ./ tmpDenom );

	phiGrad = [ dphi_da, dphi_dt ]';

case 'gradhess'
	dwda2 = dw_da2( a, t, sigma );
	dwdadt = dw_dadt( a, t, sigma );
	dwdt2 = dw_dt2( a, t, sigma );

	% Find dphi_da2
	tmpNumer = (1 + w*betaY) .* ( dwda2 * betaY + ...
								  (2*dwda*dgdaY + w*dgda2Y) .* (1./fY) ) - ...
		( dwda * betaY + w* dgdaY .* (1./fY) ).^2;
	tmpDenom = (1 + w*betaY).^2;
	dphi_da2 = sum( tmpNumer ./ tmpDenom );

	% Find dphi_dadt
	tmpNumer = (1 + w*betaY) .* ( dwdadt*betaY + dwdt*dgdaY.*(1./fY) ) - ...
		( dwdt*betaY ) .* ( dwda*betaY + w*dgdaY.*(1./fY) );
	dphi_dadt = sum( tmpNumer ./ tmpDenom );

	% Find dphi_dt2
	s1 = sum( betaY ./ ( 1 + w*betaY ) );
	s2 = sum( betaY.^2 ./ (1 + w*betaY).^2 );
	dphi_dt2 = dwdt2 * s1 - (dwdt)^2 * s2;

	phiHess = [ dphi_da2, dphi_dadt; dphi_dadt, dphi_dt2 ];

end

% Derivatives
function f = dw_dt( a, t, sigma )
f = -2*2^(1/2)/pi^(1/2)/sigma^3*t*exp(-t^2/sigma^2)/(a*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))))-2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)/(a*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))))^2*(-a^2*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+a*exp(-a*t+1/2*a^2*sigma^2)/pi^(1/2)*exp(-1/2*(-t+a*sigma^2)^2/sigma^2)*2^(1/2)/sigma-2*2^(1/2)/pi^(1/2)/sigma^3*t*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(-a*exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(-a*t)/pi^(1/2)*exp(-1/2*(-t+a*sigma^2)^2/sigma^2)*2^(1/2)/sigma+a*exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))-exp(a*t)/pi^(1/2)*exp(-1/2*(-t-a*sigma^2)^2/sigma^2)*2^(1/2)/sigma));

function f = dw_da( a, t, sigma )
f = -2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)/(a*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))))^2*(exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+a*(-t+a*sigma^2)*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-a*exp(-a*t+1/2*a^2*sigma^2)/pi^(1/2)*exp(-1/2*(-t+a*sigma^2)^2/sigma^2)*sigma*2^(1/2)-1/2*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma)))-1/2*a^2*sigma^2*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma)))-1/2*a*exp(1/2*a^2*sigma^2)*(-t*exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-exp(-a*t)/pi^(1/2)*exp(-1/2*(-t+a*sigma^2)^2/sigma^2)*sigma*2^(1/2)+t*exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))-exp(a*t)/pi^(1/2)*exp(-1/2*(-t-a*sigma^2)^2/sigma^2)*sigma*2^(1/2)));

function f = dg_da( y, a, t, sigma )
f = 1/pi/sigma^2*exp(-t^2/sigma^2)/(a*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))))^2*(exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+a*(-t+a*sigma^2)*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-a*exp(-a*t+1/2*a^2*sigma^2)/pi^(1/2)*exp(-1/2*(-t+a*sigma^2)^2/sigma^2)*sigma*2^(1/2)-1/2*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma)))-1/2*a^2*sigma^2*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma)))-1/2*a*exp(1/2*a^2*sigma^2)*(-t*exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-exp(-a*t)/pi^(1/2)*exp(-1/2*(-t+a*sigma^2)^2/sigma^2)*sigma*2^(1/2)+t*exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))-exp(a*t)/pi^(1/2)*exp(-1/2*(-t-a*sigma^2)^2/sigma^2)*sigma*2^(1/2)))*exp(-y^2/sigma^2)+1/4*2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)/(a*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))))*exp(1/2*a^2*sigma^2)*(exp(-y*a)+exp(y*a)+exp(-y*a)*erf(1/2*(y-a*sigma^2)*2^(1/2)/sigma)-exp(y*a)*erf(1/2*(y+a*sigma^2)*2^(1/2)/sigma))-1/4*a*2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)/(a*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))))^2*exp(1/2*a^2*sigma^2)*(exp(-y*a)+exp(y*a)+exp(-y*a)*erf(1/2*(y-a*sigma^2)*2^(1/2)/sigma)-exp(y*a)*erf(1/2*(y+a*sigma^2)*2^(1/2)/sigma))*(exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+a*(-t+a*sigma^2)*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-a*exp(-a*t+1/2*a^2*sigma^2)/pi^(1/2)*exp(-1/2*(-t+a*sigma^2)^2/sigma^2)*sigma*2^(1/2)-1/2*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma)))-1/2*a^2*sigma^2*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma)))-1/2*a*exp(1/2*a^2*sigma^2)*(-t*exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-exp(-a*t)/pi^(1/2)*exp(-1/2*(-t+a*sigma^2)^2/sigma^2)*sigma*2^(1/2)+t*exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))-exp(a*t)/pi^(1/2)*exp(-1/2*(-t-a*sigma^2)^2/sigma^2)*sigma*2^(1/2)))+1/4*a^2*2^(1/2)/pi^(1/2)*sigma*exp(-t^2/sigma^2)/(a*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))))*exp(1/2*a^2*sigma^2)*(exp(-y*a)+exp(y*a)+exp(-y*a)*erf(1/2*(y-a*sigma^2)*2^(1/2)/sigma)-exp(y*a)*erf(1/2*(y+a*sigma^2)*2^(1/2)/sigma))+1/4*a*2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)/(a*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))))*exp(1/2*a^2*sigma^2)*(-y*exp(-y*a)+y*exp(y*a)-y*exp(-y*a)*erf(1/2*(y-a*sigma^2)*2^(1/2)/sigma)-exp(-y*a)/pi^(1/2)*exp(-1/2*(y-a*sigma^2)^2/sigma^2)*sigma*2^(1/2)-y*exp(y*a)*erf(1/2*(y+a*sigma^2)*2^(1/2)/sigma)-exp(y*a)/pi^(1/2)*exp(-1/2*(y+a*sigma^2)^2/sigma^2)*sigma*2^(1/2));

function f = dw_da2( a, t, sigma )
f = 2*2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)/(a*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))))^3*(exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+a*(-t+a*sigma^2)*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-a*exp(-a*t+1/2*a^2*sigma^2)/pi^(1/2)*exp(-1/2*(-t+a*sigma^2)^2/sigma^2)*sigma*2^(1/2)-1/2*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma)))-1/2*a^2*sigma^2*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma)))-1/2*a*exp(1/2*a^2*sigma^2)*(-t*exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-exp(-a*t)/pi^(1/2)*exp(-1/2*(-t+a*sigma^2)^2/sigma^2)*sigma*2^(1/2)+t*exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))-exp(a*t)/pi^(1/2)*exp(-1/2*(-t-a*sigma^2)^2/sigma^2)*sigma*2^(1/2)))^2-2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)/(a*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))))^2*(2*(-t+a*sigma^2)*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-2*exp(-a*t+1/2*a^2*sigma^2)/pi^(1/2)*exp(-1/2*(-t+a*sigma^2)^2/sigma^2)*sigma*2^(1/2)+a*sigma^2*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+a*(-t+a*sigma^2)^2*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-2*a*(-t+a*sigma^2)*exp(-a*t+1/2*a^2*sigma^2)/pi^(1/2)*exp(-1/2*(-t+a*sigma^2)^2/sigma^2)*sigma*2^(1/2)-a*exp(-a*t+1/2*a^2*sigma^2)/pi^(1/2)*(t-a*sigma^2)*exp(-1/2*(-t+a*sigma^2)^2/sigma^2)*sigma*2^(1/2)-3/2*a*sigma^2*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma)))-exp(1/2*a^2*sigma^2)*(-t*exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-exp(-a*t)/pi^(1/2)*exp(-1/2*(-t+a*sigma^2)^2/sigma^2)*sigma*2^(1/2)+t*exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))-exp(a*t)/pi^(1/2)*exp(-1/2*(-t-a*sigma^2)^2/sigma^2)*sigma*2^(1/2))-1/2*a^3*sigma^4*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma)))-a^2*sigma^2*exp(1/2*a^2*sigma^2)*(-t*exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-exp(-a*t)/pi^(1/2)*exp(-1/2*(-t+a*sigma^2)^2/sigma^2)*sigma*2^(1/2)+t*exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))-exp(a*t)/pi^(1/2)*exp(-1/2*(-t-a*sigma^2)^2/sigma^2)*sigma*2^(1/2))-1/2*a*exp(1/2*a^2*sigma^2)*(t^2*exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+2*t*exp(-a*t)/pi^(1/2)*exp(-1/2*(-t+a*sigma^2)^2/sigma^2)*sigma*2^(1/2)-exp(-a*t)/pi^(1/2)*(t-a*sigma^2)*exp(-1/2*(-t+a*sigma^2)^2/sigma^2)*sigma*2^(1/2)+t^2*exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))-2*t*exp(a*t)/pi^(1/2)*exp(-1/2*(-t-a*sigma^2)^2/sigma^2)*sigma*2^(1/2)-exp(a*t)/pi^(1/2)*(-t-a*sigma^2)*exp(-1/2*(-t-a*sigma^2)^2/sigma^2)*sigma*2^(1/2)));

function f = dg_da2( y, a, t, sigma )
f = -2*(exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+a*(-t+a*sigma^2)*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-sigma*a/pi^(1/2)*2^(1/2)*exp(-a*t+1/2*a^2*sigma^2-1/2*(-t+a*sigma^2)^2/sigma^2)+(-1/2*exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-1/2*exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma)))*exp(1/2*a^2*sigma^2)-1/2*a^2*sigma^2*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma)))+(1/2*t*exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+1/2*sigma/pi^(1/2)*2^(1/2)*exp(-a*t-1/2*(-t+a*sigma^2)^2/sigma^2)-1/2*t*exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))+1/2*sigma/pi^(1/2)*2^(1/2)*exp(a*t-1/2*(-t-a*sigma^2)^2/sigma^2))*a*exp(1/2*a^2*sigma^2))^2/(a*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))))^3/sigma^2/pi*exp(-t^2/sigma^2-y^2/sigma^2)+((2-2*erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))*(-t+a*sigma^2)*exp(-a*t+1/2*a^2*sigma^2)-2*sigma/pi^(1/2)*2^(1/2)*exp(-a*t+1/2*a^2*sigma^2-1/2*(-t+a*sigma^2)^2/sigma^2)+a*sigma^2*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+a*(-t+a*sigma^2)^2*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-2*sigma*a*(-t+a*sigma^2)/pi^(1/2)*2^(1/2)*exp(-a*t+1/2*a^2*sigma^2-1/2*(-t+a*sigma^2)^2/sigma^2)-sigma*a*(t-a*sigma^2)/pi^(1/2)*2^(1/2)*exp(-a*t+1/2*a^2*sigma^2-1/2*(-t+a*sigma^2)^2/sigma^2)-3/2*a*sigma^2*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma)))+(t*exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+sigma/pi^(1/2)*2^(1/2)*exp(-a*t-1/2*(-t+a*sigma^2)^2/sigma^2)-t*exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))+sigma/pi^(1/2)*2^(1/2)*exp(a*t-1/2*(-t-a*sigma^2)^2/sigma^2))*exp(1/2*a^2*sigma^2)-1/2*a^3*sigma^4*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma)))-a^2*sigma^2*exp(1/2*a^2*sigma^2)*(-t*exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-sigma/pi^(1/2)*2^(1/2)*exp(-a*t-1/2*(-t+a*sigma^2)^2/sigma^2)+t*exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))-sigma/pi^(1/2)*2^(1/2)*exp(a*t-1/2*(-t-a*sigma^2)^2/sigma^2))+(-1/2*t^2*exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-sigma*t/pi^(1/2)*2^(1/2)*exp(-a*t-1/2*(-t+a*sigma^2)^2/sigma^2)+1/2*sigma*(t-a*sigma^2)/pi^(1/2)*2^(1/2)*exp(-a*t-1/2*(-t+a*sigma^2)^2/sigma^2)-1/2*t^2*exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))+sigma*t/pi^(1/2)*2^(1/2)*exp(a*t-1/2*(-t-a*sigma^2)^2/sigma^2)+1/2*sigma*(-t-a*sigma^2)/pi^(1/2)*2^(1/2)*exp(a*t-1/2*(-t-a*sigma^2)^2/sigma^2))*a*exp(1/2*a^2*sigma^2))/(a*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))))^2/sigma^2/pi*exp(-t^2/sigma^2-y^2/sigma^2)+(-1/2*exp(-y*a)-1/2*exp(y*a)-1/2*exp(-y*a)*erf(1/2*(y-a*sigma^2)*2^(1/2)/sigma)+1/2*exp(y*a)*erf((1/2*y+1/2*a*sigma^2)/sigma*2^(1/2)))/(a*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))))^2/sigma*(exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+a*(-t+a*sigma^2)*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-sigma*a/pi^(1/2)*2^(1/2)*exp(-a*t+1/2*a^2*sigma^2-1/2*(-t+a*sigma^2)^2/sigma^2)+(-1/2*exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-1/2*exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma)))*exp(1/2*a^2*sigma^2)-1/2*a^2*sigma^2*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma)))-1/2*a*exp(1/2*a^2*sigma^2)*(-t*exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-sigma/pi^(1/2)*2^(1/2)*exp(-a*t-1/2*(-t+a*sigma^2)^2/sigma^2)+t*exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))-sigma/pi^(1/2)*2^(1/2)*exp(a*t-1/2*(-t-a*sigma^2)^2/sigma^2)))/pi^(1/2)*2^(1/2)*exp(-t^2/sigma^2+1/2*a^2*sigma^2)+(3/4*exp(-y*a)+3/4*exp(y*a)+3/4*exp(-y*a)*erf(1/2*(y-a*sigma^2)*2^(1/2)/sigma)-3/4*exp(y*a)*erf((1/2*y+1/2*a*sigma^2)/sigma*2^(1/2)))/(a*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))))*sigma*a/pi^(1/2)*2^(1/2)*exp(-t^2/sigma^2+1/2*a^2*sigma^2)+1/2/(a*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))))/sigma*(-y*exp(-y*a)+y*exp(y*a)-y*exp(-y*a)*erf(1/2*(y-a*sigma^2)*2^(1/2)/sigma)-sigma/pi^(1/2)*2^(1/2)*exp(-y*a-1/2*(y-a*sigma^2)^2/sigma^2)-y*exp(y*a)*erf((1/2*y+1/2*a*sigma^2)/sigma*2^(1/2))-sigma/pi^(1/2)*2^(1/2)*exp(y*a-1/2*(y+a*sigma^2)^2/sigma^2))/pi^(1/2)*2^(1/2)*exp(-t^2/sigma^2+1/2*a^2*sigma^2)+(1/2*exp(-y*a)+1/2*exp(y*a)+1/2*exp(-y*a)*erf(1/2*(y-a*sigma^2)*2^(1/2)/sigma)-1/2*exp(y*a)*erf((1/2*y+1/2*a*sigma^2)/sigma*2^(1/2)))/(a*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))))^3/sigma*a*(exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+a*(-t+a*sigma^2)*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-sigma*a/pi^(1/2)*2^(1/2)*exp(-a*t+1/2*a^2*sigma^2-1/2*(-t+a*sigma^2)^2/sigma^2)+(-1/2*exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-1/2*exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma)))*exp(1/2*a^2*sigma^2)-1/2*a^2*sigma^2*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma)))-1/2*a*exp(1/2*a^2*sigma^2)*(-t*exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-sigma/pi^(1/2)*2^(1/2)*exp(-a*t-1/2*(-t+a*sigma^2)^2/sigma^2)+t*exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))-sigma/pi^(1/2)*2^(1/2)*exp(a*t-1/2*(-t-a*sigma^2)^2/sigma^2)))^2/pi^(1/2)*2^(1/2)*exp(-t^2/sigma^2+1/2*a^2*sigma^2)+(-1/2*exp(-y*a)-1/2*exp(y*a)-1/2*exp(-y*a)*erf(1/2*(y-a*sigma^2)*2^(1/2)/sigma)+1/2*exp(y*a)*erf((1/2*y+1/2*a*sigma^2)/sigma*2^(1/2)))/(a*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))))^2*sigma*a^2*(exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+a*(-t+a*sigma^2)*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-sigma*a/pi^(1/2)*2^(1/2)*exp(-a*t+1/2*a^2*sigma^2-1/2*(-t+a*sigma^2)^2/sigma^2)+(-1/2*exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-1/2*exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma)))*exp(1/2*a^2*sigma^2)-1/2*a^2*sigma^2*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma)))-1/2*a*exp(1/2*a^2*sigma^2)*(-t*exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-sigma/pi^(1/2)*2^(1/2)*exp(-a*t-1/2*(-t+a*sigma^2)^2/sigma^2)+t*exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))-sigma/pi^(1/2)*2^(1/2)*exp(a*t-1/2*(-t-a*sigma^2)^2/sigma^2)))/pi^(1/2)*2^(1/2)*exp(-t^2/sigma^2+1/2*a^2*sigma^2)-1/2/(a*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))))^2/sigma*a*(exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+a*(-t+a*sigma^2)*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-sigma*a/pi^(1/2)*2^(1/2)*exp(-a*t+1/2*a^2*sigma^2-1/2*(-t+a*sigma^2)^2/sigma^2)+(-1/2*exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-1/2*exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma)))*exp(1/2*a^2*sigma^2)-1/2*a^2*sigma^2*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma)))-1/2*a*exp(1/2*a^2*sigma^2)*(-t*exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-sigma/pi^(1/2)*2^(1/2)*exp(-a*t-1/2*(-t+a*sigma^2)^2/sigma^2)+t*exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))-sigma/pi^(1/2)*2^(1/2)*exp(a*t-1/2*(-t-a*sigma^2)^2/sigma^2)))*(-y*exp(-y*a)+y*exp(y*a)-y*exp(-y*a)*erf(1/2*(y-a*sigma^2)*2^(1/2)/sigma)-sigma/pi^(1/2)*2^(1/2)*exp(-y*a-1/2*(y-a*sigma^2)^2/sigma^2)-y*exp(y*a)*erf((1/2*y+1/2*a*sigma^2)/sigma*2^(1/2))-sigma/pi^(1/2)*2^(1/2)*exp(y*a-1/2*(y+a*sigma^2)^2/sigma^2))/pi^(1/2)*2^(1/2)*exp(-t^2/sigma^2+1/2*a^2*sigma^2)+(-1/4*exp(-y*a)-1/4*exp(y*a)-1/4*exp(-y*a)*erf(1/2*(y-a*sigma^2)*2^(1/2)/sigma)+1/4*exp(y*a)*erf((1/2*y+1/2*a*sigma^2)/sigma*2^(1/2)))/(a*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))))^2*((2-2*erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))*(-t+a*sigma^2)*exp(-a*t+1/2*a^2*sigma^2)-2*sigma/pi^(1/2)*2^(1/2)*exp(-a*t+1/2*a^2*sigma^2-1/2*(-t+a*sigma^2)^2/sigma^2)+a*sigma^2*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+a*(-t+a*sigma^2)^2*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-2*sigma*a*(-t+a*sigma^2)/pi^(1/2)*2^(1/2)*exp(-a*t+1/2*a^2*sigma^2-1/2*(-t+a*sigma^2)^2/sigma^2)-sigma*a*(t-a*sigma^2)/pi^(1/2)*2^(1/2)*exp(-a*t+1/2*a^2*sigma^2-1/2*(-t+a*sigma^2)^2/sigma^2)-3/2*a*sigma^2*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma)))+(t*exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+sigma/pi^(1/2)*2^(1/2)*exp(-a*t-1/2*(-t+a*sigma^2)^2/sigma^2)-t*exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))+sigma/pi^(1/2)*2^(1/2)*exp(a*t-1/2*(-t-a*sigma^2)^2/sigma^2))*exp(1/2*a^2*sigma^2)-1/2*a^3*sigma^4*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma)))-a^2*sigma^2*exp(1/2*a^2*sigma^2)*(-t*exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-sigma/pi^(1/2)*2^(1/2)*exp(-a*t-1/2*(-t+a*sigma^2)^2/sigma^2)+t*exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))-sigma/pi^(1/2)*2^(1/2)*exp(a*t-1/2*(-t-a*sigma^2)^2/sigma^2))-1/2*a*exp(1/2*a^2*sigma^2)*(t^2*exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+2*sigma*t/pi^(1/2)*2^(1/2)*exp(-a*t-1/2*(-t+a*sigma^2)^2/sigma^2)-sigma*(t-a*sigma^2)/pi^(1/2)*2^(1/2)*exp(-a*t-1/2*(-t+a*sigma^2)^2/sigma^2)+t^2*exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))-2*sigma*t/pi^(1/2)*2^(1/2)*exp(a*t-1/2*(-t-a*sigma^2)^2/sigma^2)-sigma*(-t-a*sigma^2)/pi^(1/2)*2^(1/2)*exp(a*t-1/2*(-t-a*sigma^2)^2/sigma^2)))/sigma*a/pi^(1/2)*2^(1/2)*exp(-t^2/sigma^2+1/2*a^2*sigma^2)+(1/4*exp(-y*a)+1/4*exp(y*a)+1/4*exp(-y*a)*erf(1/2*(y-a*sigma^2)*2^(1/2)/sigma)-1/4*exp(y*a)*erf((1/2*y+1/2*a*sigma^2)/sigma*2^(1/2)))/(a*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))))*sigma^3*a^3/pi^(1/2)*2^(1/2)*exp(-t^2/sigma^2+1/2*a^2*sigma^2)+1/2/(a*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))))*sigma*a^2*(-y*exp(-y*a)+y*exp(y*a)-y*exp(-y*a)*erf(1/2*(y-a*sigma^2)*2^(1/2)/sigma)-sigma/pi^(1/2)*2^(1/2)*exp(-y*a-1/2*(y-a*sigma^2)^2/sigma^2)-y*exp(y*a)*erf((1/2*y+1/2*a*sigma^2)/sigma*2^(1/2))-sigma/pi^(1/2)*2^(1/2)*exp(y*a-1/2*(y+a*sigma^2)^2/sigma^2))/pi^(1/2)*2^(1/2)*exp(-t^2/sigma^2+1/2*a^2*sigma^2)+1/4/(a*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))))/sigma*a*(y^2*exp(-y*a)+y^2*exp(y*a)+y^2*exp(-y*a)*erf(1/2*(y-a*sigma^2)*2^(1/2)/sigma)+2*y*sigma/pi^(1/2)*2^(1/2)*exp(-y*a-1/2*(y-a*sigma^2)^2/sigma^2)-sigma*(y-a*sigma^2)/pi^(1/2)*2^(1/2)*exp(-y*a-1/2*(y-a*sigma^2)^2/sigma^2)-y^2*exp(y*a)*erf((1/2*y+1/2*a*sigma^2)/sigma*2^(1/2))-2*y*sigma/pi^(1/2)*2^(1/2)*exp(y*a-1/2*(y+a*sigma^2)^2/sigma^2)-sigma*(-y-a*sigma^2)/pi^(1/2)*2^(1/2)*exp(y*a-1/2*(y+a*sigma^2)^2/sigma^2))/pi^(1/2)*2^(1/2)*exp(-t^2/sigma^2+1/2*a^2*sigma^2);

function f = dw_dadt( a, t, sigma )
f = 2*2^(1/2)/pi^(1/2)/sigma^3*t*exp(-t^2/sigma^2)/(a*exp(-a*t+1/2*a^2*sigma^2)*(1-erf((-1/2*t+1/2*a*sigma^2)/sigma*2^(1/2)))+2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf((-1/2*t+1/2*a*sigma^2)/sigma*2^(1/2)))+exp(a*t)*(1+erf((-1/2*t-1/2*a*sigma^2)/sigma*2^(1/2)))))^2*(exp(-a*t+1/2*a^2*sigma^2)*(1-erf((-1/2*t+1/2*a*sigma^2)/sigma*2^(1/2)))+a*(-t+a*sigma^2)*exp(-a*t+1/2*a^2*sigma^2)*(1-erf((-1/2*t+1/2*a*sigma^2)/sigma*2^(1/2)))-a*sigma/pi^(1/2)*2^(1/2)*exp(-a*t+1/2*a^2*sigma^2-1/2*(-t+a*sigma^2)^2/sigma^2)+(-1/2*exp(-a*t)*(1-erf((-1/2*t+1/2*a*sigma^2)/sigma*2^(1/2)))-1/2*exp(a*t)*(1+erf((-1/2*t-1/2*a*sigma^2)/sigma*2^(1/2))))*exp(1/2*a^2*sigma^2)-1/2*a^2*sigma^2*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf((-1/2*t+1/2*a*sigma^2)/sigma*2^(1/2)))+exp(a*t)*(1+erf((-1/2*t-1/2*a*sigma^2)/sigma*2^(1/2))))-1/2*a*exp(1/2*a^2*sigma^2)*((-1+erf((-1/2*t+1/2*a*sigma^2)/sigma*2^(1/2)))*t*exp(-a*t)-sigma/pi^(1/2)*2^(1/2)*exp(-a*t-1/2*(-t+a*sigma^2)^2/sigma^2)+t*exp(a*t)*(1+erf((-1/2*t-1/2*a*sigma^2)/sigma*2^(1/2)))-sigma/pi^(1/2)*2^(1/2)*exp(a*t-1/2*(-t-a*sigma^2)^2/sigma^2)))+(2*(-1+erf((-1/2*t+1/2*a*sigma^2)/sigma*2^(1/2)))*a^2*exp(-a*t+1/2*a^2*sigma^2)+2*a/sigma/pi^(1/2)*2^(1/2)*exp(-a*t+1/2*a^2*sigma^2-1/2*(-t+a*sigma^2)^2/sigma^2)-4*2^(1/2)/pi^(1/2)/sigma^3*t*exp(-t^2/sigma^2)-a*exp(1/2*a^2*sigma^2)*((-1+erf((-1/2*t+1/2*a*sigma^2)/sigma*2^(1/2)))*a*exp(-a*t)+1/sigma/pi^(1/2)*2^(1/2)*exp(-a*t-1/2*(-t+a*sigma^2)^2/sigma^2)+a*exp(a*t)*(1+erf((-1/2*t-1/2*a*sigma^2)/sigma*2^(1/2)))-1/sigma/pi^(1/2)*2^(1/2)*exp(a*t-1/2*(-t-a*sigma^2)^2/sigma^2)))*(exp(-a*t+1/2*a^2*sigma^2)*(1-erf((-1/2*t+1/2*a*sigma^2)/sigma*2^(1/2)))+a*(-t+a*sigma^2)*exp(-a*t+1/2*a^2*sigma^2)*(1-erf((-1/2*t+1/2*a*sigma^2)/sigma*2^(1/2)))-a*sigma/pi^(1/2)*2^(1/2)*exp(-a*t+1/2*a^2*sigma^2-1/2*(-t+a*sigma^2)^2/sigma^2)+(-1/2*exp(-a*t)*(1-erf((-1/2*t+1/2*a*sigma^2)/sigma*2^(1/2)))-1/2*exp(a*t)*(1+erf((-1/2*t-1/2*a*sigma^2)/sigma*2^(1/2))))*exp(1/2*a^2*sigma^2)-1/2*a^2*sigma^2*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf((-1/2*t+1/2*a*sigma^2)/sigma*2^(1/2)))+exp(a*t)*(1+erf((-1/2*t-1/2*a*sigma^2)/sigma*2^(1/2))))-1/2*a*exp(1/2*a^2*sigma^2)*((-1+erf((-1/2*t+1/2*a*sigma^2)/sigma*2^(1/2)))*t*exp(-a*t)-sigma/pi^(1/2)*2^(1/2)*exp(-a*t-1/2*(-t+a*sigma^2)^2/sigma^2)+t*exp(a*t)*(1+erf((-1/2*t-1/2*a*sigma^2)/sigma*2^(1/2)))-sigma/pi^(1/2)*2^(1/2)*exp(a*t-1/2*(-t-a*sigma^2)^2/sigma^2)))/sigma/(a*exp(-a*t+1/2*a^2*sigma^2)*(1-erf((-1/2*t+1/2*a*sigma^2)/sigma*2^(1/2)))+2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf((-1/2*t+1/2*a*sigma^2)/sigma*2^(1/2)))+exp(a*t)*(1+erf((-1/2*t-1/2*a*sigma^2)/sigma*2^(1/2)))))^3/pi^(1/2)*2^(1/2)*exp(-t^2/sigma^2)+(-(-2+2*erf((-1/2*t+1/2*a*sigma^2)/sigma*2^(1/2)))*a*exp(-a*t+1/2*a^2*sigma^2)-1/sigma/pi^(1/2)*2^(1/2)*exp(-a*t+1/2*a^2*sigma^2-1/2*(-t+a*sigma^2)^2/sigma^2)-(t-a*sigma^2)*(1-erf((-1/2*t+1/2*a*sigma^2)/sigma*2^(1/2)))*a^2*exp(-a*t+1/2*a^2*sigma^2)-a^2*sigma/pi^(1/2)*2^(1/2)*exp(-a*t+1/2*a^2*sigma^2-1/2*(-t+a*sigma^2)^2/sigma^2)-(-1/2*(-1+erf((-1/2*t+1/2*a*sigma^2)/sigma*2^(1/2)))*a*exp(-a*t)-1/2/sigma/pi^(1/2)*2^(1/2)*exp(-a*t-1/2*(-t+a*sigma^2)^2/sigma^2)-1/2*a*exp(a*t)*(1+erf((-1/2*t-1/2*a*sigma^2)/sigma*2^(1/2)))+1/2/sigma/pi^(1/2)*2^(1/2)*exp(a*t-1/2*(-t-a*sigma^2)^2/sigma^2))*exp(1/2*a^2*sigma^2)+1/2*a^2*sigma^2*exp(1/2*a^2*sigma^2)*((-1+erf((-1/2*t+1/2*a*sigma^2)/sigma*2^(1/2)))*a*exp(-a*t)+1/sigma/pi^(1/2)*2^(1/2)*exp(-a*t-1/2*(-t+a*sigma^2)^2/sigma^2)+a*exp(a*t)*(1+erf((-1/2*t-1/2*a*sigma^2)/sigma*2^(1/2)))-1/sigma/pi^(1/2)*2^(1/2)*exp(a*t-1/2*(-t-a*sigma^2)^2/sigma^2))+1/2*a*exp(1/2*a^2*sigma^2)*((-1+erf((-1/2*t+1/2*a*sigma^2)/sigma*2^(1/2)))*exp(-a*t)+t*a*exp(-a*t)*(1-erf((-1/2*t+1/2*a*sigma^2)/sigma*2^(1/2)))-t/sigma/pi^(1/2)*2^(1/2)*exp(-a*t-1/2*(-t+a*sigma^2)^2/sigma^2)+a*sigma/pi^(1/2)*2^(1/2)*exp(-a*t-1/2*(-t+a*sigma^2)^2/sigma^2)+(t-a*sigma^2)/sigma/pi^(1/2)*2^(1/2)*exp(-a*t-1/2*(-t+a*sigma^2)^2/sigma^2)+exp(a*t)*(1+erf((-1/2*t-1/2*a*sigma^2)/sigma*2^(1/2)))+t*a*exp(a*t)*(1+erf((-1/2*t-1/2*a*sigma^2)/sigma*2^(1/2)))-t/sigma/pi^(1/2)*2^(1/2)*exp(a*t-1/2*(-t-a*sigma^2)^2/sigma^2)-a*sigma/pi^(1/2)*2^(1/2)*exp(a*t-1/2*(-t-a*sigma^2)^2/sigma^2)+(t+a*sigma^2)/sigma/pi^(1/2)*2^(1/2)*exp(a*t-1/2*(-t-a*sigma^2)^2/sigma^2)))/sigma/(a*exp(-a*t+1/2*a^2*sigma^2)*(1-erf((-1/2*t+1/2*a*sigma^2)/sigma*2^(1/2)))+2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf((-1/2*t+1/2*a*sigma^2)/sigma*2^(1/2)))+exp(a*t)*(1+erf((-1/2*t-1/2*a*sigma^2)/sigma*2^(1/2)))))^2/pi^(1/2)*2^(1/2)*exp(-t^2/sigma^2);

function f = dw_dt2( a, t, sigma )
f = -2*2^(1/2)/pi^(1/2)/sigma^3*exp(-t^2/sigma^2)/(a*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))))+4*2^(1/2)/pi^(1/2)/sigma^5*t^2*exp(-t^2/sigma^2)/(a*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))))+(-4*a^2*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+4*a/sigma/pi^(1/2)*2^(1/2)*exp(-a*t+1/2*a^2*sigma^2-1/2*(-t+a*sigma^2)^2/sigma^2)-8*2^(1/2)/pi^(1/2)/sigma^3*t*exp(-t^2/sigma^2)-2*a*exp(1/2*a^2*sigma^2)*(-a*exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+1/sigma/pi^(1/2)*2^(1/2)*exp(-a*t-1/2*(-t+a*sigma^2)^2/sigma^2)+a*exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))-1/sigma/pi^(1/2)*2^(1/2)*exp(a*t-1/2*(-t-a*sigma^2)^2/sigma^2)))*t/(a*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))))^2/sigma^3/pi^(1/2)*2^(1/2)*exp(-t^2/sigma^2)+2*2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)/(a*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))))^3*(-a^2*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+a/sigma/pi^(1/2)*2^(1/2)*exp(-a*t+1/2*a^2*sigma^2-1/2*(-t+a*sigma^2)^2/sigma^2)-2*2^(1/2)/pi^(1/2)/sigma^3*t*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(-a*exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+1/sigma/pi^(1/2)*2^(1/2)*exp(-a*t-1/2*(-t+a*sigma^2)^2/sigma^2)+a*exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))-1/sigma/pi^(1/2)*2^(1/2)*exp(a*t-1/2*(-t-a*sigma^2)^2/sigma^2)))^2-2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)/(a*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+2^(1/2)/pi^(1/2)/sigma*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))+exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))))^2*(a^3*exp(-a*t+1/2*a^2*sigma^2)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-2*a^2/sigma/pi^(1/2)*2^(1/2)*exp(-a*t+1/2*a^2*sigma^2-1/2*(-t+a*sigma^2)^2/sigma^2)+a/sigma^3*(-t+a*sigma^2)/pi^(1/2)*2^(1/2)*exp(-a*t+1/2*a^2*sigma^2-1/2*(-t+a*sigma^2)^2/sigma^2)-2*2^(1/2)/pi^(1/2)/sigma^3*exp(-t^2/sigma^2)+4*2^(1/2)/pi^(1/2)/sigma^5*t^2*exp(-t^2/sigma^2)-1/2*a*exp(1/2*a^2*sigma^2)*(a^2*exp(-a*t)*(1-erf(1/2*(-t+a*sigma^2)*2^(1/2)/sigma))-2*a/sigma/pi^(1/2)*2^(1/2)*exp(-a*t-1/2*(-t+a*sigma^2)^2/sigma^2)+1/sigma^3*(-t+a*sigma^2)/pi^(1/2)*2^(1/2)*exp(-a*t-1/2*(-t+a*sigma^2)^2/sigma^2)+a^2*exp(a*t)*(1+erf(1/2*(-t-a*sigma^2)*2^(1/2)/sigma))-2*a/sigma/pi^(1/2)*2^(1/2)*exp(a*t-1/2*(-t-a*sigma^2)^2/sigma^2)-1/sigma^3*(-t-a*sigma^2)/pi^(1/2)*2^(1/2)*exp(a*t-1/2*(-t-a*sigma^2)^2/sigma^2)));


% Densities
function f = f1( x, sigma )
f = normpdf( x, 0, sigma );
f = f(:);
f = max( [f'; 1e3*eps*ones(size(f'))] );
f = f(:);

function f = f2( x, a, sigma )
f = 1/4*a*exp(a^2*sigma^2/2)*( exp(a*x)*(1-erf( (x+a*sigma^2)/sqrt(2)/sigma )) + exp(-a*x)*(1+erf( (x-a*sigma^2)/sqrt(2)/sigma )) );

function f = fn_w( a, t, sigma )
f = 2*f1( -t, sigma )/( a*exp(-a*t+a^2*sigma^2/2)*(1-erf( (-t+a*sigma^2)/sqrt(2)/sigma )) + 2*f1( -t, sigma ) - 2*f2( -t, a, sigma ) );

function f = fn_g( x, a, t, sigma ) 
w = fn_w( a, t, sigma );
f = (1-w)*f1( x, sigma ) + 1/4*a*w*exp(a^2*sigma^2/2)*( exp(-a*x)+exp(a*x)+exp(-a*x).*erf( (x-a*sigma^2)/sqrt(2)/sigma ) - exp(a*x).*erf( (x+a*sigma^2)/sqrt(2)/sigma ) );
