% Plot a 3-d image in slices. Usage is
% plotSlices( nFig, M, L, bColourMapGray, nDelaySec )
% where M = 3d matrix, L = #levels to plot

function plotSlices( nFig, M, L, bColourMapGray, nDelaySec )
%###########################################################

disp( sprintf( 'Plotting %g slices...', L ) );
win = 220;
figure( nFig ), clf;

numSet = floor( L/4 );
extraSlices = L - 4*numSet;

for i = 1:numSet
    
    vecZValue = [1,2,3,4] + 4*(i-1);
    indBad = find( vecZValue > L );
    vecZValue( indBad ) = L*ones( 1, length(indBad) ); % Cap the values to L
    
    for j = 1:4
        figure( nFig );
        subplot( win+j ), imagesc( M( :, :, vecZValue(j) )' );    
        axis equal tight; title( sprintf( 'slice %g', vecZValue(j) ) );
        if bColourMapGray, colormap( gray ), end;
        colorbar;
    end
    
    if ( i < numSet )
        if ( nargin == 5 )
            pause( nDelaySec );
        else
            disp( 'Pausing...' ), pause;
        end
    end
    
end

if ( extraSlices > 0 ) % Plot the remaining slices
    
    if ( nargin == 5 )
        pause( nDelaySec );
    else
        disp( 'Pausing...' ), pause;
    end
    
    figure( nFig );
    for slice = 1:extraSlices
        subplot( win+slice );
        imagesc( M( :, :, 4*numSet+slice )' );
        axis equal tight; title( sprintf( 'slice %g', 4*numSet+slice ) );
        if bColourMapGray, colormap( gray ), end;
        colorbar;
    end
    
end


