
% Function to transform an area in (a,t) space to (a,w) space
% Keys: arange, trange, sdev, debug

function plot_ataw( varargin )

% Initialize

%aRange = [1.33824e-5, 40.1472];
%tRange = [1e-6, .040788];
%sdev = 1e-2;

aRange = [];
tRange = [];
sdev = -1;
bDebug = 0;

numArgs = length( varargin );
i = 1;

while 1
	szKey = lower( varargin{i} );
	switch szKey
		case 'arange' 
			if ( i < numArgs ), aRange = varargin{i+1}; i = i+1; 
			else, error( 'argument for arange missing' ), end;
		case 'trange'
			if ( i < numArgs ), tRange = varargin{i+1}; i = i+1;
			else, error( 'argument for trange missing' ), end;

		case 'sdev'
			if ( i < numArgs ), sdev = varargin{i+1}; i = i+1;
			else, error( 'argument for sdev missing' ), end;

		case 'debug'
			bDebug = 1;

		otherwise, error( 'unrecognized key' );
	end
	i = i + 1;
	if ( i > numArgs ), break, end;
end

% Check inputs
if	length( aRange ) ~= 2 | aRange(1) >= aRange(2) | ...
 	length( tRange ) ~= 2 | tRange(1) >= tRange(2) | ...
	sdev <= 0
	error( 'invalid inputs' );
end

%[A,T]= ndgrid( logspace( log10(aRange(1)), log10(aRange(2)), 50 ), ...
%	logspace( log10(tRange(1)), log10(tRange(2)), 50 ) );
[A,T]= ndgrid( logspace( log10(aRange(1)), log10(aRange(2)), 100 ), ...
	logspace( log10(tRange(1)), log10(tRange(2)), 200 ) );
[m,n] = size( A );
W = zeros( [m,n] );
for indM = 1:m
	for indN = 1:n
		W( indM, indN ) = convert_tsigma2w( A(indM,indN), T(indM,indN), sdev, 'median' );
		if bDebug
			disp( sprintf( 'a=%g, t=%g, w=%g', A(indM,indN), T(indM,indN), W(indM,indN) ) );
		end
	end
end

figure( 1 ), clf;
semilogx( A(:), T(:), 'bx' ), xlabel( 'a' ), ylabel( 't' );
title( 't vs. a plot' );

figure( 2 ), clf;
semilogx( A(:), W(:), 'ro' ), xlabel( 'a' ), ylabel( 'w' );
title( 'w vs. a plot' );
		
