% powplot: plots the power curve based on data contained in files
%-------------------------------------------------------------------------------
% Given that the PD values have been saved in files, this function plots the
% power curve. It is assumed that the variable roc_pow has been saved in all
% the files, and that it is a N x 2 matrix. Each row is a (PD, sigma) pair,
% where sigma is the standard deviation. Error bars shall be plotted for +/-
% sigma.
%
% powplot( vecSnr, vecFilename, vecLegend, vecMarker, vecRocIndex )
%    vecSnr: SNR values
%    vecFilename: filenames corresponding to vecSnr
%    vec{Legend, Marker}: labels for each row of roc_pow
%    vecRocIndex: indices into roc_pow for the detectors in vecMarker
%
function powplot( vecSnr, vecFilename, vecLegend, vecMarker, vecRocIndex )

if ( nargin ~= 5 ), error( 'incorrect # arguments' ), end;

numSnr = length( vecSnr );
numCurves = length( vecLegend );

% The p* arrays contain the power (PD), while the s* arrays contain the
% standard deviation.
matPower = [];
matStd = [];

for snrInd = 1:numSnr
	snr = vecSnr( snrInd );
	load( char( vecFilename(snrInd) ), 'roc_pow' );
	tmp = roc_pow( vecRocIndex, 1 );
	matPower = [matPower, tmp(:)];
	tmp = roc_pow( vecRocIndex, 2 );
	matStd = [matStd, tmp(:)];
	clear roc_pow;
end

% DEBUG
%matPower
%matStd
%disp( 'C1' );

close all,figure;

% Create dummy plots 
for curveInd = 1:numCurves
	plot( vecSnr(1), matPower( curveInd, 1 ), char( vecMarker(curveInd) ) );
	if ( curveInd == 1 ), hold on, end
end 

% Create the legend
legend( vecLegend, 0 );

% DEBUG
%disp( 'C2' );

% Then erase the dummy plots
for curveInd = 1:numCurves
	plot( vecSnr(1), matPower( curveInd, 1 ), ...
		whiteOut( char( vecMarker(curveInd) ) ) );
end

% Plot the real curves
for curveInd = 1:numCurves
	errorbar( vecSnr, matPower( curveInd, : ), matStd( curveInd, : ), ...
		char( vecMarker(curveInd) ) );
end

xlabel( 'SNR (dB)' );
ylabel( 'Probability of detection (P_D)' );
axis tight;

%...............................................................................
function szOut = whiteOut( sIn )
%###############################################################################% Assume that the plot background is white. If not, then change to the
% appropriate colour.
szOut = strcat( 'w', sIn(2:end) );

