% pseudoinv_eqdim: Applies the pseudo-inverse to an image. Assumes that the
% dimensions of h and g are the same. Apply the circular convolution approx
% as well.
%
% Usage:
%    f = pseudoinv_eqdim( h, g )
%
%    f - output of deconvolution (pseudoinverse)
%    h - forward convolution operator
%    g - output
%

function f = pseudoinv_eqdim( h, g )

ZERO_THRES = 1e5*eps;

% Check that the dimensions of h and g are the same
hSize = size( h );
gSize = size( g );
numDiffSizes = sum( (hSize == gSize) ~= 1 );
if ( numDiffSizes > 0 )
	error( 'pseudoinv_eqdim: h and g different sizes' );
end

% Compute the FFTs of h and g. Zero pad the convolution operator as
% necessary.
hDft = fftn( h, size( g ) );
gDft = fftn( g, size( g ) );

%hDftInv = (1 ./ hDft) .* ( abs(hDft) > ZERO_THRES );
hDftLinear = hDft(:);
warning off
hDftInvRough = 1 ./ hDftLinear;
warning on
indValid = find( isfinite( hDftInvRough ) );
hDftInv = zeros( size( hDftInvRough ) );
hDftInv( indValid ) = hDftInvRough( indValid );
hDftInv = hDftInv .* ( abs(hDftLinear) > ZERO_THRES );
hDftInv = reshape( hDftInv, size(hDft) );

f = real( ifftn( gDft .* hDftInv, size( g ) ) );

