% recon_mlt: implement the ML-T method described in the image recon paper
%-------------------------------------------------------------------------------
% [fMLT, mseMLT, bestThres, mseTest] = recon_mlt( out, fML )
%
function [fMLT, mseMLT, bestThres, mseTest] = recon_mlt( out, fML )

L = prod( size( out.f ) );

fMLmin = min( fML(:) );
fMLmax = max( fML(:) );
diff = (fMLmax - fMLmin) + 1;

numThres = 300;
vecThres = linspace( fMLmin - 0.2*diff, fMLmax + 0.2*diff, numThres );

disp( sprintf( 'fMLmin: %.5e, fMLmax: %.5e, vecThres = [%g,%g]', ...
	fMLmin, fMLmax, vecThres(1), vecThres(end) ) );

mseTest = zeros( numThres, 1 );

for i = 1:numThres
	thres = vecThres(i);
	fTest = fML .* ( fML > thres );
	mseTest(i) = (1/L) * sum( (fTest(:) - out.f(:)).^2 );
	if ( i == 1 )
		fMLT = fTest;
		mseMLT = mseTest(i);
		bestThres = thres;
	else % i > 1, so compare with best candidate so far
		if ( mseTest(i) < mseMLT )
			fMLT = fTest;
			mseMLT = mseTest(i);
			bestThres = thres;
		end
	end
end

disp( sprintf( 'mseMLT: %.5e, bestThres: %g', mseMLT, bestThres ) ); 

