
function tBetter = refine_js_threshold( varargin )

% Initialize variables

a = -1;
w = -1;
noiseSigma = -1;
t0 = -1;
bInit_t0 = 0;

% Process input arguments

lenVarargin = length( varargin );
numArgsIn = floor( lenVarargin/2 );

if 2*numArgsIn ~= lenVarargin, error( 'even ip args expected' ), end

for argInInd = 1:numArgsIn
    
    szKey = lower( varargin{2*argInInd-1} );    
    switch szKey
        case 'a', a = varargin{2*argInInd};
        case 'w', w = varargin{2*argInInd};
        case 'sigma', noiseSigma = varargin{2*argInInd};
        case 't0', t0 = varargin{2*argInInd}; bInit_t0 = 1;
        otherwise
            error( sprintf( 'unrecognized option %s', szKey ) );
    end
    
end

% Check that all the arguments were initialized

if a < 0 | w < 0 | noiseSigma < 0 | ~bInit_t0
    error( 'there are uninitialized variables' );
end

% Compute the derivative of K(t) at t=t_0

KValAt_t0 = a*w * exp(-a*t0+1/2*a^2*noiseSigma^2) * (1 - erf( (-t0+a*noiseSigma^2)/sqrt(2)/noiseSigma )) - ...
    2 * fLaplacianGauss( -t0, a, w, noiseSigma );

KDerivAt_t0 = -1/2*a^2*w * exp(-a*t0+1/2*a^2*noiseSigma^2) * (1 + erf( (t0-a*noiseSigma^2)/sqrt(2)/noiseSigma )) - ...
    1/2*a^2*w * exp(a*t0+1/2*a^2*noiseSigma^2) * (1 - erf( (t0+a*noiseSigma^2)/sqrt(2)/noiseSigma )) + ...
    sqrt(2/pi)/noiseSigma^3 * exp(-t0^2/2/noiseSigma^2) * ( w*a*noiseSigma^2 + (1-w)*t0 );

% Expand K(t) as a linear equation and solve
% Get tBetter = t_0 - K(t_0)/K'(t_0)
tBetter = t0 - KValAt_t0 / KDerivAt_t0;

if tBetter < 0
    disp( sprintf( 'WARNING: tBetter = %g', tBetter ) );
end
