% restore_example.m
% Example of edge-preserving image restoration
% using Gblur object for system model (shift-invariant blur)
% and using Robject.m to form nonquadratic penalty function.
%
% Copyright 2002-5-10, Jeff Fessler, The University of Michigan

if ~isvar('xtrue'), disp 'read image'
%	xtrue = double(imread('fig-tomos-old.gif'))';
%	xtrue = (1 - xtrue / 3) * 100;
%	xtrue = xtrue(50+[1:1100], 60+[1:360], :);
	try
		xtrue = double(imread('fig,tomos.gif'))';
	catch
		xtrue = double(imread('fig,tomos.tif'))';
	end
	xtrue = (1 - xtrue / max(xtrue(:))) * 100;
	xtrue = xtrue(50+[1:1000], 0+[1:400], :);
	kern = gaussian_kernel(4);
	xtrue = conv2(xtrue, kern, 'same');
	xtrue = conv2(xtrue, kern', 'same');
	xtrue = xtrue(2:4:end,2:4:end);	% downsample for speed
%	xtrue = xtrue(1:2:end,:) + xtrue(2:2:end,:);
%	xtrue = xtrue(:,1:2:end) + xtrue (:,2:2:end);
	[nx,ny] = size(xtrue);
end


% this PSF makes visible blur yet is still invertible (for illustration)
if ~isvar('psf'), disp 'psf'
	psf = ones(5,1);
	psf((end+1)/2) = 5;
	psf = psf / sum(psf(:));
	psf = psf * psf';

	psfpad = zeros(nx,ny);
	psfpad(1:size(psf,1), 1:size(psf,2)) = psf;
	psfpad = psfpad([3:nx 1 2], :);
	psfpad = psfpad(:, [3:ny 1 2]);
	Psf = reale(fft2(psfpad));
end

clim = [0 100];

if ~isvar('xinv')
	y0 = conv2(xtrue, psf, 'same');

	x0 = reale(ifft2((1 ./ Psf) .* fft2(y0)));
	x0 = max(x0,0);

	randn('state', 0)
	estd = 10;
	yi = y0 + estd * randn(size(y0));

	xinv = reale(ifft2((1 ./ Psf) .* fft2(yi)));

	xml = 100 * (xinv > 50);
end

if ~isvar('G')
	G = Gblur(logical(ones(nx, ny)), 'psf', psf);
	mask = G.arg.mask;
end

if ~isvar('xqpls')
	f.l2b_q = -1;
	f.niter = 20;
	Rq = Robject(mask, 'type_denom', 'matlab', ...
		'potential', 'quad', 'beta', 2^f.l2b_q);

	xinit = yi;
	xqpls = pwls_sps_os(xinit(:), yi(:), [], G, Rq, ...
			f.niter, inf, [], [], 1);
	xqpls = embed(xqpls, mask);
	xqpls = max(xqpls(:,:,end), 0);
end

if ~isvar('xnpls') | 1
%	f.l2b_n = 1.5; % cauchy
	f.l2b_n = 8;
	Rn = Robject(mask, 'type_denom', 'matlab', ...
		'potential', 'hyper3', 'beta', 2^f.l2b_n, 'delta', 0.1);
%		'potential', 'cauchy', 'beta', 2^f.l2b_n, 'delta', 10);
	xinit = yi;
	xnpls = pwls_sps_os(xinit(:), yi(:), [], G, Rn, ...
		3*f.niter, inf, [], [], 1);
	xnpls = embed(xnpls, mask);
	xnpls = max(xnpls(:,:,end), 0);
end

% wiener filter
if ~isvar('xwien'), disp 'wiener filter'
	yi0 = yi;
%	yi0 = eye(4);
	yi0 = yi0 - mean(yi0(:)); % zero-mean
	Ry = xcorr2(yi0) / prod(size(yi0)); % empirical autocorrelation
	[mx my] = size(yi0);
	Ry = Ry([-mx/2:mx/2-1]+mx,[-my/2:my/2-1]+my);
	Ry(:,1) = 0; Ry(1,:) = 0;
	Py = reale(fftshift(fft2(ifftshift(Ry))), 'warn');
	Px = max(Py-estd^2, 0);
	H = Px ./ (Px + estd^2); % Wiener filter
	xwien = reale(ifft2(fft2(yi) .* fftshift(H)));
%	imax2(Ry);
%	plot(Ry(:,(end+1)/2))
%	im(max(Ry-estd^2, 0))
prompt
end

Nrms = 'sprintf(''%5.2f'', nrms(x(:), xtrue(:)))';
Nrms = inline(Nrms, 'x', 'xtrue');

% individual figures for preface example
if 0
	clf
	set(0, 'DefaultAxesFontSize', 22)
	im('notick', xtrue, clim), colormap(1-gray)
	title 'True image x'
	savefig 'fig_front_xtrue'

	im('notick', yi, clim), colormap(1-gray)
	title 'Noisy/blurry data y'
	savefig 'fig_front_y'

	im('notick', xinv, clim), colormap(1-gray)
	title 'Inverse filter estimated x'
	savefig 'fig_front_xinv'

	im('notick', xnpls, clim), colormap(1-gray)
	title 'Regularized, statistical estimated x'
	savefig 'fig_front_xhat'
return
end

if 1 & im
	clf, pl=520;
	set(0, 'DefaultAxesFontSize', 16)
	im(pl+1, 'notick', xtrue, clim)
	title 'True x'
	ylabel(Nrms(xtrue,xtrue))

	im(pl+2, 'notick', y0, clim)
	title 'Blurry y (no noise)'
	ylabel(Nrms(y0,xtrue))

	im(pl+3, 'notick', x0, clim)
	title 'Inverse, no noise'
	ylabel(Nrms(x0,xtrue))

	im(pl+4, 'notick', yi, clim)
	title 'Noisy/blurry y'
	ylabel(Nrms(yi,xtrue))

	im(pl+5, 'notick', xinv, clim)
	title 'Inverse, with noise'
	ylabel(Nrms(xinv,xtrue))

	im(pl+6, 'notick', xml, clim)
	title 'Thresholded Inverse'
	ylabel(Nrms(xml,xtrue))

	im(pl+7, 'notick', xqpls, clim)
	title 'Quadratic Penalized LS'
	ylabel(Nrms(xqpls,xtrue))

	im(pl+8, 'notick', xnpls, clim)
	title 'Non-Quadratic PLS'
	ylabel(Nrms(xnpls,xtrue))

	im(pl+9, 'notick', xwien, clim)
	title 'Wiener Filter'
	ylabel(Nrms(xwien,xtrue))

	colormap(1-gray)

%	set(gcf, 'papersize', [8.5 8]), orient tall
%	savefig 'fig_tomos'
return
end

if 1 & im
	c = (Rn.C1) * xnpls(mask);
	c = reshape(c, [size(mask) 4]);
	c = Rn.wpot(1, c);
	clf
	im(321, 'notick', c(:,:,1), 'Horizontal')
	im(322, 'notick', c(:,:,2), 'Vertical')
	im(323, 'notick', c(:,:,3), 'Diagonal Up-Left')
	im(324, 'notick', c(:,:,4), 'Diagonal Up-Right')
%	set(gcf, 'papersize', [8.5 6]), orient tall
%	savefig 'fig_tomos_wpot'
end
