
% Compute error norm. Usage is: sErr = sp_img_error( keys )
% Valid keys: xrecon, xtrue, bplot

function sErr = sp_img_error( varargin )

% Default arguments
xrecon = [];
xtrue = [];
B_PLOT = 0;

numArg = floor( length(varargin)/2 );
if 2*numArg ~= length( varargin ), error( 'even args expected' ), end;

for indArg = 1:numArg
    szKey = char( varargin{2*indArg-1} );
    switch lower( szKey )
        case 'xrecon', xrecon = varargin{2*indArg};
        case 'xtrue', xtrue = varargin{2*indArg};
        case 'bplot', B_PLOT = varargin{2*indArg};
        otherwise, error( 'unrecognized key' );
    end
end

if isempty( xrecon ) || isempty( xtrue )
    warning( 'xrecon and/or xtrue not specified' );
    sErr = struct( 'numMetrics', 6 );
    return;
end

xLast = reshape( xrecon, size(xtrue) );

xLast = xLast(:);
n = length( xLast );
xLastZeroMean = xLast - mean(xLast); 
xLastZeroMed = xLast - median(xLast); 
xtrue = xtrue(:);
xDiff = xtrue - xLast;

sErr = struct( 'l0', norm0( xtrue, xLast ), ...
	'l1', norm( xDiff, 1 ), ...
	'l2', norm( xDiff, 2 ), ...
	'K1', 1e-3, ...
	'K2', 3e-3, ...
	'L', max(abs(xtrue)), ...
	'delta', 1e-2*max(abs(xtrue)) ...
	);

if B_PLOT
    figure( 1 ), clf;
%     plot( xLast, 'bx' ), hold on;
%     plot( xtrue, 'ro' );
    stem( xLast, 'bx' ), hold on;
    stem( xtrue, 'ro' );
    plot( [1,n], [sErr.delta,sErr.delta], 'k-' );
    plot( [1,n], [-sErr.delta,-sErr.delta], 'k-' );
    legend( 'estimated', 'true', 0 );
end

% Use C1 = (K1*L)^2, C2 = (K2*L)^2, C3 = C2/2, as well as
% w = [alpha,beta,gamma] = [0,1,1]
sErr.ssim = compute_ssim( 'x', xtrue, 'y', xLast, ...
	'w', [0,1,1], ...
	'C', [(sErr.K1*sErr.L)^2, (sErr.K2*sErr.L)^2, 0.5*(sErr.K2*sErr.L)^2] ...
	);

% The values of l{hard,soft} should be less than the l1 error term
sErr.lhard = sum( abs( hard_thres( xDiff, sErr.delta ) ) );
sErr.lsoft = sum( abs( soft_thres( xDiff, sErr.delta ) ) );

% Figure out the number of non-zero locations in xtrue
sErr.trueNnz = sum( abs( xtrue ) > eps );

% Figure out if the non-zero and zero locations were successfully resolved
xorLoc = xor( abs( xtrue ) > eps, abs( xLast ) > sErr.delta );
sErr.numXorLoc = sum( xorLoc );

sErr.numMetrics = 7;



