
% Script to analyze the results of a simulation. Usage:
% sp_img_eval( arg pairs )
% bplotconv => {0,1}
% bploterror1 => {0,1}

function sp_img_eval( varargin )

% Initialize

sErr = sp_img_error;
NUM_ERROR_METRICS = sErr.numMetrics + 1;

% szDataFilename = { 'sbl.mat', ...
%     'eb_better.mat', ...
%     'eb_fixed.mat', ...
%     'map_prior.mat', ...
%     'eb_fix_w.mat', ...
%     'soft_fixed.mat' ...
%     };
% szMethods = { 'sbl', 'ebla', 'ebf', 'mapp', 'ebl', 'l1f' };
% szMarkers = { 'b-', 'r-', 'm-', 'g-', 'k-', 'c-' };

szDataFilename = { 'sbl.mat', ...
    'eb_better_nov05.mat', ...
    'map_prior.mat', ...
    'map_revised.mat', ...
    'eb_fix_w_nov05.mat',  ...
    'soft_fixed.mat', ...
    'exp1.mat', ...
    'exp2.mat', ...
    'dec15_xp1.mat' ...
    % Commented out
    %'landweb.mat' ...
    };
%szMethods = { 'sbl', 'ebla', 'mapp', 'ebl', 'l1f', 'exp1', 'exp2', 'lw' };
szMethods = { 'sbl', 'ebla', 'mapp', 'map', 'ebl', 'l1f', 'exp1', 'exp2', 'xp1' };
szMarkers = { 'b-', 'r-', 'm-', 'b:', 'g-', 'k-', 'c-', 'k:', 'r:' };

NUM_METHODS = length( szDataFilename );

bLoadedFile = zeros( 1, NUM_METHODS );

hPlotConv = 2;
hPlotError1 = hPlotConv+1;

problemProp = {};
errorMetrics = [];
szLegend = {};
bInitProblemProp = 0;

bCreateConvergencePlot = 0;
bCreateError1Plot = 0;

% Process input arguments

lenVarargin = length( varargin );
numArgsIn = floor( lenVarargin/2 );

if 2*numArgsIn ~= lenVarargin, error( 'even ip args expected' ), end

for argInInd = 1:numArgsIn
    
    szKey = lower( varargin{2*argInInd-1} );    
    switch szKey
        case 'bplotconv', bCreateConvergencePlot = varargin{2*argInInd};
        case 'bploterror1', bCreateError1Plot = varargin{2*argInInd};
        otherwise
            error( sprintf( 'unrecognized option %s', szKey ) );
    end
    
end

close all;

% Start loading data files

disp( '___LOADING___' );

for ind = 1:NUM_METHODS
    szCurrentFilename = char( szDataFilename(ind) );
    
    try
        fprintf( sprintf( '%s...', szCurrentFilename ) );
        load( szCurrentFilename );
        fprintf( 'ok  ' );
        bLoadedFile(ind) = 1;

        sErr = sp_img_error( 'srestore', sRestore, 'xtrue', xtrue );

        if ~bInitProblemProp
            problemProp = struct( 'SZ_IMAGE_TYPE', SZ_IMAGE_TYPE, ...
                'SZ_PSF_TYPE', SZ_PSF_TYPE, ...
                'sigma', propNoise.sigma, ...                         
                'peakSnr', 20*log10(max( y0(:) )/propNoise.sigma), ...
                'nnz', sErr.trueNnz ...
                );
            bInitProblemProp = 1;
        end

        errorMetrics = [ errorMetrics; sErr.l0, sErr.numXorLoc, sErr.l1, sErr.lhard, sErr.lsoft, sErr.l2, nnz(sRestore.last) ];
    catch
        fprintf( 'ERROR!  ' );
        errorMetrics = [ errorMetrics; inf*ones( 1, NUM_ERROR_METRICS ) ];
    end
    
    if bLoadedFile(ind)
        % Plot sRestore.thetaMaxDiff
        try
            if bCreateConvergencePlot
                figure( hPlotConv ), semilogy( sRestore.thetaMaxDiff, char(szMarkers(ind)) ), hold on;
            end
        catch
            warning( sprintf( 'cannot plot thetaMaxDiff for file %s', szCurrentFilename ) );
            bLoadedFile(ind) = 0; % XXX: Hack
        end
        
        % Plot norm_1 error
        try
            if bCreateError1Plot
                figure( hPlotError1 ), semilogy( sRestore.errorNorm1, char(szMarkers(ind)) ), hold on;
            end
        catch
            warning( sprintf( 'cannot plot errorNorm1 for file %s', szCurrentFilename ) );
            bLoadedFile(ind) = 0; % XXX: Hack
        end
    end
end

disp('done');

% Check if *any* of the files were loaded. If not, then quit with error
if sum( bLoadedFile ) == 0
    error( 'did not load any files' );
end

% Create legends, if applicable
if bCreateConvergencePlot
    figure( hPlotConv ), legend( szMethods(find(bLoadedFile==1)), 0 ), title( 'convergence' );
end
if bCreateError1Plot
    figure( hPlotError1 ), legend( szMethods(find(bLoadedFile==1)), 0 ), title( 'norm1 error' );
end

%
% Figure out which is the best in terms of minimizing each error metric
%
[minError, minErrorInd] = min( errorMetrics );

disp( '___PROBLEM___' );
disp( ['image: ', problemProp.SZ_IMAGE_TYPE, ...
    ', psf: ', problemProp.SZ_PSF_TYPE, ...
    ', sigma: ', sprintf('%g',problemProp.sigma), ...
    ', peakSnr: ', sprintf('%g',problemProp.peakSnr), ...
    'dB, nnz: ', sprintf('%g',problemProp.nnz) ] ...
    );

disp( '___RESULTS___' );
disp( ['     ', sprintf( '%-12s', 'l0', 'pNnz', 'l1', 'pHard', 'pSoft', 'l2', 'nnz' )] );

for ind = 1:NUM_METHODS
    szRowHeader = sprintf( '%-5s', char(szMethods(ind)) );
    szResult = [];
    bWinner = (minErrorInd == ind);
    for metricInd = 1:NUM_ERROR_METRICS
        if bWinner(metricInd), szFormat = '%10.4e# ';
        else szFormat = '%10.4e  ';
        end        
        szResult = [szResult, sprintf(szFormat, errorMetrics(ind,metricInd))];
    end
    disp( [szRowHeader, szResult] );
end

