
% Framework for running reconstruction algorithms:
%    EM-EBD-L, EM-EBD-LA, EM-B-LA, EM-F-L
%
% Parts copied from recon_htest.m. This file is to be run after
% sp_img_recon. Refer to sp_img_recon2.m for more comments.

szDiaryFilename = 'goo';
diary off; % Make sure diary is turned off
if cdir_file_exists( szDiaryFilename ), delete( szDiaryFilename ), end;
diary( szDiaryFilename ); % Turn diary back on

%
% Variables that dictate the reconstruction
%

B_CONTINUE_SIM = 0;

NUM_EM_ITERS = 2e4;
GRADIENT_SEARCH_FAILURES_MAX = NUM_EM_ITERS;

SZ_M_FILE = 'emgauss_restore';
SZ_M_METHOD = 'map1';
SZ_THETA0_SELECTOR = 'Hty2';
THETA0 = [];
SZ_GAMMA0_SELECTOR = 'ones';
GAMMA0 = [];
PHI = [];
LAST_MUHAT = [];

wGiven = 1;
%wGiven = nnz( fClean(:) ) / length( fClean(:) );

l2b_n = 0;
rx_delta = 1e-2;
tSoft = 1/sqrt(3) * 2^l2b_n * rx_delta;
%tSoft = 1e-3;

% Handle G_SPECIAL
if ~exist( 'G_SPECIAL', 'var' ), disp( 'Initializing G_SPECIAL' )
    %G_SPECIAL = 1;
	%G_SPECIAL = 1/sqrt(2)/propImage.ampl; % Default value
    G_SPECIAL = 1/sqrt(2);
    %G_SPECIAL = 1e3;
else
    warning( sprintf( 'Using existing G_SPECIAL: %g', G_SPECIAL ) );
    disp( 'Pausing for 5s...' ), pause( 5 );
end

if B_CONTINUE_SIM

    % Back up old variables that would be over-written
    %-------------------------------------------------
    try
        sRestoreOld = sRestore;
    catch
        error( sprintf( 'B_CONTINUE_SIM=1, backing sRestore, %s', lasterr ) );
    end

    try
        sRestoreOptionsOld = sRestoreOptions;
    catch
        % Non-critical error
        warning( sprintf( 'B_CONTINUE_SIM=1, backing sRestoreOptions, %s', lasterr ) );
    end

    % Restore the state of the previous simulation
    %---------------------------------------------
    if strcmp( SZ_M_FILE, 'emgauss_restore' ) == 1
        try
            SZ_THETA0_SELECTOR = 'custom';
            THETA0 = sRestoreOld.last;
        catch
            error( sprintf( 'B_CONTINUE_SIM=1, adjusting theta0, %s', lasterr ) );
        end
    elseif strcmp( SZ_M_FILE, 'sbl_restore' ) == 1
        try
            SZ_GAMMA0_SELECTOR = 'custom';
            GAMMA0 = sRestoreOld.lastGam;
            PHI = sRestoreOld.phi;
            LAST_MUHAT = sRestoreOld.last;
        catch
            error( sprintf( 'B_CONTINUE_SIM=1, adjusting gamma0 & phi, %s', lasterr ) );
        end
    end

else

    % For SBL: load phi from a file?
    %-------------------------------
    if ( strcmp( SZ_M_FILE, 'sbl_restore' ) == 1 ) | ...
       ( strcmp( SZ_M_FILE, 'sbl_restore2' ) == 1 )
        %load( 'sbl_gaussian_psf.mat' );
        %load( '~/work/w06/tip/2d/benz/type_i/sbl_mrfm_psf2d64.mat' );
        load( '~/work/w06/tip/2d/1uud/type_i/sbl_mrfm_psf2d64.mat' );
        PHI = SBL_PHI;
        warning( 'sbl_restore: loading PHI from file' );
    end
    
end

if ~exist( 'psfZeroPadded', 'var' ), disp( 'Generating psfZeroPadded' )
    if ~exist( 'psf', 'var' ), error( 'psf does not exist?' ), end;
    if ndims( psf ) ~= 2, error( 'cannot handle dim of psf' ), end;
    
    % Dimension of psf is 2
    if ( size( psf, 1 ) == size( xtrue, 1 ) ) & ( size( psf, 2 ) == size( xtrue, 2 ) )
        psfZeroPadded = psf;
    else
        psfZeroPadded = zeros( size( xtrue ) );
        if ( size( psf, 1 ) <= size( xtrue, 1 ) ) & ( size( psf, 2 ) <= size( xtrue, 2 ) )
            psfZeroPadded( 1:size(psf,1), 1:size(psf,2) ) = psf;
        else
            error( 'size of psf is larger than xtrue??' );
        end
    end
end

tic; % Start timing

switch lower( SZ_M_FILE )
    case 'emgauss_restore'
        % Use non-zero padded psf
        if B_CONTINUE_SIM == 0
            [sRestore, sRestoreOptions] = emgauss_restore( 'forward_op', psf, ...
                'noisy_output', yi, ...
                'noise_sigma', propNoise.sigma, ...
                'theta_true', xtrue, ...
                'm_selector', SZ_M_METHOD, ...
                'num_em_iters', NUM_EM_ITERS, ...
                'num_max_grad_search_failures', GRADIENT_SEARCH_FAILURES_MAX, ...
                'theta0_selector', SZ_THETA0_SELECTOR, ...
                'theta0', THETA0, ...
                'a_val', 1e-2, ...
                'w_val', wGiven, ...
                't_val', -1, ...
                'sparse_ind', 'w', ...            
                'b_save_history', 0, ...
                'stopping_cond', 'fixed', ...
                'stopping_cond_tol', propNoise.sigma, ...
                'stopping_cond_win', 3e3, ...
                'stopping_cond_last_fac', 2, ...
                'thetastar', 1, ...
                'soft_thres_t', tSoft, ...
                'hparam_rest_interval', 1e3, ...
                'bseparate_hparam_est', 1, ...
                'gspecial', G_SPECIAL, ...
                'lambda', 998.909 ...
                );
        else
            [sRestore, sRestoreOptions] = emgauss_restore( 'forward_op', psf, ...
                'noisy_output', yi, ...
                'noise_sigma', propNoise.sigma, ...
                'theta_true', xtrue, ...
                'm_selector', SZ_M_METHOD, ...
                'num_em_iters', NUM_EM_ITERS, ...
                'num_max_grad_search_failures', GRADIENT_SEARCH_FAILURES_MAX, ...
                'theta0_selector', SZ_THETA0_SELECTOR, ...
                'theta0', THETA0, ...
                'a_val', 1e-2, ...
                'w_val', wGiven, ...
                't_val', -1, ...
                'sparse_ind', 'w', ...            
                'b_save_history', 0, ...
                'stopping_cond', 'fixed', ...
                'stopping_cond_tol', propNoise.sigma, ...
                'stopping_cond_win', 3e3, ...
                'stopping_cond_last_fac', 2, ...
                'thetastar', 1, ...
                'soft_thres_t', tSoft, ...
                'sboundingbox', sRestoreOptionsOld.sBoundingBox, ...
                'bcloseboundingbox', sRestoreOptionsOld.bCloseBoundingBox, ...
                'hparam_rest_interval', 1e2, ...
                'bseparate_hparam_est', 0 ...
                );
        end
        
    case 'sbl_restore'
        % Use zero-padded psf
        sRestore = sbl_restore( 'forward_op', psfZeroPadded, ...
            'noisy_output', yi(:), ...
            'noise_sigma', propNoise.sigma, ...
            'theta_true', xtrue(:), ...
            'num_em_iters', NUM_EM_ITERS, ...
            'stopping_cond', 'fixed', ...
            'stopping_cond_tol', propNoise.sigma, ...
            'stopping_cond_win', 5e3, ...
            'stopping_cond_last_fac', 5, ...
            'gamma0_selector', SZ_GAMMA0_SELECTOR, ...
            'gamma0', GAMMA0, ...
            'last_muhat', LAST_MUHAT, ...
            'phi', PHI ...
            );
        
    case 'sbl_restore2'
        % Use zero-padded psf
        sRestore = sbl_restore2( 'forward_op', psfZeroPadded, ...
            'noisy_output', yi(:), ...
            'noise_sigma', propNoise.sigma, ...
            'theta_true', xtrue(:), ...
            'num_em_iters', NUM_EM_ITERS, ...
            'stopping_cond', 'fixed', ...
            'stopping_cond_tol', propNoise.sigma, ...
            'stopping_cond_win', 5e3, ...
            'stopping_cond_last_fac', 5, ...
            'gamma0_selector', SZ_GAMMA0_SELECTOR, ...
            'gamma0', GAMMA0, ...
            'last_muhat', LAST_MUHAT, ...
            'phi', PHI ...
            );
        
    case 'dec15_xp1'
        % Use zero-padded psf
        sRestore = dec15_xp1( 'forward_op', psfZeroPadded, ...
            'noisy_output', yi(:), ...
            'noise_sigma', propNoise.sigma, ...
            'theta_true', xtrue(:), ...
            'u0', y0(:), ...
            'num_em_iters', NUM_EM_ITERS, ...
            'stopping_cond', 'fixed', ...
            'stopping_cond_tol', propNoise.sigma, ...
            'stopping_cond_win', 5e3, ...
            'stopping_cond_last_fac', 5, ...
            'gamma0_selector', SZ_GAMMA0_SELECTOR, ...
            'gamma0', GAMMA0, ...
            'last_muhat', LAST_MUHAT, ...
            'phi', PHI ...
            );

    case 'map_revised'
        % Use non-zero padded psf
        sRestoreOptions = struct( 'num_iters', 2e4, ...
            'step_size', 2e3, ...
            'bUseSigmaReal', 1 ...
            );
        [x_hat, a_hat, w_hat, sigma_hat] = map_revised( yi, ...
            psf, ...
            sRestoreOptions.num_iters, ...
            sRestoreOptions.step_size, ...
            propNoise.sigma, ...
            sRestoreOptions.bUseSigmaReal );
        sRestore = struct( 'last', x_hat, ...
            'hyperParamHist', [a_hat, w_hat, sigma_hat] );
        n = numel( yi );
        x_hat_vec = x_hat(:);
        switch nConvMethod
            case 1
                sRestore.cost = -logp_ytheta_cond_gauss( yi, Gfft, x_hat, sigma_hat ) - ...
                    n/2 * log(2*pi) - n * log( sigma_hat ) + ...
                    sum( abs(x_hat_vec) > 1e2*eps ) * log( (1-w_hat)/w_hat ) + ...
                    n*a_hat - a_hat*sum( abs(x_hat_vec) );
            case 2
                sRestore.cost = -logp_ytheta_cond_gauss( yi, Gconv, x_hat, sigma_hat ) - ...
                    n/2 * log(2*pi) - n * log( sigma_hat ) + ...
                    sum( abs(x_hat_vec) > 1e2*eps ) * log( (1-w_hat)/w_hat ) + ...
                    n*a_hat - a_hat*sum( abs(x_hat_vec) );
            otherwise
                sRestore.cost = -1;
        end
        
    otherwise
        error( 'unrecognized SZ_M_FILE' );
end

tElapsed = toc

if B_CONTINUE_SIM
    sRestoreConcat = sres_concat( sRestoreOld, sRestore, SZ_M_FILE );
end

diary off; % Turn diary off

