
% Script to analyze inner workings of the reconstruction methods.
% Usage is: sp_methods_dig( args )
% defined keys: 'method', 'filename'

function sp_methods_dig( varargin )

% Initialize

szMarkers = { 'b.', 'r.', 'm.', 'k-', 'c-' };
szDigMethod = '';
szDataFilename = '';
%alpha = -1;

% Process input arguments

lenVarargin = length( varargin );
numArgsIn = floor( lenVarargin/2 );

if 2*numArgsIn ~= lenVarargin, error( 'even ip args expected' ), end

for argInInd = 1:numArgsIn
    
    szKey = lower( varargin{2*argInInd-1} );    
    switch szKey
        case 'method', szDigMethod = varargin{2*argInInd};
        case 'filename', szDataFilename = varargin{2*argInInd}; 
        otherwise
            error( sprintf( 'unrecognized option %s', szKey ) );
    end
    
end

% Check arguments
if length( szDigMethod ) == 0, error( 'dig method unspecified' ), end;
% Construct the default data filename, if not specified
if length( szDataFilename ) == 0, szDataFilename = strcat( szDigMethod, '.mat'  ); end;

%close all;

try
    load( szDataFilename );
catch
    error( sprintf( 'Could not load data file %s', szDataFilename ) );
end

bRangeAvail = zeros( 1,2 );

switch lower( szDigMethod )
    case { 'eb_better','map_prior','eb_fix_w', 'eb_fixed' }
        
        figure( 1 ), clf; % Figure of EB estimates
        hWin = 310;
        
        subplot( hWin+1 ), plot( sRestore.a, char(szMarkers(1)) );
        if length( sRestore.a ) >= 1
            title( sprintf( 'a estimate, mean=%g', mean(sRestore.a) ) );
        end
        
        try
            if ( min( size( sRestore.aRange ) ) >= 1 )
                hold on;
                plot( sRestore.aRange(:,1), 'k-' );
                plot( sRestore.aRange(:,2), 'k-' );
                bRangeAvail(1) = 1;
            else
                disp( 'WARNING: aRange empty' );
            end
        catch
            disp( 'WARNING: aRange not defined' );
        end
        
        subplot( hWin+2 ), plot( sRestore.t, char(szMarkers(2)) );
        if length( sRestore.t ) >= 1
            title( sprintf( 't estimate, mean=%g', mean(sRestore.t) ) );
        end

        try
            if ( min( size( sRestore.tRange ) ) >= 1 )
                hold on;
                plot( sRestore.tRange(:,1), 'k-' );
                plot( sRestore.tRange(:,2), 'k-' );
                bRangeAvail(2) = 1;
            else
                disp( 'WARNING: tRange empty' );
            end
        catch
            disp( 'WARNING: tRange not defined' );
        end
        
        subplot( hWin+3 ), plot( sRestore.w, char(szMarkers(3)) ); 
        if length( sRestore.w ) >= 1
            title( sprintf( 'w estimate, mean=%g', mean(sRestore.w) ) );
        end
      
        figure( 2 ), clf; % Figure of convergence criteria and error norm 1+2
        hWin = 210;
        subplot( hWin+1 ), semilogy( sRestore.thetaMaxDiff ), title( 'thetaMaxDiff' );
        subplot( hWin+2 ), semilogy( sRestore.errorNorm1, 'r' ), title( 'errorNorm1+2' );
        hold on, semilogy( sRestore.errorNorm2, 'g' ), 
        legend( 'errorNorm1', 'errorNorm2', 0 );
                
        figure( 3 ), clf; % Figure of joint prob
        if length( sRestore.jointProb ) >= 1
            plot( real(sRestore.jointProb) ), title( 'jointProb' ), hold on;
            plot( [1,length(sRestore.jointProb)], [sRestore.jointProbReal,sRestore.jointProbReal], 'k:' );
        end
        
        figure( 4 ), clf; % Time series comparison
        stem( xtrue(:), 'ko' ), hold on;
        stem( sRestore.last(:), 'bx' ), title( 'time series comparison' );
        legend( 'xtrue', 'last', 0 );
        
    case 'sbl'
        
        figure( 1 ), clf; % Time series comparison
        stem( xtrue(:), 'ko' ), hold on;
        stem( sRestore.last(:), 'bx' ), title( 'time series comparison' );
        legend( 'xtrue', 'last', 0 );

        figure( 2 ), clf; % Gamma estimates
        stem( sqrt(sRestore.lastGam(:)), 'ms' ), title( 'sqrt(gamma) estimate' );

        figure( 3 ), clf; % Figure of convergence criteria and error norm 1+2
        hWin = 210;
        subplot( hWin+1 ), semilogy( sRestore.thetaMaxDiff ), title( 'thetaMaxDiff' );
        subplot( hWin+2 ), semilogy( sRestore.errorNorm1, 'r' ), title( 'errorNorm1+2' );
        hold on, semilogy( sRestore.errorNorm2, 'g' ),
        legend( 'errorNorm1', 'errorNorm2', 0 );
        
    case 'landweb'
        
        figure( 1 ), clf; % Figure of convergence criteria and error norm 1+2
        hWin = 210;
        subplot( hWin+1 ), semilogy( sRestore.thetaMaxDiff ), title( 'thetaMaxDiff' );
        subplot( hWin+2 ), semilogy( sRestore.errorNorm1, 'r' ), title( 'errorNorm1+2' );
        hold on, semilogy( sRestore.errorNorm2, 'g' ),
        legend( 'errorNorm1', 'errorNorm2', 0 );

        figure( 2 ), clf; % Time series comparison
        stem( xtrue(:), 'ko' ), hold on;
        stem( sRestore.last(:), 'bx' ), title( 'time series comparison' );
        legend( 'xtrue', 'last', 0 );
        
    otherwise
        error( 'unrecognized method' );
end
        