
% sres_concat: concatenate the results of simulations
% Usage: sRestore12 = sres_concat( sRestore1, sRestore2, szMFile )

function sRestore12 = sres_concat( sRestore1, sRestore2, szMFile )

bSuccess = 0;

switch lower( szMFile )
    case 'emgauss_restore'
        
        % Old structure
        try
            sRestore12 = struct( 'last', sRestore2.last, 'init', sRestore1.init, ...
                't', vect_concat_horz(sRestore1.t, sRestore2.t(2:end)), ...
                'a', vect_concat_horz(sRestore1.a, sRestore2.a(2:end)), ...
                'w', vect_concat_horz(sRestore1.w(:), sRestore2.w(2:end)), ...
                'aRange', [sRestore1.aRange; sRestore2.aRange(2:end,:)], ...
                'tRange', [sRestore1.tRange; sRestore2.tRange(2:end,:)], ...
                'thetaStar', vect_concat_horz(sRestore1.thetaStar, sRestore2.thetaStar(2:end)), ...
                'thetaMaxDiff', vect_concat_horz(sRestore1.thetaMaxDiff, sRestore2.thetaMaxDiff(2:end)), ...
                'thetaSumDiff', vect_concat_horz(sRestore1.thetaSumDiff, sRestore2.thetaSumDiff(2:end)), ...
                'errorNorm1', vect_concat_horz(sRestore1.errorNorm1, sRestore2.errorNorm1(2:end)), ...
                'errorNorm2', vect_concat_horz(sRestore1.errorNorm2, sRestore2.errorNorm2(2:end)), ...
                'jointProb', vect_concat_horz(sRestore1.jointProb, sRestore2.jointProb(2:end)), ...
                'jointProbReal', sRestore2.jointProbReal, ...
                'cost', [], ...
                'cputime', [sRestore1.cputime; sRestore2.cputime] ...
                );
            bSuccess = 1;
        catch
            warning( 'sRestore{1,2} not old structures (emgauss_restore)' );
            sRestore12 = struct( 'szError', lasterr );
        end
        
        if bSuccess, return, end;
        
        % New structure
        try
          sRestore12 = struct( 'last', sRestore2.last, 'init', sRestore1.init, ...
                'hyperParamHist', [sRestore1.hyperParamHist; sRestore2.hyperParamHist(2:end,:)], ...
                'hyperParamRangeHist', [sRestore1.hyperParamRangeHist; sRestore2.hyperParamRangeHist(2:end,:)], ...
                'thetaMaxDiff', vect_concat_horz(sRestore1.thetaMaxDiff, sRestore2.thetaMaxDiff(2:end)), ...
                'thetaSumDiff', vect_concat_horz(sRestore1.thetaSumDiff, sRestore2.thetaSumDiff(2:end)), ...
                'errorNorm1', vect_concat_horz(sRestore1.errorNorm1, sRestore2.errorNorm1(2:end)), ...
                'errorNorm2', vect_concat_horz(sRestore1.errorNorm2, sRestore2.errorNorm2(2:end)), ...
                'jointProb', vect_concat_horz(sRestore1.jointProb, sRestore2.jointProb(2:end)), ...
                'jointProbReal', sRestore2.jointProbReal, ...
                'cost', [], ...
                'cputime', [sRestore1.cputime; sRestore2.cputime] ...
                );
            bSuccess = 1;

        catch
            warning( 'sRestore{1,2} not new structures either (emgauss_restore)' );
            sRestore12.szError2 = lasterr;
        end
            
    case 'sbl_restore'
        try
            sRestore12 = struct( 'last', sRestore2.last, ...
                'initGam', sRestore1.initGam, 'lastGam', sRestore2.lastGam, ...
                'thetaMaxDiff', vect_concat_horz(sRestore1.thetaMaxDiff, sRestore2.thetaMaxDiff), ...
                'thetaSumDiff', vect_concat_horz(sRestore1.thetaSumDiff, sRestore2.thetaSumDiff), ...
                'errorNorm1', vect_concat_horz(sRestore1.errorNorm1, sRestore2.errorNorm1), ...
                'errorNorm2', vect_concat_horz(sRestore1.errorNorm2, sRestore2.errorNorm2), ...
                'cost', [sRestore1.cost; sRestore2.cost], ...
                'cputime', [sRestore1.cputime; sRestore2.cputime] ...
                );
            
            % Save the system matrix
            if isempty( sRestore1.phi )
                if isempty( sRestore2.phi )
                    warning( 'sRestore1.phi = sRestore2.phi = []!' );
                    sRestore12.phi = [];
                else
                    warning( 'sRestore1.phi = [], but sRestore2.phi ~= []!' );
                    sRestore12.phi = sRestore2.phi;
                end
            else
                sRestore12.phi = sRestore1.phi;
            end
            
        catch
            warning( 'could not concat sim results (sbl_restore)' );
            sRestore12 = struct( 'szError', lasterr );
        end
        
    otherwise
        error( 'invalid szMFile' );
end

function v = vect_concat_horz( v1, v2 )
v = [v1(:)', v2(:)'];
