% visual3: Plots 3-d volume scalar data
%
% visual3( X, Y, Z, V, szPlotType, sOptions )
%
%    X,Y,Z - Matrices generated by gen_img
%    V - Matrix of volume scalar data to plot 
%    szPlotType - Type of plot to create. Valid values are: 'slice', 'isosurf'
%       'cslice'
%    sOptions - Structure of options. The following are fields might be used
%       under the values of szPlotType:
%       'slice': bSpecSlices, xSlice, ySlice, zSlice
%       'isosurf': bPlotHistOnly
%

function visual3( X, Y, Z, V, szPlotType, sOptions )

global Xfix Yfix Zfix

global m n p

% Initialize
bAxisGiven = 0;
[m,n,p] = size( V );
Xfix = [];
Yfix = [];
Zfix = [];

if ~isempty( X ) & ~isempty( Y ) & ~isempty( Z ), bAxisGiven = 1; end;

% If given, must fix X, Y, Z, V, so it appears like they were generated by
% meshgrid.

if bAxisGiven
% 	Xfix = zeros( [n,m,p] );
% 	Yfix = zeros( size( Xfix ) );
% 	Zfix = zeros( size( Xfix ) );
% 	Vfix = zeros( size( Xfix ) );
% 
% 	for i = 1:p
% 		Xfix( :, :, i ) = X( :, :, i )';
% 		Yfix( :, :, i ) = Y( :, :, i )';
% 		Zfix( :, :, i ) = Z( :, :, i )';
% 		Vfix( :, :, i ) = V( :, :, i )';
% 	end
    Xfix = X;
    Yfix = Y;
    Zfix = Z;
    Vfix = V;
else
	[Xfix, Yfix, Zfix] = meshgrid( 1:n, 1:m, 1:p );
	Vfix = V;
end

switch ( lower(szPlotType) )
case 'slice'

	figure( 1 ), clf;

	if ( ~isempty( sOptions ) & ( sOptions.bSpecSlices == 1 ) )
		slice( Xfix, Yfix, Zfix, Vfix, sOptions.xSlice, sOptions.ySlice, ...
			sOptions.zSlice );
	else
		%xSlice = Xfix( 1, round(0.1*m), 1 );
		xSlice = [  64  ];
		%ySlice = min( Yfix( :, 1, 1 ) );
		ySlice = [ 110 ];
		%zSlice = [Zfix( 1, 1, round(1/3*p) ), Zfix( 1, 1, round(2/3*p) )];
		%zSlice = Zfix( 1, 1, round(0.3*p) );
        zSlice = [ 4 ];
		slice( Xfix, Yfix, Zfix, Vfix, xSlice, ySlice, zSlice );
	end

	colormap( jet( 256 ) );
	set_graph_settings;

case 'isosurf'

	% Plot the histogram of values in V
	numBins = 100;
	[binCount, binPos] = hist( Vfix(:), numBins );
	binPercent = binCount / sum( binCount );
	figure( 1 ), clf, subplot( 211 ), bar( binPos, binPercent );

	% Build the cumulative distribution of the amplitudes
	binCumPercent = [];
	for i = 1:numBins, binCumPercent(i) = sum( binPercent(1:i) ); end;
	subplot( 212 ), plot( binPos, binCumPercent, 'r*-' );
	
	cumPercentPlot = [ 0.95, 0.99, 0.999 ];
	vecIsoVal = zeros( size( cumPercentPlot ) );
	for i = 1:length( cumPercentPlot )
		ind = min( find( binCumPercent >= cumPercentPlot(i) ) );
		vecIsoVal(i) = binPos( ind );
	end

	% DEBUG 
	if ( ~isempty( sOptions ) & ( sOptions.bPlotHistOnly == 1 ) )
		return;
	end

	if ( vecIsoVal(end) < binPos( round(0.8*numBins) ) )
		vecIsoVal = [ vecIsoVal, binPos( round(0.8*numBins) ) ];
	end

	for isoVal = vecIsoVal
		disp( sprintf( 'isoVal = %g', isoVal ) );
		figure( 2 ), clf, isosurface( Xfix, Yfix, Zfix, Vfix, isoVal );
		set_graph_settings;
		disp( 'Press a key to continue...' ), pause;
	end

case 'cslice'

	% Plot the histogram of values in V
	[binCount, binPos] = hist( Vfix(:), 1e2 );
    binProp = binCount / sum( binCount );
    indPos = find( binPos > 0 );    
	figure( 1 ), clf, semilogy( binPos(indPos), binProp(indPos), 'b.', 'MarkerSize', 10 );

	binPosInd = find( binPos >= 0 );
	zSlice = Zfix(1,1,:);
	%isoVals = linspace( binPos( binPosInd(1) ), binPos( binPosInd(end) ), 10 );
	%isoVals = linspace( binPos( 1 ), binPos( end ), 5 );
	VfixMax = max( Vfix(:) );
	disp( sprintf( 'VfixMax=%g', VfixMax ) );
	%isoVals = [1, (1+VfixMax)/2];
    %isoVals = [ 1, 0.8, 1.2, 1.5 ];
    %isoVals = [ 1, 2, 3, 4, 5, 6, 7, 8, 9 ];
    %isoVals = logspace( -3, 0, 5 );
    isoVals = logspace( -3, log10( VfixMax ), 5 );
    isoVals = sort( isoVals )

	figure( 2 ), clf;
	%h = contourslice( Xfix, Yfix, Zfix, Vfix, [], [], zSlice, isoVals );
    h = contourslice( Vfix, [], [], zSlice, isoVals );
	%set_graph_settings;

otherwise
	error( 'Unrecognized plot type' );
end

% ................................................................
% set_graph_settings: Sets the graph settings for 3d visualization
% ################################################################

function set_graph_settings

global Xfix Yfix Zfix

global m n p

% Camera settings
%campos( [ Xfix( 1, 10, 1 ), Yfix( 1, 1, 1 ), Zfix( 1, 1, end ) ] );
camva( 'auto' ); 
camproj orthographic;
% Colour settings
set( gcf, 'Renderer', 'zbuffer' );
%set( gca, 'Color', 'black', 'XColor', 'white', 'YColor', 'white', ...
%	'ZColor', 'white' );
%xlabel( 'x [nm]' ), ylabel( 'y [nm]' ), zlabel( 'z [nm]' );
% Grid
%axis tight; 
% axis( [ Xfix(1,1,1), Xfix(1,end,1), ...
% 	Yfix(1,1,1), Yfix(end,1,1),...
% 	Zfix(1,1,1), Zfix(1,1,round(p/2)) ] );
daspect( [1, 1, 1] );
box on;
refresh;


