% bmag_lev_surf2: same as bmag_lev_surf, but we only care about z > R0
%-------------------------------------------------------------------------------
% Plots the level surface of Bmag(x,y,z) for z > R0
%
% sRes = bmag_lev_surf2( Bext, c, m, R0 )
%    Bext, m - parameters of the experiment
%    c       - value of Bmag^2 for which we want to plot the level surface. 
%              Note that Bmag^2 >= Bext^2
%
function sRes = bmag_lev_surf2( Bext, c, m, R0 )

[meshTheta, meshPhi, meshR] = bmag_lev_surf( Bext, c, m, 1 );

% Convert to Cartesian coords (x,y,z)
meshX = meshR .* sin( meshPhi ) .* cos( meshTheta );
meshY = meshR .* sin( meshPhi ) .* sin( meshTheta );
meshZ = meshR .* cos( meshPhi );

% Now that we have the surface for Z > 0, we want to only plot the part of
% the surface which satisfies Z > R0. Note that it could very well be that
% no points satisfy this previous condition.
[goodRow, goodCol] = find( meshZ > R0 );
if ( length( goodRow ) == 0 | length( goodCol ) == 0 )
	error( 'no mesh points with z > R0' );
end

% Iterate over the "good" points and find the max phi value. Note that this 
% technique is not very efficient.
maxPhiValue = 0;
for i = 1:length( goodRow )
	for j = 1:length( goodCol )
		tmp = meshPhi( goodRow(i), goodCol(j) );
		if ( tmp > maxPhiValue ), maxPhiValue = tmp; end;
	end
end

disp( sprintf( 'maxPhiValue = %.3e', maxPhiValue ) );

% Call bmag_lev_surf a second time, but with a constraint on the max phi
% value
[meshTheta, meshPhi, meshR] = bmag_lev_surf( Bext, c, m, 1, maxPhiValue );

sRes = struct( 'theta', meshTheta, 'phi', meshPhi, 'r', meshR );

% Convert to Cartesian coords (x,y,z-R0)
meshX = meshR .* sin( meshPhi ) .* cos( meshTheta );
meshY = meshR .* sin( meshPhi ) .* sin( meshTheta );
meshZNorm = meshR .* cos( meshPhi ) - R0; % normalized z

create_surf_plot( 1, '', [], meshX, meshY, meshZNorm, 1, 'z - R_0 [nm]' );

sRes.meshX = meshX;
sRes.meshY = meshY;
sRes.meshZNorm = meshZNorm;

% Compute the largest value of (z-R0)
meshZNormMax = max( max( meshZNorm ) );
disp( sprintf( 'max z-R0 = %.6e', meshZNormMax ) );
