% bnb_reorder2: reorder A and x0 before using in bnb_tst [version2]
%-------------------------------------------------------------------------------
% Here, reorder x0. Then, decide which elements of x0 are 0s and which ones 
% are 1s. Specifically, if any element of x0 is < 0, we set that to 0, and if 
% any element of x0 is > 1, we set that to 1. We leave everything in (0,1) as 
% they are. 

% [newA, newX0, T] = bnb_reorder2( oldA, oldX0 )
%    old{A,X0} - old A and x0
%
% Outputs
%    new{A,X0} - new A and x0
%    T         - the transformation matrix where newX0 = T * oldX0
%
function [newA, newX0, T] = bnb_reorder2( oldA, oldX0 )

n = length( oldX0 );
[newX0, ind] = sort( oldX0 );

% Go through x0 and cut off any values outside [0,1]
numCutVals = 0;
for i = 1:n
	if ( newX0(i) < 0 )
		newX0(i) = 0;
		numCutVals = numCutVals + 1;
	elseif ( newX0(i) > 1 )
		newX0(i) = 1;
		numCutVals = numCutVals + 1;
	end
end

disp( sprintf( 'bnb_reorder2: number of cut vals = %4d/%4d', ...
	numCutVals, n ) );

T = zeros( n, n );
for i = 1:n
	T( i, ind(i) ) = 1;
end
newA = T * oldA * T';

