% f_bstr_sd: uses steepest descent on binary strings
%-------------------------------------------------------------------------------
% fHat = f_bstr_sd( par, stMesh, h, g )
%
function fHat = f_bstr_sd( par, stMesh, Argh, Argg )

global ATA fML

% Get the maximum likelihood estimate first
arfML = f_deconv( stMesh, Argh, Argg );

% Get the matrix A
[Arow, Acol, Aval] = mex_A_matrix( par.Bext, par.Bres, par.m, par.xPk, ...
	[stMesh.numXPts, stMesh.numYPts, stMesh.numZPts], ...
	[stMesh.deltaX, stMesh.deltaY, stMesh.deltaZ], ...
	stMesh.vecLowerLeftCoords );

L = stMesh.numXPts * stMesh.numYPts * stMesh.numZPts;
A = sparse( Arow, Acol, Aval, L, L );

% Reshape matrices into vectors
h = reshape( Argh, [L, 1] );
g = reshape( Argg, [L, 1] );
fML = reshape( arfML, [L, 1] );

% Pre-compute these
ATA = A'*A;

% Formulate an initial guess
%fHat0 = bstr_bin_test( fML );
fHat0 = zeros( [L, 1] );

% DEBUG
disp( 'size arfML:' ), disp( size( arfML ) );
disp( 'size h:' ), disp( size( h ) );
disp( 'size g:' ), disp( size( g ) );
disp( 'size fML:' ), disp( size( fML ) );
disp( 'size fHat0:' ), disp( size( fHat0 ) );

vecfHat = bstr_steepest_desc( 8, fHat0, 1, 'notgreedy', @phi, @phiRowGrad );
fHat = reshape( vecfHat, [stMesh.numXPts, stMesh.numYPts, stMesh.numZPts] );

%...............................................................................
% The objective function to minimize, which is:
% phi( f ) = (f - fML)' * (A'*A) (f - fML) = | A(f - fML) |^2
%...............................................................................
function rc = phi( f )
global ATA fML
rc = ( f - fML )' * ATA * ( f - fML );

%...............................................................................
% The row gradient of the objective function above
%...............................................................................
function rc = phiRowGrad( f )
global ATA fML
rc = 2 * ( f - fML )' * ATA; 
