% f_conls: performs constrained least-squared; derived from h_weiner
%-------------------------------------------------------------------------------
% See IV.B of the following ref:
%    J. Biemond, R. L. Lagendijk, R. M. Mersereau, "Iterative Methods for
%    Image Deblurring", Proc. of IEEE, Vol. 78, No. 5, 1990, pp 856-883.
%
% We use the discrete approximation to the 3d Laplacian for C.
%
% fHat = f_conls( stMesh, h, g, gamma )
%    stMesh - mesh structure
%    h, g   - output of gen_img
%    gamma  - tunable parameter
%
% Output
%    fHat   - estimated image
%
function fHat = f_conls( stMesh, h, g, gamma )

psf_mesh_chk( stMesh );

% Pad the fft so that the length in each dimension is a power of 2. 
fftSize = pad2_fft_size( [stMesh.numPts(1), stMesh.numPts(2), stMesh.numPts(3)], 0 );

H = fftn( h, fftSize );
[row, col] = find( abs( H ) < eps );
if ( length( row ) > 0 ), error( 'H has zeros, bailing out...' ), end;

% Construct the estimation filter
if 0
    [I, J, K] = size( h );
else
    I = fftSize( 1 );
    J = fftSize( 2 );
    K = fftSize( 3 );
end

C = zeros( [I, J, K] );
for k1 = 1:I
	for k2 = 1:J
		for k3 = 1:K
			C( k1, k2, k3 ) = -6 + 2*cos( 2*pi/I*k1 ) + ...
				2*cos( 2*pi/J*k2 ) + 2*cos( 2*pi/K*k3 );
		end
	end
end

if 0 % DEBUG
    whos H C
end

num = conj( H );
denom = abs( H ).^2 + gamma * abs( C ).^2;
estFilt = num ./ denom;

G = fftn( g, fftSize );
F = G .* estFilt;
fLarge = ifftn( F );
fHat = fLarge( 1:stMesh.numPts(1), 1:stMesh.numPts(2), 1:stMesh.numPts(3) );

