% f_deconv: performs a deconvolution using the vertical cantilever PSF
%-------------------------------------------------------------------------------
% Given that we know h and g, use deconvolution to get the estimated f.
%
% fHat = f_deconv( stMesh, h, g )
%    stMesh - mesh structure
%    h, g   - output of gen_img
%
% Output
%    fHat       - estimated image
%
function fHat = f_deconv( stMesh, h, g )

NORM_THRES = 100*eps;

psf_mesh_chk( stMesh );

% Pad the fft so that the length in each dimension is a power of 2. 
fftSize = pad2_fft_size( [stMesh.numPts(1), stMesh.numPts(2), stMesh.numPts(3)], 0 );

H = fftn( h, fftSize );
HSize = size( H );
HInv = zeros( HSize );

HSizeLen = length( HSize );

switch ( HSizeLen )
case 2 % only 2d
	for i = 1:HSize(1)
		for j = 1:HSize(2)
			if ( abs( H(i,j) ) > NORM_THRES )
				HInv( i, j ) = 1 / H( i, j );
			end
		end
	end

case 3 % 3d
	for i = 1:HSize(1)
		for j = 1:HSize(2)
			for k = 1:HSize(3)
				if ( abs( H(i,j,k) ) > NORM_THRES )
					HInv( i, j, k ) = 1 / H( i, j, k );
				end
			end
		end
	end

otherwise
	error( 'unexpected HSizeLen' );
end

G = fftn( g, fftSize );
F = G .* HInv;
fLarge = ifftn( F );

if ( HSizeLen == 2 )
	fHat = fLarge( 1:stMesh.numPts(1), 1:stMesh.numPts(2) );
else
	fHat = fLarge( 1:stMesh.numPts(1), 1:stMesh.numPts(2), 1:stMesh.numPts(3) );
end
