% f_me: penalized likelihood with maximum entropy
%-------------------------------------------------------------------------------
% [fHat, psi, exitFlag, output] = f_me( par, stMesh, h, g, lambda )
%
function [fHat, psi, exitFlag, output] = f_me( par, stMesh, Argh, Argg, ...
	ArgLambda )

global ATA ATg lambda g2 row col val

% Obtain A
[Arow, Acol, Aval] = mex_A_matrix( par.Bext, par.Bres, par.m, par.xPk, ...
	[stMesh.numXPts, stMesh.numYPts, stMesh.numZPts], ...
	[stMesh.deltaX, stMesh.deltaY, stMesh.deltaZ], ...
	stMesh.vecLowerLeftCoords );

L = stMesh.numXPts * stMesh.numYPts * stMesh.numZPts;
A = sparse( Arow, Acol, Aval, L, L );

h = reshape( Argh, [L, 1] );
g = reshape( Argg, [L, 1] );

% Pre-compute these
lambda = ArgLambda;
ATA = A'*A;
ATg = A'*g;
g2 = sum( g.^2 );

% Initial fHat
fHat0 = 0.5*ones( [L, 1] );

% Set optimization options
optimOptions = optimset( 'GradObj', 'on', 'Hessian', 'on', ...
	'LargeScale', 'on', ...
	'Diagnostics', 'on', ...
	'Display', 'final', ...
	'MaxFunEvals', 1e3, 'MaxIter', 1e3, ...
	'TolFun', 1e-10, 'TolX', 1e-10, ...
	'HessUpdate', 'bfgs' );

[fHat, psi, exitFlag, output] = fminunc( @fnPsi, fHat0, optimOptions );

%...............................................................................
% The objective function to minimize. Optionally return the gradient and
% Hessian if the appropriate number of output arguments is specified
%...............................................................................
function [psi, psiGrad, psiHess] = fnPsi( fHat )
global ATg lambda g2 row col val

% Always compute the value of the function
psi = g2 - 2 * fHat' * ATg + fHat' * ATA * fHat;

if ( nargout > 1 )
	psiGrad = 2*( ATA*fHat - ATg ) + lambda * log( fHat );
	if ( nargout > 2 )
		tmp = ones( size( fHat ) ) ./ fHat;
		psiHess = 2*ATA + lambda * diag( tmp );
	end
end

