% f_weiner: performs Weiner filtering (ASSUMES AWGN)
%-------------------------------------------------------------------------------
% Use Weiner filtering. Here, we are assuming that the noise is WGN. Moreoever,
% we are assuming that f is well modelled by a Gaussian distribution with
% zero mean. NOTE that this is a flawed assumption.
%
% fHat = f_weiner( stMesh, h, g, sigma, beta )
%    stMesh - mesh structure
%    h, g   - output of gen_img
%    sigma  - noise std deviation
%    beta   - tunable parameter
%
% Output
%    fHat   - estimated image
%
function fHat = f_weiner( stMesh, h, g, sigma, beta )

psf_mesh_chk( stMesh );

% Pad the fft so that the length in each dimension is a power of 2. 
fftSize = pad2_fft_size( [stMesh.numXPts, stMesh.numYPts, stMesh.numZPts], 1 );

H = fftn( h, fftSize );
[row, col] = find( abs( H ) < eps );
if ( length( row ) > 0 ), error( 'H has zeros, bailing out...' ), end;

% Construct the Weiner filter
num = conj( H );
denom = abs( H ).^2 + beta*sigma^2;
weinerFilt = num ./ denom;

G = fftn( g, fftSize );
F = G .* weinerFilt;
fLarge = ifftn( F );
fHat = fLarge( 1:stMesh.numXPts, 1:stMesh.numYPts, 1:stMesh.numZPts );
