% find_dist_minmax: Finds the smallest and largest inter spin distance
%
% [smallestDist, largestDist] = find_dist_minmax( spinPos )
%

function [smallestDist, largestDist] = find_dist_minmax( spinPos )

spinPosSize = size( spinPos );
numSpins = spinPosSize( 1 );
if ( (numSpins <= 1) | (spinPosSize( 2 ) ~= 3) ) 
	error( 'unexpected input for spinPos' );
else
	% Initialize
	smallestDistSquared = ( spinPos(1,1) - spinPos(2,1) )^2 + ...
		( spinPos(1,2) - spinPos(2,2) )^2 + ( spinPos(1,3) - spinPos(2,3) )^2;
	largestDistSquared = smallestDistSquared;
	% Iterate over pairs (i1,i2), i1 ~= i2
	for ind1 = 1:numSpins
		for ind2 = 1:numSpins
			if ( ind2 == ind1 ), continue, end
			dx = spinPos( ind1,1 ) - spinPos( ind2,1 );
			dy = spinPos( ind1,2 ) - spinPos( ind2,2 );
			dz = spinPos( ind1,3 ) - spinPos( ind2,3 );
			distSquared = dx^2 + dy^2 + dz^2;
			if ( distSquared < smallestDistSquared )
				smallestDistSquared = distSquared;
			end
			if ( distSquared > largestDistSquared )
				largestDistSquared = distSquared;
			end
		end
	end
	% Check that squared distances are non-negative
	if ( (smallestDistSquared < 0) | (largestDistSquared < 0) )
		error( 'negative squared distance' );
	end
	smallestDist = sqrt( smallestDistSquared );
	largestDist = sqrt( largestDistSquared );
end

