% get_A_matrix: get the operator matrix A
%-------------------------------------------------------------------------------
% The convolution g = h*f can be represented in matrix-vector format as y=Ax.
% We return the matrix A. Note that this will be a large matrix with a lot of
% zeros, so we return a sprase matrix.
%
% Note that an alternative is mex_A_matrix, which is faster.
%
% A = get_A_matrix( par, stMesh )
%    par     - parameter structure
%    stMesh  - mesh structure
%
% Output
%    A       - (sparse ) A matrix
%
function [X,Y,Z,A] = get_A_matrix( par, stMesh )

% Constants
DOUBLE_THRES = 1e2*eps;

psf_mesh_chk( stMesh );

I = stMesh.numXPts;
J = stMesh.numYPts;
K = stMesh.numZPts;

L = I * J * K;

X = zeros( L, L );
Y = zeros( L, L );
Z = zeros( L, L );

disp( 'Constructing matrices X, Y, Z...' );

for rowInd = 1:L
	for colInd = 1:L
		[i, j, k] = ind_matrix_trans( I, J, K, rowInd, colInd );

		% Replace the linear convolution with a circular convolution. Note that
		% this is an approximation.
		i = mod( i, I );
		j = mod( j, J );
		k = mod( k, K );

		[x, y, z] = ind_cart_d2c( stMesh, i, j, k );
		X( rowInd, colInd ) = x;
		Y( rowInd, colInd ) = y;
		Z( rowInd, colInd ) = z;
	end
end


disp( 'Calling mex_psf_vert...' );
[nStatus, Bmag, G, s, P] = mex_psf_vert( X, Y, Z, par.Bext, par.Bres, ...
	par.m, par.xPk, 0 );

disp( 'Converting to sparse matrix...' );
P = ( abs( P ) > DOUBLE_THRES ) .* P;
%A = sparse( P );
A = P;

