% get_h_op: get the h operator for indices in the range of f, g
%-------------------------------------------------------------------------------
% Gets the h (forward) operator for the vertical cantilever.
%
% {{{ History }}}
%
% 06-aug-04: modified to work with mex_psf_vert. In fact, psf_vert can be
% replaced in all instances by mex_psf_vert.
%
% h = get_h_op( par, stMesh )
%    par     - parameter structure
%    stMesh  - mesh structure
%
% Outputs
%    h       - operator (a 3-d matrix)
%
function h = get_h_op( par, sMesh )

% Constants
XY_MESH_ALL_ONES = ones( sMesh.numPts(1), sMesh.numPts(2) );
XY_MESH_ALL_ZEROS = zeros( size( XY_MESH_ALL_ONES ) );

h = zeros( sMesh.numPts );
meshX = XY_MESH_ALL_ZEROS;
meshY = XY_MESH_ALL_ZEROS;

% Typically, the size of the z-dir is smaller than either x- or y-. So 
% call mex_psf_vert with z fixed.

for i = 1:sMesh.numPts(1)
	for j = 1:sMesh.numPts(2)
		[x, y, z] = ind_cart_d2c( sMesh, i, j, 1 );
		meshX( i, j ) = x;
		meshY( i, j ) = y;
	end
end

for k = 1:sMesh.numPts(3)
	[x, y, z] = ind_cart_d2c( sMesh, 1, 1, k );
	% DEBUG
	%disp( sprintf( 'k = %3d, z = %.3e', k, z ) );
	[nStatus, Bmag, G, s, P] = mex_psf_vert( meshX, meshY, ...
		z*XY_MESH_ALL_ONES, par.Bext, par.Bres, par.m, par.xPk, 0 );
	h( :, :, k ) = P;
end	 



