% get_int_bbox: determine the "bounding box" for an intensity plot
%-------------------------------------------------------------------------------
% We assume that the image is centred around the origin.
%
% [llRow, llCol, urRow, urCol] = get_int_box( P, thresMin ) 
%    P        - matrix of intensity values
%    thresMin - threshold used to determine which part of the plot we're
%               interested in
%
% Outputs
%    ll{Row,Col} - lower left row and column indices
%    ur{Row,Col} - upper right ... ...
%
function [llRow, llCol, urRow, urCol] = get_int_bbox( P, thresMin )

PSize = size( P );
numRows = PSize( 1 );
numCols = PSize( 2 );

PMin = min( min( P ) );
PMax = max( max( P ) );
thresMinValue = PMin + thresMin * ( PMax - PMin );
[row, col] = find( P > thresMinValue );

% Now, compute the bounding box
llRow = max( row );
llCol = min( col );
urRow = min( row );
urCol = max( col );

% Check that the bounding box values are ok
if ( llRow <= urRow ), error( 'llRow <= urRow' ), end;
if ( llCol >= urCol ), error( 'llCol >= urCol' ), end;

if 1 % DEBUG
	disp( 'get_int_bbox -->' );
	disp( sprintf( '   original dimensions: %5d x %5d', numRows, numCols ) );
	disp( sprintf( '      new bounding box: (%5d,%5d) - (%5d,%5d)', ...
		llRow, llCol, urRow, urCol ) );
end

