% ind_matrix_trans: translate matrix indices into cartesian coord indices
%-------------------------------------------------------------------------------
% Assume that we enumerate in the following order: x, y, z (ENUM2). The matrix
% referred here is A, where g = f*h has been re-written in matrix-vector format
% as y = Ax.
%
% [i, j, k] = ind_matrix_trans( I, J, K, rowInd, colInd )
%    I, J, K       - the number of points of f (and g) in the x-, y-, and z-dir.
%                    Really should be called num{X,Y,Z}Pts resp.
%    {row,col}Ind  - row and col index of A to translate
%    
% Outputs
%    i, j, k       - discrete cartesian coords. To obtain the actual cont.
%                    coords, use ind_cart_d2c
%
function [i, j, k] = ind_matrix_trans( I, J, K, rowInd, colInd )

% Check arguments
if ( (I < 1) | (J < 1) | (K < 1) ), error( 'invalid mesh size' ), end;
if ( (rowInd < 1) | (colInd < 1) ), error( 'invalid row/col indices' ), end;

i = fni( I, rowInd ) - fni( I, colInd );
j = fnj( I, J, rowInd ) - fnj( I, J, colInd );
k = fnk( I, J, rowInd ) - fnk( I, J, colInd );

function rc = fni( I, l )
rc = mod( l - 1, I );

function rc = fnj( I, J, l )
rc = mod( floor( (l - 1)/I ), J );

function rc = fnk( I, J, l )
rc = floor( (l - 1)/(I * J) );

