% min_hinv_energy: finds the optimal params to min hinv's energy
%-------------------------------------------------------------------------------
% Runs fminsearch() with obj_hinv_energy. Note that if you change the args
% of psf_param3 here, you should also change them in obj_hinv_energy.m. Note
% that Matlab takes a long time to run this minimization routine! This is
% also because the method we're using (the simplex method) doesn't used any
% derivatives and is rather slow.
%
% [x, x0, fVal, exitFlag, output] = min_hinv_energy( nSearchFunc, tolFun,
%    tolX, maxFunEvals, maxIter )
%    nSearchFunc                        - 0 to use fminsearch; 1 to use fminunc
%    tolFun, tolX, maxFunEvals, maxIter - options for minimization procs
%
% Outputs
%    x0 - initial guess
%    x  - result of minimization
%    fVal, exitFlag, output - output of minimization routine
%
function [x, x0, fVal, exitFlag, output] = min_hinv_energy( nSearchFunc, ...
	tolFun, tolX, maxFunEvals, maxIter )

par = psf_param3( 1, 3e4, 3, 1 );
x0 = [ par.Bext, par.Bres, par.R0 ];

options = optimset( 'Display', 'final', ...
	'TolFun', tolFun, 'TolX', tolX, ...
	'MaxFunEvals', maxFunEvals, 'MaxIter', maxIter  );

switch ( nSearchFunc )
case 0
	% Do an unconstrained nonlinear optimization using the simplex method
	[x, fVal, exitFlag, output] = fminsearch( @obj_hinv_energy, x0, options );
	save fminsearch_run;

case 1 
	% Unconstrained nonlinear optimization using quasi-newton methods
	%options2 = optimset( options, 'LargeScale', 'off' );
	options2 = options;
	[x, fVal, exitFlag, output] = fminunc( @obj_hinv_energy, x0, options2 );
	save fminunc_run;

otherwise
	error( 'invalid nSearchFunc' );
end
