% pad2_fft_size: rounds up the length in each dimension to a power of 2
%-------------------------------------------------------------------------------
% A 1d FFT can be efficiently done when the length is a power of 2. As FFTs in
% multiple dimensions is a series of 1d FFTs, we want to pad the length in
% each dimension to be radix-2.
%
% vecNewSize = pad2_fft_size( vecSize [, bPrint] )
%    vecSize - vector of original sizes
%    bPrint  - print the vector of new sizes
%
% Output
%    vecNewSize - row vector of new sizes where, in each dimension, the size 
%                 has been rounded up to a power of 2
%
function vecNewSize = pad2_fft_size( vecSize, bPrint )

n = length( vecSize );
vecNewSize = zeros( 1, n );

for ind = 1:n
	s = vecSize( ind );
	if ( s <= 0 ), error( 'must have positive size' ), end;
	vecNewSize( ind ) = round( 2^( ceil( log2( s ) ) ) );
end

if ( ( nargin == 2 ) & ( bPrint == 1 ) )
	disp( 'new fft sizes:' );
	disp( vecNewSize );
end
