% pdb_disp: displays ATOM and HETATM data from a PDB file
%-------------------------------------------------------------------------------
% Displays ATOM and HETATM data obtained from a PDB file. Must have used
% pdb_read_file prior to calling this function.
%
% pdb_disp( nPlotType, A, AType, H, HType [, sRePlotType] )
%    nPlotType          - specify what type of plot to create
%    A, AType, H, HType - data returned from pdb_read_file
%    sRePlotType        - a RE specifying the atoms of interest to plot
%

function pdb_disp( nPlotType, A, AType, H, HType, sRePlotType )

switch ( nPlotType )
case 1
	figure( 1 ), hold off, clf, hold on;

	ATypeStr = my_strtrim( cellstr( AType ) );

	ASize = size( A );
	if ( ( ASize(1) >= 1 ) & ( ASize(2) == 3 ) )
		for atomInd = 1:ASize(1)
			atomType = char( ATypeStr(atomInd) );

			% DEBUG
			%disp( sprintf( 'atomType = %s', atomType ) );

			% Make Carbon atoms a black asterisk. They are the ones that
			% start with C
			if ( strcmp( atomType(1), 'C' ) == 1 )
				plot3( A(atomInd,1), A(atomInd,2), A(atomInd,3), 'k*' );
			else
				% Make all other atoms green circles
				plot3( A(atomInd,1), A(atomInd,2), A(atomInd,3), 'go' );
			end
		end
	else
		warning( 'No ATOM data' );
	end

	HSize = size( H );
	if ( ( HSize(1) >= 1 ) & ( HSize(2) == 3 ) )
		% Make HETATM entries yellow diamonds
		plot3( H(:,1), H(:,2), H(:,3), 'yd' );
	else
		warning( 'No HETATM data' );
	end

	grid on, axis image, hold off;
	xlabel( 'x [Ang]' ), ylabel( 'y [Ang]' ), zlabel( 'z [Ang]' );

case 2
	figure( 2 ), hold off, clf;

	if ( (nargin < 6) | (length( sRePlotType ) <= 0) )
		error( 'must specify a RE pattern' );
	end

	spinPos = pdb_match( A, AType, H, HType, sRePlotType );

	spinPosSize = size( spinPos );
	numSpins = spinPosSize( 1 );
	vecOnes = ones( numSpins, 1 );

	spinPosExp = mean( spinPos );

	% Centre the position of the spins around (0,0,0).
	spinPos( :, 1 ) = spinPos( :, 1 ) - spinPosExp(1)*vecOnes;
	spinPos( :, 2 ) = spinPos( :, 2 ) - spinPosExp(2)*vecOnes;
	spinPos( :, 3 ) = spinPos( :, 3 ) - spinPosExp(3)*vecOnes;

	plot3( spinPos( :, 1 ), spinPos( :, 2 ), spinPos( :, 3 ), 'bo' );
	grid on, axis image, hold off;
	xlabel( 'x [Ang]' ), ylabel( 'y [Ang]' ), zlabel( 'z [Ang]' );

otherwise
	error( 'unrecognized plot type' );
end



