% pdb_lower_left: Positions the lower left edge of the molecule at 
% the supplied position. Optionally, handle the positioning in the z direction
% differently.
% 
% spinPos = pdb_lower_left( lowerLeftCoord, spinPosOld, bCentreZ )
%
%    lowerLeftCoord - Coordinates of the lower-left point of the grid
%    spinPosOld - Old spin coordinates
%    bCentreZ - If set to 1 (true), we centre the grid around the given
%       z value in lowerLeftCoord
%

function spinPos = pdb_lower_left( lowerLeftCoord, spinPosOld, bCentreZ )

% Add some padding so that the spins are not exactly on the
% boundary edge
[smallestDist, largestDist] = find_dist_minmax( spinPosOld );
spinPosLowerLeft = [ min( spinPosOld(:,1) ) - smallestDist, ...
	min( spinPosOld(:,2) ) - smallestDist, ...
    min( spinPosOld(:,3) ) ];

disp( sprintf( 'spinPosLowerLeft: (%g,%g,%g)', spinPosLowerLeft(1), ...
	spinPosLowerLeft(2), spinPosLowerLeft(3) ) );

if ( bCentreZ ~= 1 )
	tmp = lowerLeftCoord(:)' - spinPosLowerLeft(:)';
	spinPos = spinPosOld + repmat( tmp(:)', size(spinPosOld,1), 1 );
else
	spinPos = zeros( size( spinPosOld ) );
	tmp = lowerLeftCoord(1:2)' - spinPosLowerLeft(1:2)';
	spinPos( :, 1:2 ) = spinPosOld( :, 1:2 ) + repmat( tmp(:)', size(spinPosOld,1), 1 );
	deltaZ = lowerLeftCoord(3) - mean( spinPosOld(:,3) );
	spinPos( :, 3 ) = spinPosOld( :, 3 ) + deltaZ * ones( size(spinPosOld,1), 1 );
end
