% Evolved from psf_param3. Usage is sPsf = psf_param4( varargin ), 
% where the options are: 
%
%    Bres, Bext, d, R0, bComputeOpt{R0,Bext}, bUseSmaller{R0,B0}, 
%    nVerboseLevel

function sPsf = psf_param4( varargin )

% Initialize parameters with values from psf_param2
%==================================================

% [Gauss]
ParBext = 2e4;					
% [Gauss]
ParBres = 2.25e4;				
% [nm]
ParR0 = 6.017e1;				
% [???] dunno what units this is
%ParM = 800;					
ParM = 1700;
% [emu] Assume spherical tip
Parm = 4*pi/3*ParM*ParR0^3;		
% [nm]
Parz0 = 8.35e1;					
% [nm] Not previously given
Pard = -1;						
% [nm]
ParxPk = 2e-1;					
% [nm] Not previously computed
ParxOpt = -1;					
% [nm]
ParxSpan = 10;					
% [Gauss/nm] Not previously computed
ParGMax = -1;					 
% [Gauss]
%deltaB0 = 50;				
deltaB0 = 100;

%103D
%% EXPERIMENTAL
%

% dimensionless
%ParxInc = .02;
ParxInc = .002;					

%
%% Flags that control behaviour
%

B_USE_SMALLER_R0 = 0;
B_USE_SMALLER_B0 = 0;
B_COMPUTE_OPT = 0;
N_VERBOSE_LEVEL = 0;

% Process arguments
%==================

nParam = floor( nargin/2 );
if ( 2*nParam ~= nargin ), error( 'expected even # of ip args' ), end;
for i = 1:nParam
    switch varargin{2*i-1}
        case 'Bres', ParBres = varargin{2*i};
        case 'Bext', ArgBext = varargin{2*i};
        case 'd', Pard = varargin{2*i};
        case 'R0', ArgR0 = varargin{2*i};
        case 'bComputeOptR0', B_COMPUTE_OPT_R0 = varargin{2*i};
        case 'bComputeOptBext', B_COMPUTE_OPT_BEXT = varargin{2*i};
        case 'bUseSmallerR0', B_USE_SMALLER_R0 = varargin{2*i};
        case 'bUseSmallerB0', B_USE_SMALLER_B0 = varargin{2*i};
        case 'nVerboseLevel', N_VERBOSE_LEVEL = varargin{2*i};
        otherwise, error( sprintf( 'unrecognized key: %s', varargin{2*i-1} ) );
    end
end

% Check arguments
if ( ParBres <= 0 | Pard <= 0 ), error( 'invalid Bres or d' ), end;

% Compute optimal R0
if ( B_COMPUTE_OPT_R0 == 1 )

	optR0Fac = 3; % optimal R0 factor
	ParR0 = optR0Fac*Pard;

	% Because the optimal R0 is too large, go with something smaller. This 
	% causes the resonant slice to shrink in radius
	if ( B_USE_SMALLER_R0 == 1 )
		expR0Fac = 1;
		ParR0 = expR0Fac*Pard;
		disp( 'Using smaller optimal R0' );
		if N_VERBOSE_LEVEL >= 1
			disp( sprintf( 'Using ParR0 = %g instead of %g', ParR0, optR0Fac*Pard ) );
		end
    end

else % Do not compute optimal R0    
    ParR0 = ArgR0;
    disp( sprintf( 'Not computing optimal R0, using %g', ParR0 ) );
end

Parm = 4*pi/3*ParM*ParR0^3;
Parz0 = ParR0 + Pard;

optXFac = -( sqrt(35) - sqrt(19) )/4; % optimal x-pos factor for max G
ParxOpt = optXFac*Parz0;
ParxSpan = abs( 2*ParxOpt );

optGFac = 2.7428; % max factor for G in terms of m,z0
ParGMax = optGFac * Parm / Parz0^4;

% Use a smaller B0
if ( B_USE_SMALLER_B0 == 1 )
    DELTA_B0 = 20;
    disp( 'Using smaller B0' );
    if N_VERBOSE_LEVEL >= 1
        disp( sprintf( 'Using deltaB0 = %g instead of %g', DELTA_B0, deltaB0 ) );
    end
    deltaB0 = DELTA_B0;
end

ParxPk = deltaB0/ParGMax;

% Compute optimal Bext
if ( B_COMPUTE_OPT_BEXT == 1 )
	a1 = -(2 - optXFac^2)/(1 + optXFac^2)^(5/2);
	a2 = -9 * optXFac^2 / (1 + optXFac^2)^5;
	quadSol1 = a1 * (Parm/Parz0^3) + sqrt( ParBres^2 + a2*(Parm/Parz0^3)^2 );
	quadSol2 = a1 * (Parm/Parz0^3) - sqrt( ParBres^2 + a2*(Parm/Parz0^3)^2 );

	if N_VERBOSE_LEVEL >= 1 
		disp( sprintf( '%12s %12s %12s %12s', 'a1', 'a2', 'quadSol1', ...
			'quadSol2' ) );
		disp( sprintf( '%12.3e %12.3e %12.3e %12.3e', a1, a2, quadSol1, ...
			quadSol2 ) );
	end

	ParBext = quad_pos_sol( quadSol1, quadSol2 );
	disp( sprintf( 'Bext is +soln to quad eqn => %g', ParBext ) );
    
else % Do not compute optimal Bext
    ParBext = ArgBext;
    disp( sprintf( 'Not computing optimal Bext, using %g', ParBext ) );
end

sPsf = struct( 'Bext', ParBext, ... % Simulation parameters
	'Bres', ParBres, ...
	'R0', ParR0, ...
	'M', ParM, ...	
	'm', Parm, ...	
	'xPk', ParxPk, ...	
	'xOpt', ParxOpt, ...
	'z0', Parz0, ...
	'd', Pard, ...
	'GMax', ParGMax, ...
	'xSpan', ParxSpan );
