% psf_sim_mult: simulation file for cantilever PSF, multiple spins
%-------------------------------------------------------------------------------
% 
% {{{ History }}}
% 
% 27-jul-04: created from psf_sim
%

% Load parameter values from another file
psf_param2

% meshSpin{X,Y} are the mesh variables associated with the output of a single
% spin
vecSpinX = xSpan*[-1:xInc:1];
vecSpinY = ySpan*[-1:yInc:1];
spinXLen = length( vecSpinX );
spinYLen = length( vecSpinY );
[meshSpinX, meshSpinY] = meshgrid( vecSpinX, vecSpinY );

% meshImg{X,Y} are the mesh variables for the entire image, i.e. it has the
% images of multiple spins
vecImgX = xSpan*[-spaceFac:xInc:spaceFac];
vecImgY = ySpan*[-spaceFac:yInc:spaceFac];
imgXLen = length( vecImgX );
imgYLen = length( vecImgY );
[meshImgX, meshImgY] = meshgrid( vecImgX, vecImgY );

% Constants
MESH_SPIN_ALL_ZEROS = zeros( spinYLen, spinXLen );
MESH_IMG_ALL_ZEROS = zeros( imgYLen, imgXLen );
bRandZPos = 0;

Bmag = MESH_SPIN_ALL_ZEROS;
G = MESH_SPIN_ALL_ZEROS;
s = MESH_SPIN_ALL_ZEROS;

% Pspin is the output due to a single spin while Pimg is the output for the
% entire image
Pspin = MESH_SPIN_ALL_ZEROS;
Pimg = MESH_IMG_ALL_ZEROS;

nErrs = 0; % Count the number of errors from psf_vert

disp( sprintf( 'spin image size: %6d x %6d, total image size: %6d x %6d', ...
	spinYLen, spinXLen, imgYLen, imgXLen ) );

% This contains the Cartesian coords of the spins, one per row
spinLoc = zeros( numSpins, 3 );

for spinInd = 1:numSpins
	% Ensure that the spin's x and y-coords don't exceed the valid range
	spinLoc( spinInd, 1 ) = ...
		max( 1, floor( (spaceFac-1)*(spinXLen-1)*rand )  );
	spinLoc( spinInd, 2 ) = ...
		max( 1, floor( (spaceFac-1)*(spinYLen-1)*rand )  );
	spinLoc( spinInd, 3 ) = z0;
	if ( bRandZPos == 1 ) % Randomize the z-coord
		spinLoc( spinInd, 3 ) = spinLoc( spinInd, 3 ) + ...
			deltaZUnc * ( 2*rand - 1 );
	end
	disp( sprintf( 'spin %3d at (%6d, %6d, %.3e)', spinInd, ...
		spinLoc(spinInd,1), spinLoc(spinInd,2), spinLoc(spinInd,3) ) );
end

for spinInd = 1:numSpins % Super-impose the spin energies
	x = spinLoc( spinInd, 1 ); 
	y = spinLoc( spinInd, 2 ); 
	z = spinLoc( spinInd, 3 );
	for j = 1:spinYLen
		for k = 1:spinXLen
			% Obtain the PSF for a spin at (0, 0, zSpin)
			[nStatus, Bmag(j,k), G(j,k), s(j,k), Pspin(j,k)] = ...
				psf_vert( meshSpinX(j,k), meshSpinY(j,k), z, ParBext, ...
					ParBres, Parm, Parxpk, 0 );
			if ( nStatus ~= 0 ), nErrs = nErrs + 1; end
		end
	end
	for j = 1:spinYLen
		for k = 1:spinXLen
			% Then, translate it in the x-y plane by the proper amount
			Pimg( j + y, k + x ) = Pimg( j + y, k + x ) + Pspin( j, k );
		end
	end
end

disp( sprintf( 'nErrs = %5d', nErrs ) );

PimgAbsMax = max( max( abs( Pimg ) ) );
PimgMax = max( max( Pimg ) );
PimgMin = min( min( Pimg ) );
noiseSigma = noiseSigmaFrac * PimgAbsMax;

disp( sprintf( '%15s %15s %15s %15s', 'PimgMin', 'PimgMax', 'PimgAbsMax', ...
	'noiseSigma' ) );
disp( sprintf( '%15.3e %15.3e %15.3e %15.3e', PimgMin, PimgMax, PimgAbsMax, ...
	noiseSigma ) );

%...............................................................................
% XXX: Add noise. Note that the noise must be non-negative, because the
%      measurement is energy. COMMENT: this has not been nailed down, hence
%      should only be used to create pretty pictures. Should NOT be used for
%      analysis!
%...............................................................................

if 0
	disp( 'XXX: adding noise' );
	for j = 1:imgYLen
		for k = 1:imgXLen
			P( j, k ) = P( j, k ) + noiseSigma*(rand)^2;
		end
	end
end %%% if 0/1

close all;
sTitle = sprintf( '%g random spins, z_0=%g, deltaZUnc=%g', ...
	numSpins, z0, deltaZUnc ); 
create_int_plot( 1, meshImgX, meshImgY, Pimg, sTitle, [0, 0.6] );
if 0, print( '-f1', '-djpeg', 'pics/mult.jpg' ), end
