% psf_sim_sing: simulation file for cantilever PSF, single spin
%-------------------------------------------------------------------------------
%
% {{{ History }}}
% 
% 27-jul-04: created from psf_sim. 
% 30-jul-04: modified so that it works with psf_param3
%

%par = psf_param3( 0 );
par = psf_param3( 1, 3e4, 3, 1 );

if 0 % EXPERIMENTAL
	par.Bext = 8.3331e7;
	par.Bres = 8.5708e7;
	par.R0 = 45.285;
	par.m = 4*pi/3*par.M*(par.R0)^3;
	par.xPk = .048931;
	par.z0 = par.R0 + par.d;
	return;
end

xMeshSize = par.xSpan*par.xInc;
yMeshSize = par.ySpan*par.yInc;
disp( sprintf( 'Mesh size: x-dir = %.3e, y-dir = %.3e', xMeshSize, ...
	yMeshSize ) );

vecX = par.xSpan*[-1:par.xInc:1]; 
xLen = length( vecX );
vecY = par.ySpan*[-1:par.yInc:1];
yLen = length( vecY );

% Constants
MESH_ALL_ZEROS = zeros( yLen, xLen );
MESH_ALL_ONES = ones( yLen, xLen );

% Create a mesh. Note that we should avoid the origin.
[meshX, meshY] = meshgrid( vecX, vecY );
meshZ = par.z0*MESH_ALL_ONES;

Bmag = MESH_ALL_ZEROS;
G = MESH_ALL_ZEROS;
s = MESH_ALL_ZEROS;
P = MESH_ALL_ZEROS;
nErrs = 0; % Count the number of errors from psf_vert

disp( 'Computing PSF...' );
tic;

if 0 % Non-MEX fashion
	for i = 1:yLen
		for j = 1:xLen
			[nStatus, Bmag(i,j), G(i,j), s(i,j), P(i,j)] = ...
				psf_vert( meshX(i,j), meshY(i,j), meshZ(i,j), par.Bext, ...
					par.Bres, par.m, par.xPk, 0 );
				if ( nStatus ~= 0 ), nErrs = nErrs + 1; end
		end
	end
else
	% Use MEX function
	disp( '   NOTE: using mex_psf_vert' );
	[nErrs, Bmag, G, s, h] = ...
		mex_psf_vert( meshX, meshY, meshZ, ...
			par.Bext, par.Bres, par.m, par.xPk, 0 );
end

tElapsed = toc;
disp( sprintf( 'nErrs = %5d', nErrs ) );
disp( sprintf( 'tElapsed = %g', tElapsed ) );

% Find the max gradient
GMax = max( max( G ) );
[GMaxRow, GMaxCol] = find( G == GMax );
GMaxX = meshX( GMaxRow, GMaxCol );
GMaxY = meshY( GMaxRow, GMaxCol );
disp( sprintf( 'max of G = %g at ( %.3e, %.3e )', GMax, GMaxX, GMaxY ) );

% Find the inverse filter F^{-1}
fftSize = [ 2^(ceil( log2( yLen - 1 ) )), 2^(ceil( log2( xLen - 1 ) )), 1 ];
disp( sprintf( 'fftSize: %5d, %5d, %5d', fftSize(1), fftSize(2), ...
	fftSize(3) ) );

H = fft2( h, fftSize(1), fftSize(2) ); 
HInv = ones( fftSize(1), fftSize(2) ) ./ H;
hInvLarge = ifft2( HInv );
hInv = hInvLarge( 1:yLen, 1:xLen );

% Create pcolor plots 
disp( 'Creating intensity plots...' );
tic;

close all; 
create_int_plot( 1, meshX, meshY, Bmag, [],  sprintf( 'B_{mag}(x,y,z_0), z_0=%.3e', par.z0 ) );
create_int_plot( 2, meshX, meshY, G, [], sprintf( 'G(x,y,z_0), z_0=%.3e', par.z0 ) );
create_int_plot( 3, meshX, meshY, s, [], sprintf( 's(x,y,z_0), z_0=%.3e', par.z0 ) );
create_int_plot( 4, meshX, meshY, h, [0 0.3], sprintf( 'h(x,y,z_0), z_0=%.3e', par.z0 ), [0 0.3], 1 );
create_int_plot( 5, meshX, meshY, abs( H(1:yLen, 1:xLen) ), [0, 1e6], ...
    sprintf( '|H(x,y,z_0)|, z_0=%.3e', par.z0 ), 1, 0, 'abs' );
create_int_plot( 6, meshX, meshY, abs( HInv(1:yLen, 1:xLen) ), [0 1e-5], ...
    sprintf( '|HInv(x,y,z_0)|, z_0=%.3e', par.z0 ), 1, 1, 'abs' );
create_int_plot( 7, meshX, meshY, hInv, [-1.5e-5 0], ...
    sprintf( 'hInv(x,y,z_0), z_0=%.3e', par.z0 ), 1, 1, 'abs' );

figure( 8 ), surf( meshX, meshY, hInv );
figure( 9 ), hist( reshape( log(abs(HInv)), [fftSize(1)*fftSize(2), 1] ), 50 );


tElapsed = toc;
disp( sprintf( 'tElapsed = %g', tElapsed ) );

matHEnergy = abs( H ).^2;
HEnergy = sum( sum( matHEnergy ) );

matHInvEnergy = abs( HInv ).^2;
HInvEnergy = sum( sum( matHInvEnergy ) );

disp( sprintf( 'HEnergy = %.4e, HInvEnergy = %.4e', HEnergy, HInvEnergy ) );
