% rotate_3d: rotate 3-d pts in Cartesian coords
%-------------------------------------------------------------------------------
%
% PPrime = rotate_3d( P, sAxisName, theta )
%    P         - a matrix of points where each row corresponds to one point
%    sAxisName - axis around which to rotate. Valid values are {'x','y','z'}
%                (case-insensitive)
%    theta     - amount to rotate in the CCW direction in radians
%
% Output
%    PPrime    - new points
%
function PPrime = rotate_3d( P, sAxisName, theta )

% Check arguments
PSize = size( P );
numPts = PSize( 1 );
if ( (numPts < 1) | (PSize(2) ~= 3) ), error( 'incompatible P' ), end;
if ( (theta < 0) | (theta > 2*pi) ), erorr( 'invalid theta' ), end;

% Initialize
PPrime = [];
switch ( lower( sAxisName ) )
case 'x'
	A = [ 1, 0, 0; 0, cos(theta), -sin(theta); 0, sin(theta), cos(theta) ];
case 'y'
	A = [ cos(theta), 0, sin(theta); 0, 1, 0; -sin(theta), 0, cos(theta) ];
case 'z'
	A = [ cos(theta), -sin(theta), 0; sin(theta), cos(theta), 0; 0, 0, 1 ];
otherwise
	error( 'invalid sAxisName' );
end

for i = 1:numPts
	x = P( i, : )';
	y = A*x;
	PPrime = [ PPrime; y' ];
end
	
	
