% steer_dprod: steer dot product of <v,d> in a maximal or minimal fashion
%-------------------------------------------------------------------------------
% We want to steer the dot product of <v,d> to be as maximal (positive) or
% minimal (negative) as possible, given that the d_i's can only take certain
% values. Not that d_i \in {0, 1, -1}.
%
% delta = steer_dprod( v, possibleDelta, sDir )
%    v             - vector in <v,d>
%    possibleDelta - vector of possible delta values. Note that the elements
%                    should be one of {0, 1, -1}
%    sDir          - string describing how to steer the dot product. There are
%                    two valid options: 'positive' (maximal) or 'negative'
%                    (minimal)
% Output
%    delta         - row vector that maximizes or minimizes dot product
%
function delta = steer_dprod( v, possibleDelta, sDir )

% Constants
DOUBLE_THRES = 10*eps;

% Check arguments
n = length( v );
if ( length( possibleDelta ) ~= n )
	error( 'incompatible args in steer_dprod' );
end

for j = 1:n
	dig = abs( possibleDelta( j ) );
	if ( ( abs( dig ) > DOUBLE_THRES ) & ( abs( dig - 1 ) > DOUBLE_THRES ) )
		error( 'invalid possibleDelta' );
	end
end

delta = zeros( 1, n );

switch ( lower( sDir ) )
case 'positive'
	for j = 1:n
		if ( possibleDelta( j ) > 0 ) % equals +1
			if ( v( j ) > 0 )
				delta( j ) = 1;
			end
		elseif ( possibleDelta( j ) < 0 ) % equals -1
			if ( v( j ) < 0 )
				delta( j ) = -1;
			end
		end
	end

case 'negative'
	for j = 1:n
		if ( possibleDelta( j ) > 0 ) % equals +1
			if ( v( j ) < 0 )
				delta( j ) = 1;
			end
		elseif ( possibleDelta( j ) < 0 ) % equals -1
			if ( v( j ) > 0 )
				delta( j ) = -1;
			end
		end
	end

otherwise
	error( 'invalid sDir' );
end


