%
% Plot the PSF for the vertical cantilever
%

M = 800; % 800 emu/cm^3
R_0 = 2250 * 1e-8; % 2250 angstrom, 1 angstrom = 10^{-8} cm
                   % (converted to cm)
 
p_Bext = 300; % 300 Gauss
p_Bres = 1060; % 1060 Gauss
p_m = 4*pi/3*M*R_0^3;
p_xpk = 100 * 1e-8; % 100 angstrom, converted to cm

[X, Y] = meshgrid( 1e-9*[-1:.01:1], 1e-9*[-1:.01:1] );
Xlen = length(X);
Ylen = length(Y);
Z = zeros( Xlen, Ylen );
z = 1e-9*0.5;

for i = 1:Xlen
	for j = 1:Ylen
		p = psf_vert( X(i,j), Y(i,j), z, p_Bext, p_Bres, p_m, p_xpk );
		disp( sprintf( '[%g,%g] p = %g', X(i,j), Y(i,j), p ) );
		Z( i, j ) = log( p );
	end
end

pMin = min( min( Z ) );
disp( sprintf( 'pMin = %g', pMin ) );
Z = Z - pMin;
disp( sprintf( 'Xlen = %g, Ylen = %g', Xlen, Ylen ) );

close all;
mesh( X, Y, Z );
