%-------------------------------------------------------------------------------
% Asymptotic behaviour of mean-squared errors for various estimates for the
% noisy two-state Markov process.
%
% Ref: Y-C Yao, Estimation of Noisy Telegraph Processes: Nonlinear Filtering
% Versus Nonlinear Smoothing, IEEE Trans. on Info Thry, IT-31, No. 3, 1985,
% pp 444-446
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all; % Close all figures

%%% Case 1: gamma -> 0+

gammaVec = linspace( 1e-6, 1e-2, 100 );
linFiltMse = 2 .* sqrt( gammaVec );
linSmoothMse = sqrt( gammaVec );
nlFiltMse = -2 .* gammaVec .* log( gammaVec );
nlSmoothMse = 2 .* gammaVec;
plot( gammaVec, linFiltMse, 'b-', gammaVec, linSmoothMse, 'b:', ...
	gammaVec, nlFiltMse, 'g-', gammaVec, nlSmoothMse, 'r' ); 
legend( 'LinFilt MSE', 'LinSmooth MSE', 'NL Filt MSE', 'NL Smooth MSE', 0 );
title( 'Asymptotic behaviour as \gamma -> 0^+' );

%%% Case 2: gamma -> \infty

gammaVec = linspace( 0.4, 12, 100 );
linFiltMse = 1 - 1./(4 * gammaVec);
linSmoothMse = 1 - 1./(2 * gammaVec);
nlFiltMse = 1 - 1./(4 * gammaVec);
nlSmoothMse = 1 - 1./(2 * gammaVec);
figure; % New figure
plot( gammaVec, linFiltMse, 'bx', gammaVec, linSmoothMse, 'bo', ...
	gammaVec, nlFiltMse, 'g-', gammaVec, nlSmoothMse, 'r' );
legend( 'LinFilt MSE', 'LinSmooth MSE', 'NL Filt MSE', 'NL Smooth MSE', 0 );
title( 'Asymptotic behaviour as \gamma -> \infty' );

%%% Case 3: combine both asymptotics for NL Filt

figure; % New figure
gammaVecL = linspace( 1e-6, 1, 1e3 );
gammaVecU = linspace( 0.3, 2, 1e3 );

subplot(221),
nlFiltMseL = -2 .* gammaVecL .* log( gammaVecL );
nlFiltMseU = 1 - 1./(4 * gammaVecU);
plot( gammaVecL, nlFiltMseL, 'bx', gammaVecU, nlFiltMseU, 'r.' );
legend( 'L-bnd', 'U-bnd', 0 );
title( 'Asymptotic behaviour of NL Filt MSE' );

subplot(222),
nlSmoothMseL = 2 .* gammaVecL;
nlSmoothMseU = 1 - 1./(2 * gammaVecU);
plot( gammaVecL, nlSmoothMseL, 'bx', gammaVecU, nlSmoothMseU, 'r.' );
a = axis;
axis( [a(1) a(2) 0 1] );
legend( 'L-bnd', 'U-bnd', 0 );
title( 'Asymptotic behaviour of NL Smooth MSE' );

subplot(223),
linFiltMseL = 2 .* sqrt( gammaVecL );
linFiltMseU = 1 - 1./(4 * gammaVecU);
plot( gammaVecL, linFiltMseL, 'bx', gammaVecU, linFiltMseU, 'r.' );
a = axis;
axis( [a(1) a(2) 0 1] );
legend( 'L-bnd', 'U-bnd', 0 );
title( 'Asymptotic behaviour of Lin Filt MSE' );

subplot(224),
linSmoothMseL = sqrt( gammaVecL );
linSmoothMseU = 1 - 1./(2 * gammaVecU);
plot( gammaVecL, linSmoothMseL, 'bx', gammaVecU, linSmoothMseU, 'r.' );
a = axis;
axis( [a(1) a(2) 0 1] );
legend( 'L-bnd', 'U-bnd', 0 );
title( 'Asymptotic behaviour of Lin Smooth MSE' );

