% // 2008-05-30 // Run the script to implement Landweber iteration on the
% experimental dataset.
%//////////////////////////////////////////////////////////////////////////

% Figure out the support of the psf. The idea is to go through the xy
% slices and determine the largest support

SZ_X0_SEL   = 'given';

% Initialize
[ xInd, yInd ] = find( psfMat( :, :, 1 ) > 0 );
sPsf.supp{1} = [ min(xInd) max(xInd) ];
sPsf.supp{2} = [ min(yInd) max(yInd) ];

for zsliceInd = 2:size( psfMat, 3 )
    
    [ xInd, yInd ] = find( psfMat( :, :, zsliceInd ) > 0 );
    
    xIndMin = min( xInd  );
    if ( xIndMin < sPsf.supp{1}(1) ), sPsf.supp{1}(1) = xIndMin; end
    
    xIndMax = max( xInd );
    if ( xIndMax > sPsf.supp{1}(2) ), sPsf.supp{1}(2) = xIndMax; end
        
    yIndMin = min( yInd );
    if ( yIndMin < sPsf.supp{2}(1) ), sPsf.supp{2}(1) = yIndMin; end
    
    yIndMax = max( yInd );
    if ( yIndMax > sPsf.supp{2}(2) ), sPsf.supp{2}(2) = yIndMax; end
    
end

psfShiftAmount = [ -(sPsf.supp{1}(2)-sPsf.supp{1}(1))/2, ...
                   -(sPsf.supp{2}(2)-sPsf.supp{2}(1))/2, ...
                   0 ];
psfShiftAmount = ceil( psfShiftAmount );
disp( 'psfShiftAmount is:' ), disp( psfShiftAmount );

if ( true ) %%% Landweber iterations with downsampling
    
    sReconParam = struct( 'addNoiseVar', 30^2, ...
        'interpFac', [3 2], 'offset', [0 0], ...
        'zsliceOffset', 1 );

    sReconParam.zslices = 3*( [1:4] + sReconParam.zsliceOffset );
    sReconParam.zsliceMin = max( [1, min( sReconParam.zslices ) - 3] );
    sReconParam.zsliceMax = max( sReconParam.zslices );

    xSize = [ 270 94 size(psfMat,3)+sReconParam.zsliceMax ];
           
    mask = logical( ones( xSize ) );
    Gfft = Gblur( mask, 'psf', psfMat, 'type', 'conv,fft', 'nDim', length( xSize ) );

    switch SZ_X0_SEL
        case 'zeros'
            x0 = zeros( xSize );
            sReconParam.szX0Sel = 'Hty2';
        otherwise
            % If x0 is defined already, use it. Otherwise, croak.
            if ( ~exist( 'x0', 'var' ) )
                error( sprintf( 'Unrecognized SZ_X0_SEL: %s', SZ_X0_SEL ) );
            end
            sReconParam.szX0Sel = 'given';
    end
    
    
    if ( false )
        sPsfProp = psf_l2norm( 'psf', Gfft, 'psfmat', psfMat, 'x0', x0, ...
            'interp_fac', sReconParam.interpFac, 'offset', sReconParam.offset, ...
            'bverbose', true, 'maxiter', 1e4, 'y', tmv2Mat, ...
            'zslices', sReconParam.zslices, 'zslicemin', -1, 'zslicemax', -1 );

        return; % XXX DEBUG
    end
    
    %cGiven = 1/(8.4162^2/5;
    %cGiven = 1/(40.0448)^2;
    %cGiven = 1e-3;
    cGiven = -1; % Compute automatically 
    
    sMap = explandwebdsME( 'psf', Gfft, 'psfshift', psfShiftAmount, ...
        'psfmat', psfMat, 'y', tmv2Mat( :, :, 1:4 ) - 30, 'x0', x0, ...
        'addnvar', sReconParam.addNoiseVar, ...
        'maxiter', 5e4, 'interp_fac', sReconParam.interpFac, ...
        'offset', sReconParam.offset, 'zslices', sReconParam.zslices, ...
        'zslicemin', -1, 'zslicemax', -1, 'bplot', false, ...
        'c', cGiven, 'bnonneg', false, 'sz_msel', 'map2', ...
        'hparam_rest_intv', 1e3, 'g_special', 1/sqrt(2), ...
        'x0_sel', sReconParam.szX0Sel );

elseif ( false ) %%% Landweber iterations with decimation

    sReconParam = struct( 'addNoiseVar', 30^2, ...
        'interpFac', [3 2 3], 'offset', [0 0 0], ...
        'zsliceOffset', 1 );

    sReconParam.zslices = 3*( [1:5] + sReconParam.zsliceOffset );
    sReconParam.zsliceMin = max( [1, min( sReconParam.zslices ) - 3] );
    sReconParam.zsliceMax = max( sReconParam.zslices );

    xSize = [ 270 94 size(psfMat,3)+sReconParam.zsliceMax ];

    mask = logical( ones( xSize ) );
    Gfft = Gblur( mask, 'psf', psfMat, 'type', 'conv,fft', 'nDim', length( xSize ) );
    x0 = zeros( xSize );
    
    sLandweb = explandwebdec( 'psf', Gfft, 'psfshift', [], ...
        'psfmat', psfMat, 'y', tmv2Mat, 'x0', x0, ...
        'addnvar', sReconParam.addNoiseVar, ...
        'maxiter', 3e2, 'interp_fac', sReconParam.interpFac, ...
        'offset', sReconParam.offset, ...
        'zslicemin', -1, 'zslicemax', -1, 'c', 1e-3, 'bnonneg', false );
    
end

