% // 2008-07-16 // Add MAP{1,2} ability.
% Keys: psf, psfshift, y, x0, addnvar, maxiter, l2normstop, interp_fac,
%       zlices, offset, zslicemin, zslicemax, psfmat, bverbose, bnonneg,
%       sz_msel, hparam_rest_intv, g_special, x0_sel
% Output: sLandweb
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function sLandweb = explandwebdsME( varargin )

% Defaults
%/////////

psfMat              = [];
GPsf                = [];
y                   = [];

x0                  = [];   % Dummy variable
x0Real              = [];
x0Sel               = 'Hty2';

psfShiftAmount      = [];

interpFactor        = [];
interpFactorLen     = 2;
offset              = [];
offsetLen           = 2;

addNoiseVar         = 0;
maxIter             = 1e4;
l2NormStop          = 0;

yDeltaNorm          = [];
xDeltaNorm          = [];
xNorm               = [];
yDeltaNormSmallest  = -1;

c                   = 0;
cScaleFactor        = 0.1;
cGiven              = -1;

zSlices             = [];
zSliceMax           = -1;
zSliceMin           = -1;

bVerbose            = true;
bNonNeg             = false;
bPlot               = true;
nPlotMult           = 30;

sLowestObsErr       = struct([]);

hParamEstHist       = [];
thresHist           = [];
hParamEstHistInd    = 0;
hParamRestIntv      = 5e2;
gSpecial            = 1/sqrt(2);

% Supports: 'id', 'map1', 'map2', 'map1_fixed'
szMSel              = 'id';

% Parse arguments
%////////////////
argNum = floor( length( varargin ) / 2 );

for argInd = 1:argNum
    szKey = varargin{ 2*argInd-1 };
    ip = varargin{ 2*argInd };
    switch szKey
        case 'psf', GPsf = ip;
        case 'psfshift', psfShiftAmount = ip;
        case 'psfmat', psfMat = ip;
        case 'y', y = ip;
        case 'x0', x0 = ip;
        case 'addnvar', addNoiseVar = ip;
        case 'maxiter', maxIter = ip;
        case 'l2normstop', l2NormStop = ip;
        case 'interp_fac', interpFactor = ip; 
        case 'offset', offset = ip;
        case 'zslices', zSlices = ip;
        case 'zslicemax', zSliceMax = ip;
        case 'zslicemin', zSliceMin = ip;
        case 'bverbose', bVerbose = logical( ip );
        case 'bnonneg', bNonNeg = logical( ip );
        case 'bplot', bPlot = logical( ip );
        case 'c', cGiven = ip;
        case 'sz_msel', szMSel = ip;
        case 'hparam_rest_intv', hParamRestIntv = ip;
        case 'g_special', gSpecial = ip;
        case 'x0_sel', x0Sel = ip;
        otherwise, error( sprintf( 'Unrecognized key: %s', szKey ) );
    end
end

% Check arguments
%////////////////

if ( l2NormStop == 0 )
    l2NormStop = norm( y(:), 2 ) * 1e-2;
end

if ( length( interpFactor ) ~= interpFactorLen )
    disp( 'interpFactor:' ), disp( interpFactor );
    error( sprintf( 'expected length of interpFactor to be %g', ...
        interpFactorLen ) );
end

if ( length( offset ) ~= offsetLen )
    disp( 'offset:' ), disp( offset );
    error( sprintf( 'expected length of offset is %g', ...
        offsetLen ) );
end    

if ( isempty( zSlices ) )
    error( 'zSlices is empty' );
end
    
xCurr = x0; % Initialize xCurr
 
% Assume that the l2 norm of the linear operator H is unchanged by the
% downsampling and interpolation. 
psfDft = fftn( psfMat );
psfDftProp = emsep_comp_alpha( psfDft, sqrt( addNoiseVar ), 0 );
%cComp = (cScaleFactor * psfDftProp.alphaUpperBound)^2 / addNoiseVar;
cComp = 1 / ( psfDftProp.norm2 )^2 * cScaleFactor;
fprintf( 'computed c=%g\n', cComp ); 

if ( cGiven < 0 )
    c = cComp;    
else
    c = cGiven;
end

fprintf( 'using c=%g\n', c );

% Based on c = (\alpha/\sigma)^2, compute \alpha. Also compare with
% psfDftProp.alphaUpperBound
alphaFac = sqrt( c * addNoiseVar );
fprintf( 'using alphaFac=%g compared with ub %g\n', alphaFac, ...
    psfDftProp.alphaUpperBound );

% yEstRough should have the same dimensions as xCurr. In each xy plane, we
% will be downsampling. Compute the max multiple in the x & y directions.

maxMult = [ floor( size( xCurr, 1 )/interpFactor( 1 ) ), ...
            floor( size( xCurr, 2 )/interpFactor( 2 ) ) ];

xSel = interpFactor(1) * ( offset(1) + [1:size(y,1)] );
ySel = interpFactor(2) * ( offset(2) + [1:size(y,2)] );

% The downsampling need not start from the beginning. Ideally, we position
% the downsampled points in the "middle". Ensure that there are enough
% points to support this.
if ( ( xSel(end)/interpFactor(1) > maxMult(1) ) | ...
     ( ySel(end)/interpFactor(2) > maxMult(2) ) )
    error( '{x,y}Sel out of bounds' );
end    

bStop = false;

% Before iterating, do some initialization
hParamEstHist = zeros( 1 + floor( maxIter/hParamRestIntv ), 2 );
thresHist = zeros( size( hParamEstHist, 1 ), 1 );
hParamEstHistInd = 1;

% Iterate
for nIter = 0:maxIter
    
    % E-step
    %///////
    
    % Form Hx
    yEstRough = GPsf * xCurr;
    
    if ( bPlot & ( mod( nIter, nPlotMult ) == 0 ) )
        xysliceplot( 'x', yEstRough, 'nfig', 1, 'b_pause', true, ...
            'sz_title', 'yEstRough' );
    end
    
    yEstInterp = mydownsamp3( yEstRough, xSel, ySel, zSlices );

    if ( bPlot & ( mod( nIter, nPlotMult ) == 0 ) )
        xysliceplot( 'x', yEstInterp, 'nfig', 2, 'b_pause', true, ...
            'sz_title', 'yEstInterp' );
    end

    if ( any( size( yEstInterp ) ~= size( y ) ) )
        error( 'incompatible sizes for yEstInterp and y' );
    end
    
    yDelta = y - yEstInterp;
    yDeltaNorm = [ yDeltaNorm, norm( yDelta(:), 2 ) ]; % Keep track
    
    if ( yDeltaNormSmallest < 0 )
        yDeltaNormSmallest = yDeltaNorm( end ); % Initialize
    end
    
    if ( bPlot & ( mod( nIter, nPlotMult ) == 0 ) )
        xysliceplot( 'x', yDelta, 'nfig', 3, 'b_pause', true, ...
            'sz_title', 'yDelta' );
    end
    
    if ( false & ( yDeltaNorm(end) < l2NormStop ) )
        fprintf( '\tyDeltaNorm < l2NormStop\n' );
        bStop = true;
    end
    
    HtyDelta = zeros( size( xCurr ) );
           
    % Form H'(y-Hx)
    for zInd = 1:length( zSlices )
        HtyDelta( xSel, ySel, zSlices(zInd) ) = HtyDelta( xSel, ySel, zSlices(zInd) ) + ...
            yDelta( :, :, zInd );
    end
    
    HtyDelta = GPsf' * HtyDelta;
    
    if ( false & bVerbose )
        fprintf( '\t  |yDelta|_2=%g, |H^T*yDelta|_2=%g, ratio=%g\n', ...
            norm( yDelta(:), 2 ), norm( HtyDelta(:), 2 ), ...
            norm( HtyDelta(:), 2 ) / norm( yDelta(:), 2 ) );
    end
    
    % In this step, compute z = x + cH'(y-Hx)
    xDelta = c * HtyDelta;
    xDeltaNorm = [ xDeltaNorm, norm( xDelta(:), 2 ) ]; % Keep track
    z = xCurr + xDelta;

    if ( bPlot & ( mod( nIter, nPlotMult ) == 0 ) )
        xysliceplot( 'x', circshift( xDelta, -2*psfShiftAmount ), ...
            'nfig', 4, 'b_pause', true, 'sz_title', 'xDelta', 'b_hist', true );
        xysliceplot( 'x', circshift( xNext, -2*psfShiftAmount ), ...
            'nfig', 5, 'b_pause', true, 'sz_title', 'xNext', 'b_hist', true );
    end
    
    if ( bVerbose )
        fprintf( '\titer %5g: |yDelta|_2=%g, |xDelta|_2=%g, |z|_2=%g\n', ...
            nIter, yDeltaNorm( end ), xDeltaNorm( end ), norm( z(:), 2 ) );
    end
    
    if ( yDeltaNormSmallest >= yDeltaNorm( end ) )
        sLowestObsErr = struct( 'x', xCurr, ...
            'yRough', yEstRough, ...
            'y', yEstInterp, ...
            'yDeltaNorm', yDeltaNorm( end ) );
        yDeltaNormSmallest = yDeltaNorm( end );
    end
    
    % M-step
    %///////
    
    if ( nIter == 0 ) % Still in the initialization step
        
        switch ( x0Sel )

            case 'given'
                
                x0Real = x0;
                xNext = x0Real;
                
            case 'Hty2'

                x0Real = z;
                x0RealAbsSorted = sort( abs( x0Real(:) ), 'descend' );
                absPosInd = find( x0RealAbsSorted > eps );
                k = floor( 0.8 * length( absPosInd ) );
                thresh0 = x0RealAbsSorted( k );
                if ( bVerbose )
                    fprintf( 'At nIter=0, thresh0=%g, |x0Real| max %g/min %g\n', ...
                        thresh0, x0RealAbsSorted( 1 ), x0RealAbsSorted( end ) );
                    figure( 1 ), clf, hist( x0Real(:), 40 );
                end

                % Soft-threshold at thresh0
                x0Real = ( x0Real - sign( x0Real ) * thresh0 ) .* ...
                    ( abs( x0Real ) > thresh0 );

                xNext = x0Real; % Replace xNext
                
            otherwise, error( sprintf( 'Unrecognized x0_sel = %g', x0Sel ) );
                
        end
        
    else % Do the real M-step here
        
        sOptions = struct( 'alpha', alphaFac );
        
        switch ( szMSel )
            case 'map1'                
                sMap1 = struct( 'numThresWarning', 0, ...
                    'bUpperBoundThres', true, ...
                    'nIter', nIter, ...
                    'bDebug', true );
                sOptions.map1 = sMap1;
                
            case 'map1_fixed'
                sMap1Fixed = struct( 'a', 1917, 'w', 0.05 );
                sOptions.map1_fixed = sMap1Fixed;
                
            case 'map2'
                sMap2 = struct( 'numThresWarning', 0, ...
                    'bUpperBoundThres', false, ...
                    'nIter', nIter, ...
                    'bDebug', true, ...
                    'gSpecial', gSpecial );
                sOptions.map2 = sMap2;
                                
        end
                            
        [mData, hParamEstNew] = em_restore_m2( z, szMSel, hParamEst, ...
            sOptions );
                
        hParamEst = hParamEstNew;        
        
        switch ( szMSel )
            case 'map1'
                estI = mData( :, 1 );
                estXTilde = mData( :, 2 );
                xNext = reshape( estI .* estXTilde, size( z ) );
                
            otherwise
                xNext = reshape( mData, size( z ) );
        end
                
    end
    
    % Obtain hParamEst
    %/////////////////

    if ( mod( nIter, hParamRestIntv ) == 0 )

        switch ( szMSel )
            case 'map1', mData = [ abs( xNext(:) ) > eps, xNext(:) ];
            case 'map2', mData = xNext(:);
            otherwise, mData = [];
        end

        if ( isequal( szMSel, 'map1' ) | isequal( szMSel, 'map2' ) )
            try
                hParamEst = hparam_est( mData, szMSel );
                hParamEstHist( hParamEstHistInd, : ) = [ hParamEst.a, hParamEst.w ];
                hParamEstHistInd = hParamEstHistInd + 1;
                fprintf( '\n\t@@@ Update %g hParamEst: a=%g, w=%g\n', ...
                    hParamEstHistInd - 1, hParamEst.a, hParamEst.w );
            catch
                error( sprintf( 'Caught error in hparam_est: %s', ...
                    lasterr ) );
            end

            if ( hParamEst.nRc < 0 )
                fprintf( '\thParamEst.nRc < 0: stopping iterations\n' );
                bStop = true;
            end
        else
            hParamEst = [];
        end
        
    end
    
    if ( mod( nIter, hParamRestIntv ) == 1 )
        
        if ( isfield( hParamEst, 'thres' ) )
            fprintf( '\t    thres=%g\n', hParamEst.thres );
            thresHist( hParamEstHistInd-1 ) = hParamEst.thres;
        end
        
    end
           
    xCurrNorm2 = norm( xCurr(:), 2 );
    xNextNorm2 = norm( xNext(:), 2 );
    
    if ( bVerbose )
        fprintf( '|xCurr|_2=%g, |xNext|_2=%g\n', xCurrNorm2, ...
            xNextNorm2 );
    end
    
    if ( ( xCurrNorm2 < eps ) & ( xNextNorm2 < eps ) )
        bStop = true;
        warning( 'stopping because solution converged to 0' );
    end
        
    if ( bStop ), break, end; % Check for stopping condition
        
    if ( bVerbose )
        figure( 2 ), clf, hist( xNext(:), 40 ), pause( 2 );
    end
    
    % Update xCurr
    xCurr = xNext;
    
    xNorm = [ xNorm, norm( xCurr(:), 2 ) ];
    
end

sLast = struct( 'x', xCurr, ...
    'yRough', yEstRough, ...
    'y', yEstInterp, ...
    'yDeltaNorm', yDeltaNorm( end ) );

sLandweb = struct( 'lowestObsErr', sLowestObsErr, ...
    'last', sLast, ...
    'xDeltaNorm', xDeltaNorm, ...
    'yDeltaNorm', yDeltaNorm, ...
    'cComp', cComp, ...
    'c', c, ...
    'hParamRestIntv', hParamRestIntv, ...
    'hParamEstHist', hParamEstHist, ...
    'hParamEstHistInd', hParamEstHistInd, ...
    'thres', thresHist, ...
    'options', sOptions, ...
    'x0Real', x0Real );
  




        