% Function based on visual3, except we focus only on the cslice option.
% Args: A, fig_vec(3), b_hist_logy

function my_cslice( varargin )

% Defaults
N_HIST_BINS     = 1e2;

A               = [];

figVecLen       = 3;
figVec          = -1 * ones( 1, figVecLen );
bHistLogy       = true;


% Parse arguments
nArgInd = floor( length( varargin ) / 2 );
if ( 2*nArgInd ~= length( varargin ) ), error( 'expected even # args' ), end

for argInd = 1:nArgInd
    szKey = varargin{ 2*argInd - 1 };
    ip = varargin{ 2*argInd };
    switch szKey
        case 'A', A = ip;
        case 'fig_vec', figVec = ip;
        case 'b_hist_logy', bHistLogy = logical( ip );
        otherwise, error( sprintf( 'unrecognized key %s', szKey ) )
    end
end

if isempty( A ) | length( figVec ) ~= figVecLen, error( 'bad args' ), end

if figVec( 1 ) > 0
    [nHist, xHist] = hist( A(:), N_HIST_BINS );
    posInd = find( nHist > 0 );
    hist_plot_pieces( figVec(1), xHist, nHist, posInd, bHistLogy )
end

if figVec( 2 ) > 0
    figure( figVec(2) ), clf
    probplot( A(:) )
end

if figVec( 3 ) > 0

    AMax = max( A(:) );
    range = [ 0 AMax ];

    if false
        isoVals = logspace( log10( range(1)+0.33*(range(2)-range(1)) ), ...
            log10( range(2) ), 8 );
    elseif false
        isoVals = logspace( log10( range(1)+0.1*(range(2)-range(1)) ), ...
            log10( range(2)-0.1*(range(2)-range(1)) ), 8 );
    elseif true
        isoVals = [ .65295 .75459 .89629 1.0358 1.2303 1.6888 1.9786 ];
    elseif false
        isoVals = linspace( 0.1, 1, 8 );
    elseif false
        isoVals = linspace( min( A(:) ), max( A(:) ), 8 );
    end

    disp( 'iso vals:' ), disp( isoVals )

    figure( figVec(3) ), clf
    [m, n, p] = size( A );
    [X, Y, Z] = meshgrid( 1:n, 1:m, 1:p );
    contourslice( A, [], [], Z( 1, 1, : ), isoVals );
    set_graph_settings( X, Y, Z )
    
    caxis( [ min( isoVals ) max( isoVals ) ] )
    colorbar
    figfont

end

% Copied from visual3
%////////////////////

function set_graph_settings( X, Y, Z )

[m, n, p] = size( Z );

% Camera settings
%campos( [ 20 20 60 ] );     % For 128x128x32 image
camva( 'auto' )
camproj orthographic
% camproj perspective

% Grid
if true, axis tight, end
if false, axis( [1 128 1 128 1 32] ), end
daspect( [1 1 1] )
box on

% Colour settings
% set( gcf, 'Color', [.5 .5 .5], 'Renderer', 'zbuffer' )
% set( gca, 'Color', 'black', 'XColor', 'white', 'YColor', 'white', 'ZColor', 'white' )
% set( gcf, 'Color', 'white', 'Renderer', 'zbuffer' )
% set( gca, 'Color', 'black', 'XColor', [.5 .5 .5], 'YColor', [.5 .5 .5], 'ZColor', [.5 .5 .5] )
% xlabel( 'x' ), ylabel( 'y' ), zlabel( 'z' )

refresh


% To plot a histogram that's possible disconnected, i.e., in pieces
%//////////////////////////////////////////////////////////////////

function hist_plot_pieces( nFig, x, y, posInd, bPlotLogy )

bHoldSet = 0;

figure( nFig ), clf, hold off

nPosInd = length( posInd );

posVec = [];

for ind = 1:nPosInd
    
    if isempty( posVec )
        posVec = [ ind ];
        continue;
    end
    
    if ( posInd( ind ) - 1 > posInd( posVec(end) ) )
        
        % Ok, we already have a piece in posVec, so plot that
        posIndPiece = posInd( posVec );
        if bPlotLogy
            semilogy( x( posIndPiece ), y( posIndPiece ), 'kx-', 'LineWidth', 2 )
        else
            plot( x( posIndPiece ), y( posIndPiece ), 'kx-', 'LineWidth', 2 )
        end
        
        if ~bHoldSet
            hold on
            bHoldSet = 1;
        end
        
        % Reset posVec
        posVec = [ ind ];
       
    else
        % This index, when added to posVec, still forms a contiguous piece
        posVec = [ posVec, ind ];
    end
        
    % DEBUG
%     posVec
%     posInd( posVec )
    
end

% Plot the final piece
if ~isempty( posVec )
    posIndPiece = posInd( posVec );
    if bPlotLogy
        semilogy( x( posIndPiece ), y( posIndPiece ), 'kx-', 'LineWidth', 2 )
    else
        plot( x( posIndPiece ), y( posIndPiece ), 'kx-', 'LineWidth', 2 )
    end
end    