% // 2008-06-26 // Home-brew decimation in 3d. 
%
% Args: V0, decFac, firOrder
% Output: V
%//////////////////////////////////////////////////////////////////////////

function V = mydecimate3( V0, decFac, firOrder )

if ( ndims( V0 ) ~= 3 )
    error( 'Expect V0 to have dim 3' );
end

if ( length( decFac ) ~= 3 | length( firOrder ) ~= 3 )
    error( 'Expect decFac and firOrder to have length 3' );
end

V0Size = size( V0 );
sizeRed = floor( V0Size ./ decFac );
sizeMult = decFac .* sizeRed;

% SANITY
if ( any( sizeMult > V0Size ) )
    error( 'sizeMult cannot exceed V0Size' );
end

tmp1 = zeros( sizeRed( 1 ), V0Size( 2 ), V0Size( 3 ) );

for yInd = 1:V0Size( 2 )
    for zInd = 1:V0Size( 3 )
        tmp1( :, yInd, zInd ) = decimate( squeeze( V0( 1:sizeMult(1), yInd, zInd ) ), ...
            decFac( 1 ), firOrder( 1 ), 'fir' );
    end
end

tmp2 = zeros( sizeRed( 1 ), sizeRed( 2 ), V0Size( 3 ) );

for xInd = 1:sizeRed( 1 )
    for zInd = 1:V0Size( 3 )
        tmp2( xInd, :, zInd ) = decimate( squeeze( tmp1( xInd, 1:sizeMult(2), zInd ) ), ...
            decFac( 2 ), firOrder( 2 ), 'fir' );
    end
end

V = zeros( sizeRed( 1 ), sizeRed( 2 ), sizeRed( 3 ) );

for xInd = 1:sizeRed( 1 )
    for yInd = sizeRed( 2 )
        V( xInd, yInd, : ) = decimate( squeeze( tmp2( xInd, yInd, 1:sizeMult(3) ) ), ...
            decFac( 3 ), firOrder( 3 ), 'fir' );
    end
end
    
