% // 2008-06-01 // Home-brew downsampling in 3d. Only interpolate in
% the z direction, and assume that the x and y points require no
% d/s.
%
% Args: V0, xSel, ySel, zSel
% Output: V
%//////////////////////////////////////////////////////////////////////////

function V = mydownsamp3( V0, xSel, ySel, zSel )

VTmp = V0( xSel, ySel, : );
V = zeros( length( xSel ), length( ySel ), length( zSel ) );

for zSelInd = 1:length( zSel )
    zSlice = zSel( zSelInd );
    if ( zSlice == round( zSlice ) )
        % No need to interpolate
        V( :, :, zSelInd ) = VTmp( :, :, zSlice );
    else
        % Do a linear interpolation if necessary
        zSliceFloor = floor( zSlice );
        zSliceCeil = ceil( zSlice );
        if ( ( zSliceFloor < 1 ) | ( zSliceCeil > size( VTmp, 3 ) ) )
            error( 'zSlice{Floor,Ceil} out of bounds' );
        end
        V( :, :, zSelInd ) = ( zSlice-zSliceFloor ) * VTmp( :, :, zSliceFloor ) + ...
            ( zSliceCeil-zSlice ) * VTmp( :, :, zSliceCeil );
    end                
end