% Feb 20, 2007. This script is to run the oracular l_2 solution. I.e., we
%               assume that the non-zero positions in \theta are KNOWN, and
%               we simply solve for the least-squares solution.
%//////////////////////////////////////////////////////////////

%%% The first part is copied from try_stomp.m

% Cell array of strings copied from test_ll.m
SBL_PHI_FILE = { '/mnt/extdrv/mting/work/f05/icip06/sbl_gaussian_psf.mat', ...
    '~/work/w06/tip/2d/1uud/psf_a/sbl_mrfm_psf2d64.mat', ...
    '~/work/w06/tip/2d/benz/psf_a/sbl_mrfm_psf2d64.mat', ...
    '~/work/w06/tip/2d/benz/psf_b/sbl_mrfm_psf2d64.mat' };

objDim = [32 32];

if ( exist( 'yi', 'var' ) & exist( 'y0', 'var' ) & ...
     exist( 'xtrue', 'var' ) )
    yiVec = yi(:);
    y0Vec = y0(:);
    xtrueVec = xtrue(:);
else
    error( 'Missing variables' );
end   

load( SBL_PHI_FILE{1} ); % Load SBL_PHI into the workspace

% Find the non-zero locations in xtrueVec
xtrueVecNzLoc = find( abs( xtrueVec ) > 1e3*eps );

% Select the columns of SBL_PHI corresponding to the locations of
% xtrueVecNzLoc.
SBL_PHI_SELCOL = SBL_PHI( :, xtrueVecNzLoc );

nzSolVec = SBL_PHI_SELCOL \ yiVec;
solVec = zeros( size( xtrueVec ) );
solVec( xtrueVecNzLoc ) = nzSolVec;
sol = reshape( solVec, objDim );

%%% This part below is also copied from try_stomp.m

if true % Plot comparison of solVec and xtrue
    figure( 1 ), clf;
    plot( solVec, 'ks' );
    hold on, plot( xtrue(:), 'm<' );
    legend( 'solVec', 'xtrue', 0 );
    pause( 2 );
end

% Print comparison of solVec and xtrue
diff = solVec - xtrue(:);
disp( sprintf( 'diff of solVec vs. xtrue: l_2=%g, l_inf=%g', ...
    norm( diff, 2 ), norm( diff, inf ) ) );
