% // 2008-05-29 // Script to convert the data given in the ASCII files into
% a matrix/vector of observations. Likewise, do the same for the psf.

if ~exist( 'tmv1Data', 'var' )
    tmv1Data = load('-ascii','ascii array for Hero - TMV1.txt');
end

if ~exist( 'psfData', 'var' )
    psfData = load('-ascii','ascii array for Hero - PSF.txt');
end

tmv1Dim = [ 60 32 5 ];
tmv1Mat = reshape( tmv1Data, tmv1Dim );

% The first slice of tmv1Mat is all zeros. Get rid of it. Also, reverse the
% order of the slices.
tmv2Mat = zeros( tmv1Dim(1), tmv1Dim(2), tmv1Dim(3) );
for zSliceInd = 5:-1:1
    tmv2Mat( :, :, 5-zSliceInd+1 ) = tmv1Mat( :, :, zSliceInd );
end

sPsf.min = min( psfData );
sPsf.max = max( psfData );

% Notice that we swap around 1<-->3. This is because, in psfData, the first
% column indexes z and the third column indexes x. 
psfMat = zeros( sPsf.max(3)-sPsf.min(3)+1, ...
    sPsf.max(2)-sPsf.min(2)+1, ...
    sPsf.max(1)-sPsf.min(1)+1 );


% Go through psf and populate psfMat
for rowInd = 1:size( psfData, 1 )
    psfMat( psfData( rowInd, 3 )-sPsf.min(3)+1, ...
            psfData( rowInd, 2 )-sPsf.min(2)+1, ...
            psfData( rowInd, 1 )-sPsf.min(1)+1 ) = psfData( rowInd, 4 );
end

% Normalize psfMat so that its max element is 1
psfMatOrig = psfMat;
psfMat = psfMatOrig / sPsf.max(4);


