% Dec 19, 2007. Script to use SolveStOMP. ASSUME that base.mat has been
%               loaded, so that all of the variables used are defined.
%///////////////////////////////////////

% Cell array of strings copied from test_ll. Retain only the first entry,
% which is all we need here.
SBL_PHI_FILE = { '/media/disk/mting/work/f05/icip06/sbl_gaussian_psf.mat' };

objDim = [32 32];

maxStompIters = 50;

TRY_STOMP_COL_THRES = 1e-6;

sThresCtl = struct( 'cfdr', 0.5, ...
    'cfar', 1/maxStompIters * 1, ...
    'szSel', 'cfar' );

if ( exist( 'yi', 'var' ) & exist( 'y0', 'var' ) )
    yiVec = yi(:);
    y0Vec = y0(:);
else
    error( 'Missing variables' );
end
    
if ( false )
    % Create an anonymous function for the implicit operator of A. This is
    % because there's an additional argument of class Fatrix that is
    % needed.
    implicitAHandle = @( mode, m, n, x, I, dim ) ( implicitA( Gfft, objDim, ...
        mode, m, n, x, I, dim ) );

    [ solVec, numIters ] = SolveStOMP( implicitAHandle, y0Vec, prod(objDim), ...
        'FDR', 0.5, maxStompIters, 1 );
end


load( SBL_PHI_FILE{1} ); % Load SBL_PHI into the workspace

% Normalize X matrix so that it has unit column norm
%%%

[XN, XColumnNorm] = lars_normx( 'x', SBL_PHI );

% Check that all elements of XColumnNorm are equal
for i = 2:length(XColumnNorm)
    if abs( XColumnNorm(i) - XColumnNorm(1) ) > TRY_STOMP_COL_THRES
        error( 'XColumnNorm not constant' );
    end
end

disp( sprintf( 'll_small: column norm is %g', XColumnNorm( 1 ) ) );

switch sThresCtl.szSel
    case 'cfdr'
        [ solVec, numIters ] = SolveStOMP( XN, yiVec, prod(objDim), ...
            'FDR', sThresCtl.cfdr, maxStompIters, 1 );
    case 'cfar'
        [ solVec, numIters ] = SolveStOMP( XN, yiVec, prod(objDim), ...
            'FAR', sThresCtl.cfar, maxStompIters, 1 );
    otherwise
        error( sprintf( 'Unrecognized sThresCtl.szSel: %s', ...
            sThresCtl.szSel ) );
end

disp( sprintf( 'numIters = %g', numIters ) );

% Scale solVec: recall that we used XN in the reconstruction, NOT SBL_PHI.
solVec = solVec ./ XColumnNorm;
sol = reshape( solVec, objDim );

if true % Plot comparison of solVec and xtrue
    figure( 1 ), clf;
    plot( solVec, 'ks' );
    hold on, plot( xtrue(:), 'm<' );
    legend( 'solVec', 'xtrue', 0 );
end

% Print comparison of solVec and xtrue
diff = solVec - xtrue(:);
disp( sprintf( 'diff of solVec vs. xtrue: l_2=%g, l_inf=%g', ...
    norm( diff, 2 ), norm( diff, inf ) ) );