% // 2008-06-16 // Plot 3d image in slices. This is an alternative to
% plotSlices, which is pretty darned cumbersom.
% Keys: x, zslices, xy_shift, nfig, b_pause, npause_sec, sz_title, b_hist
% Output: None
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function xysliceplot( varargin )

% Defaults
%/////////

X           = [];
zSlices     = [];
xyShift     = [];

nFig        = -1;
bPause      = true;
nPauseSec   = 2;

szTitle     = '';
bPlotHist   = false;

% Parse inputs
%/////////////

argNum = floor( length( varargin ) / 2 );

for argInd = 1:argNum
    szKey = varargin{ 2*argInd-1 };
    ip = varargin{ 2*argInd };
    switch szKey
        case 'x', X = ip;
        case 'zslices', zSlices = ip;
        case 'xy_shift', xyShift = ip;
        case 'nfig', nFig = ip;
        case 'b_pause', bPause = logical( ip );
        case 'npause_sec', nPauseSec = ip;
        case 'sz_title', szTitle = ip;
        case 'b_hist', bPlotHist = logical( ip );
        otherwise, error( sprintf( 'unknown key: %s', szKey ) );
    end
end

% Check initializations
if ( nFig < 0 ), nFig = gcf; end
if ( ndims( xyShift ) ~= 2 ), error( 'dim of xyshift should be 2' ), end
if ( ndims( X ) ~= 3 ), error( 'dim of X should be 3' ), end
    
numZSlice = size( X, 3 );

if ( isempty( zSlices ) )
    zSlices = 1:numZSlice;
end

if false
    fprintf( 1, 'Starting to plot xy slices\n' );
end

for indZSlice = zSlices
    figure( nFig );
    
    if ( bPlotHist )
        subplot( 121 );
    end
    
    xySlice = squeeze( X( :, :, indZSlice ) );
    
    if ~isempty( xyShift )
        imagesc( circshift( xySlice, xyShift ) );
    else
        imagesc( xySlice );
    end
    
    %colormap gray;
    colormap jet;
    colorbar;
    axis image;
    
    if ~isempty( szTitle )
        title( sprintf( '%s: slice %2g/%2g', szTitle, indZSlice, length( zSlices ) ) );
    else
        title( sprintf( 'slice %2g/%2g', indZSlice, length( zSlices ) ) );
    end
    
    if ( bPlotHist )
        subplot( 122 );
        hist( xySlice(:), 20 );
    end
    
    if ( bPause )
        pause;
    else
        pause( nPauseSec );
    end
end


