%
% Creates a sparse matrix that represents convolution with the given psf.
% Assumes a 2 dimensional psf. 
%
% Usage is: Phi = create_smatrix_from_psf2( hForwardOp, bMex )
%
% Note that hForwardOp must be zero padded. Set bMex=1 to access mex
% function for faster runtime.
%
function Phi = create_smatrix_from_psf2( hForwardOp, bMex )

if ( ( nargin == 2 ) & ( bMex == 1 ) )
	% Try mex method
	[Phi, sPhiInfo] = psf2mat2( hForwardOp );
else
	% Implement in Matlab

	if ( ndims( hForwardOp ) ~= 2 ), error( 'expect hForwardOp to be 2d' ), end;
	N = size( hForwardOp, 1 );
	M = size( hForwardOp, 2 );
	Hsize = N*M;
	thres = 1e2*eps;

	% Variables to keep track of non-zero elements
	nzNum = 0;
	nzI = [];
	nzJ = [];
	nzS = [];

	for i = 1:Hsize
    	for j = 1:Hsize
			% Add 1 because Matlab arrays are indexed starting at 1
        	%hInd1 = 1 + mod( mod(i-1,N) - mod(j-1,N), N );
			% This produces the same result as above
        	hInd1 = 1 + mod( mod(i,N) - mod(j,N), N );
			hInd2 = 1 + mod( floor((i-1)/N) - floor((j-1)/N), M );
        	if ( abs( hForwardOp( hInd1, hInd2 ) ) > thres )
            	nzNum = nzNum + 1;
				nzI = [nzI; i];
				nzJ = [nzJ; j];
				nzS = [nzS; hForwardOp( hInd1, hInd2 )];
        	end
    	end
	end

	disp( sprintf( 'nzNum=%g', nzNum ) );

	% Allocate space for sparse system matrix
	Phi = sparse( nzI, nzJ, nzS, Hsize, Hsize );

end
