% Here, we want to apply the J&S method to first denoise (H*theta), and
% then apply SBL to recover theta.

function sRestore = dec15_xp1( varargin )

% Default restore options

sRestoreOptions = struct( 'B_SBL_PLOT_SANITY', 0, ...
	'Phi', [], ...
    'STOPPING_COND_LAST_FAC', 1, ...
    'GAMMA0_SELECTOR', 'ones', ...
    'GAMMA0', [], ...
	'LAST_MUHAT', [] );

% Process input arguments

lenVarargin = length( varargin );
numArgsIn = floor( lenVarargin/2 );

if 2*numArgsIn ~= lenVarargin, error( 'even ip args expected' ), end

for argInInd = 1:numArgsIn
    
    szKey = lower( varargin{2*argInInd-1} );    
    switch szKey
        case 'forward_op', hForwardOp = varargin{2*argInInd};
        case 'noisy_output', t = varargin{2*argInInd};
        case 'noise_sigma', noiseSigma = varargin{2*argInInd};
        case 'theta_true', wTrue = varargin{2*argInInd}; 
        case 'u0', u0 = varargin{2*argInInd};
                 
        case 'stopping_cond', sRestoreOptions.STOPPING_COND = varargin{2*argInInd};
        case 'stopping_cond_tol', sRestoreOptions.STOPPING_COND_TOL = varargin{2*argInInd};
        case 'stopping_cond_win', sRestoreOptions.STOPPING_COND_WIN = varargin{2*argInInd};
        case 'stopping_cond_last_fac', sRestoreOptions.STOPPING_COND_LAST_FAC = varargin{2*argInInd};
        case 'num_em_iters', sRestoreOptions.NUM_EM_ITERS = varargin{2*argInInd};    
		case 'phi', sRestoreOptions.Phi = varargin{2*argInInd};
        case 'gamma0_selector', sRestoreOptions.GAMMA0_SELECTOR = varargin{2*argInInd};
        case 'gamma0', sRestoreOptions.GAMMA0 = varargin{2*argInInd};
		case 'last_muhat', sRestoreOptions.LAST_MUHAT = varargin{2*argInInd};
            
        otherwise
            error( sprintf( 'unrecognized option %s', szKey ) );
    end
end

n = length( t(:) );
%empSdev = std( t(:) );
empSdev = noiseSigma; % We know the noise variance exactly, so use that
tUniv = sqrt( 2*log(n) );

sBoundingBox = struct( 'loBoundScale', [1e-3, 1e-3, 1e-3], ...
    'hiBoundScale', [3, tUniv, 1], ...
    'loBoundScaleMin', 1e-6, ...
    'hiBoundScaleMax', 1e3, ...
    'loBoundInit', [1/empSdev, empSdev, empSdev], ...
    'hiBoundInit', [1/empSdev, empSdev, empSdev] ...
    );

%%% Run J&S denoising method
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

paramLowerBound = sBoundingBox.loBoundScale .* sBoundingBox.loBoundInit;
paramUpperBound = sBoundingBox.hiBoundScale .* sBoundingBox.hiBoundInit;

pLen = length( paramLowerBound(:) );
tolMatrix = [ 1e-4*ones(1,pLen); ...
	1e-4*( paramUpperBound(:)' - paramLowerBound(:)' ); ...
	0.1*paramLowerBound(:)'
];
tol = min( tolMatrix );
disp( 'tol' ), disp( tol );

maxIters = 3e2;
debugFlag = 0;

sOptim2 = probjs_wrapper( 'y', t(:), ...
    'ylength', length( t(:) ), ...
    'lowerbound', paramLowerBound(1:2), ...
    'upperbound', paramUpperBound(1:2), ...
    'tol', tol(1:2), ...
    'maxiters', maxIters, ...
    'sigma', noiseSigma, ...
    'debugflag', debugFlag ...
    );

% Apply the soft threshold function with t computed above
HThetaHat = soft_thres( t(:), sOptim2.voptim(2) );

[uHatE1, uHatE2] = compute_est_stats( u0(:), sOptim2.voptim(2), noiseSigma );
uHatVar = uHatE2 - uHatE1.^2;

%%% Run SBL
%%%%%%%%%%%

sRestore = sbl_restore( 'forward_op', hForwardOp, ...
    'noisy_output', t(:), ...   
    'theta_true', wTrue(:), ...
    'num_em_iters', sRestoreOptions.NUM_EM_ITERS, ...
    'stopping_cond', sRestoreOptions.STOPPING_COND, ...
    'stopping_cond_tol', -1, ...
    'stopping_cond_win', sRestoreOptions.STOPPING_COND_WIN, ...
    'stopping_cond_last_fac', sRestoreOptions.STOPPING_COND_LAST_FAC, ...
    'gamma0_selector', sRestoreOptions.GAMMA0_SELECTOR, ...
    'gamma0', sRestoreOptions.GAMMA0, ...
    'last_muhat', sRestoreOptions.LAST_MUHAT, ...
    'phi', sRestoreOptions.Phi ...
    );

% If sRestore already exists from SBL, add to it. Otherwise, create the
% structure.
if exist( 'sRestore', 'var' )
    sRestore.uHatE1 = uHatE1(:);
    sRestore.uHatE2 = uHatE2(:);
    sRestore.uHatVar = uHatVar(:);
    sRestore.HThetaHat = HThetaHat(:);
else
    sRestore = struct( 'uHatE1', uHatE1, 'uHatE2', uHatE2', 'uHatVar', uHatVar, ...
        'HThetaHat', HThetaHat );
end

% Estimate the variance of the soft-threshold estimator
function [uHatE1, uHatE2] = compute_est_stats( u, tHat, sigma )
u = u(:);
% Sanity
% size( u )
% size( tHat )
% size( sigma )
uHatE1 = u + 0.5*(u-tHat).*erf( (u-tHat)/sqrt(2)/sigma ) - ...
    0.5*(u+tHat).*erf( (u+tHat)/sqrt(2)/sigma );
uHatE2 = sigma^2 + tHat^2 + (u.*u) + ...
    sigma/sqrt(2*pi)*(u-tHat).*exp( -(u-tHat).^2/2/sigma^2 ) - ...
    sigma/sqrt(2*pi)*(u+tHat).*exp( -(u+tHat).^2/2/sigma^2 ) + ...
    0.5*( erf( (u-tHat)/sqrt(2)/sigma ) - erf( (u+tHat)/sqrt(2)/sigma ) ).*( (u-tHat).^2 + sigma^2 );
uHatE1 = uHatE1(:);
uHatE2 = uHatE2(:);
