% [C, fMesh, sMesh] = dec5( varargin )
% Valid keys: 'a', 'w', 'alpha', 'b_plot'
function [C, fm, sm] = dec5( varargin )

global s alpha a f w

% Defaults
a = 1;
w = 0.1;
alpha = 0.1;
B_PLOT = 1;
B_DEBUG = 1;

% Process arguments
numArgs = length( varargin )/2;
for argInd = 1:numArgs
    szKey = varargin{2*argInd-1};
    switch lower( szKey )
        case 'a', a = varargin{2*argInd};
        case 'w', w = varargin{2*argInd};
        case 'alpha', alpha = varargin{2*argInd};
        case 'b_plot', B_PLOT = varargin{2*argInd};
        otherwise, error( sprintf( 'unrecognized key: %s', szKey ) );
    end
end

disp( sprintf( 'a=%g, w=%g, alpha=%g', a, w, alpha ) );

fMesh = struct( 'lims', [ -1, 1 ], 'npts', 30 );
fMesh.vect = linspace( fMesh.lims(1), fMesh.lims(2), fMesh.npts );
sMesh = struct( 'lims', [ -1, 1 ], 'npts', 50 );
sMesh.vect = linspace( sMesh.lims(1), sMesh.lims(2), sMesh.npts );

% Plot the margnized distribution
pMarg = zeros( size( sMesh.vect ) );
for sInd = 1:sMesh.npts
    try
        pMarg(sInd) = probJs( 'type', 'marg', 'a', a, 'w', w, 'sigma', alpha, 'x', sMesh.vect(sInd) );
    catch
        error( sprintf( 'plotting marg dist: %s', lasterr ) );
    end
end
figure( 1 ), plot( sMesh.vect, pMarg ), title( 'pMarg' );


C = zeros( fMesh.npts, sMesh.npts );

for fInd = 1:fMesh.npts
    for sInd = 1:sMesh.npts
        f = fMesh.vect( fInd );
        s = sMesh.vect( sInd );
        
        if B_DEBUG
            disp( sprintf( 'f=%g, s=%g', f, s ) );
        end
        
        ftInterval = lowerBoundExp( s, a, alpha, 2*alpha^2*1e2 );
        disp( sprintf( 'ftInterval=[%g,%g]', ftInterval(1), ftInterval(2) ) );
        % If f isn't in the interval, extend ftInterval so that it is
        if f > ftInterval(2), ftInterval(2) = f + ftInterval(2);
        elseif f < ftInterval(1), ftInterval(1) = f + ftInterval(1);
        end
        disp( sprintf( 'modified to [%g,%g]', ftInterval(1), ftInterval(2) ) );
                
        try
            errL1_1 = quad( @privFunc, -5, 5 );
            errL1_2 = quad( @privFunc, ftInterval(1), ftInterval(2) );
            disp( sprintf( 'errL1_1=%g, errL1_2=%g', errL1_1, errL1_2 ) );
            errL1 = errL1_2;
        catch
            error( sprintf( 'quad error: %s', lasterr ) );
        end
           
        if ~isfinite( errL1 )
            error( sprintf( 'non-finite errL1=%g ???', errL1 ) );
        end
        
        C( fInd, sInd ) = 1/pMarg(sInd) * ( (1-w)*abs(f)*normpdf(s,0,alpha) + w*errL1 );
    end
end

if B_PLOT
    figure( 2 ), surf( sMesh.vect, fMesh.vect, C );
    xlabel( 's' ), ylabel( 'f' ), grid on;
end

if nargout == 3
    fm = fMesh;
    sm = sMesh;    
end

function y = privFunc( ft )
global s alpha a f
tmp1 = normpdf( ft, s, alpha );
tmp2 = 0.5*a*exp(-a*abs(ft));
tmp3 = abs(ft-f);
y = tmp1 .* tmp2 .* tmp3;

function ftInterval = lowerBoundExp( s, a, alpha, const )
foo = alpha^2*a;
% Case 1
D1 = foo^2 - 2*s*foo + const;
if D1 < 0
    ftInterval(2) = 0;
else
    c1Sol1 = s - foo + sqrt(D1);
    c1Sol2 = s - foo - sqrt(D1);
    c1SolMax = max( [c1Sol1, c1Sol2] );
    if c1SolMax <= 0
        ftInterval(2) = 0;
    else
        ftInterval(2) = c1SolMax;
    end
end
% Case 2
D2 = foo^2 + 2*s*foo + const;
if D2 < 0
    ftInterval(1) = 0;
else
    c2Sol1 = s + foo + sqrt(D2);
    c2Sol2 = s + foo - sqrt(D2);
    c2SolMin = min( [c2Sol1, c2Sol2] );
    if c2SolMin >= 0
        ftInterval(1) = 0;
    else
        ftInterval(1) = c2SolMin;
    end
end

% ft = linspace( -5, 5, 100 );
% figure( 2 ), plot( ft, ft.^2-2*s*ft+2*alpha^2*a*abs(ft)+s^2, 'm' );
