
% Script to find marginal ML estimate of "a" and "t" via grid method

aLower = propImage.a * 0.5;
aUpper = propImage.a * 1.5;
tLower = propImage.t * 0.5;
tUpper = propImage.t * 1.5;

requiredADelta = 1e-3*propImage.a;
numAPts = ceil( (aUpper-aLower)/requiredADelta );
requiredTDelta = 1e-3*propImage.t;
numTPts = ceil( (tUpper-tLower)/requiredTDelta );

aVec = linspace( aLower, aUpper, numAPts );
tVec = linspace( tLower, tUpper, numTPts );
llVal = zeros( numTPts, numAPts );

warning off;
for tInd = 1:numTPts
    for aInd = 1:numAPts
        w = convert_tsigma2w( aVec(aInd), tVec(tInd), propImage.sigma, 'median' );
        llVal(tInd,aInd) = jsLogLik( fnoisy, aVec(aInd), w, propImage.sigma );
    end
end
warning on;

% Find the max of matrix llVal
[llValColMax, llValColMaxInd] = max( llVal, [], 1 );
[llValMax, aIndHat] = max( llValColMax );
tIndHat = llValColMaxInd( aIndHat );

aHatGridReal = aVec( aIndHat(1) );
tHatGridReal = tVec( tIndHat(1) );

disp( sprintf( 'estimate of a=%g, t=%g', aHatGridReal, tHatGridReal ) );