%
% hs_strat4: vary phi1 and phi2. Dunno if we're really going to use this,
% as the grid seems to be too coarse to gain any benefit from uncovering a
% lower SURE value.
%
% The following elements of sParam are expected:
%    X, XColumnNorm, nPhi1, nPhi2, numEmIters, szPhi1Vec
%    bigCHatHist
%    psf, y, noiseSigma, betaTrue
%    phi0, phi0LarsIndex1, phi0Cost, phi0Beta

function sOut = hs_strat4( varargin )

global XOrig GX % The original X, and the Gram matrix of X = X'*X

ipImgDim = [];
opImgDim = [];

% Process input arguments

lenVarargin = length( varargin );
numArgsIn = floor( lenVarargin/2 );

if 2*numArgsIn ~= lenVarargin, error( 'even ip args expected' ), end

for argInInd = 1:numArgsIn    
    szKey = lower( varargin{2*argInInd-1} );    
    switch szKey        
        case 'sparam', sParam = varargin{2*argInInd};           
		case 'ip_img_dim', ipImgDim = varargin{2*argInInd};
		case 'op_img_dim', opImgDim = varargin{2*argInInd};
        otherwise
            error( sprintf( 'unrecognized option %s', szKey ) );
    end
end

% Initialize

if ~exist( 'sParam', 'var' ), error( 'sParam uninitialized' ), end;

phi1Init = sParam.phi0(1);
noiseSigma = sParam.noiseSigma;
N = length( sParam.y );

% Must make sure that the i/p and o/p image dimensions are identical
if  ( length( ipImgDim ) ~= length( opImgDim ) ) | ...
	( length( ipImgDim ) == 0 ) | ...
	( length( opImgDim ) == 0 )
	error( 'i/p and o/p image dimensions must have equal length>0' );
else
	for i = 1:length( ipImgDim )
		if ipImgDim( i ) ~= opImgDim( i )
			error( sprintf( 'i/p and o/p image do not agree for dim %g', i ) );
		end
	end
end

phi1Vec = [];
phi2FracVec = [];

% Precompute XOrig and the Gram matrix of X
disp( 'Precomputing XOrig and G(X)' );

% Recall the columns of X were normalized to unit norm. Hence must
% premultiply by the norms to restore the original X.
XOrig = sParam.X .* repmat( sParam.XColumnNorm(:)', N, 1 );

% The Gram matrix of the *original* X
GX = XOrig' * XOrig;

%
% First, construct phi1Vec
%

if mod( sParam.nPhi1, 2 ) == 0 % Even
    numPaddingBefore = sParam.nPhi1/2;
    numPaddingAfter = sParam.nPhi1/2 - 1;

else % Odd
    numPaddingBefore = (sParam.nPhi1-1)/2;
    numPaddingAfter = numPaddingBefore;
end

switch sParam.szPhi1Vec

    case 'max_alpha'

        [sDIter, sDIterOptions] = emgauss_restore( 'forward_op', sParam.psf, ...
            'noisy_output', reshape( sParam.y, opImgDim ), ...
            'noise_sigma', noiseSigma, ...
            'theta_true', reshape( sParam.betaTrue, ipImgDim ), ...
            'm_selector', 'daub_hard_soft', ...
            'num_em_iters', 1, ...
            'num_max_grad_search_failures', -1, ...
            'theta0_selector', 'Hty2', ...
            'theta0', [], ...
            'a_val', -1, ...
            'w_val', -1, ...
            't_val', -1, ...
            'sparse_ind', 'w', ...
            'stopping_cond', 'fixed', ...
            'stopping_cond_tol', noiseSigma, ...
            'stopping_cond_win', 3e3, ...
            'stopping_cond_last_fac', 2, ...
            'thetastar', 1, ...
            'soft_thres_t', -1, ...
            'hparam_rest_interval', 1e2, ...
            'bseparate_hparam_est', 0, ...
            'lambda', 2*phi1Init, ...
            'hard_soft_frac', 1 ...
            );

        phi1Max = sqrt(2*log(M))*sDIterOptions.alpha;
        disp( sprintf( 'phi1Max=%g', phi1Max ) );
        if ( phi1Init >= phi1Max )
            error( sprintf( 'phi1Init=%g >= phi1Max=%g', phi1Init, phi1Max ) );
        end
        
        phi1Delta1 = (phi1Max - phi1Init)/numPaddingAfter;
        phi1Delta2 = 0.9*phi1Init/numPaddingBefore
        phiDelta = min( [phi1Delta1, phi1Delta2] );
        phi1Vec = (phi1Init-phiDelta*numPaddingBefore):phiDelta:(phi1Init+phiDelta*numPaddingAfter);

    case 'max_chat'

        % Make sure that we reach the first index of bigCHatHist
        indDelta = floor( (sParam.phi0LarsIndex1-1)/numPaddingBefore );
        if ( indDelta < 1 ), error( 'Cannot find a suitable indDelta' ), end;

        % DEBUG
        disp( sprintf( 'padding before/after=%g/%g, delta=%g, phi0LarsIndex1=%g', ...
            numPaddingBefore, numPaddingAfter, indDelta, sParam.phi0LarsIndex1 ) );

        indFirst = sParam.phi0LarsIndex1 - numPaddingBefore*indDelta;
        indLast = sParam.phi0LarsIndex1 + numPaddingAfter*indDelta;

        if ( indFirst < 1 ), error( 'indFirst too small' ), end;
        if ( indLast > length( sParam.bigCHatHist ) ), error( 'indLast too large' ), end;
        phi1Vec = sParam.XColumnNorm(1) * sParam.bigCHatHist( indFirst:indDelta:indLast );

        % DEBUG
        %disp( 'phi1Vec:' ), disp( phi1Vec ), disp( 'Pausing' ), pause;

    otherwise
        
        error( 'unrecognized szPhi1Vec' );
end

% SANITY
if length( phi1Vec ) ~= sParam.nPhi1
    error( 'length of phi1Vec is incorrect' );
end

%
% Next, construct phi2FracVec
%

% SANITY
if ( phi1Init < 1e6*eps ), error( 'phi1Init = 0' ), end;

phi0Ratio = sParam.phi0(2)/sParam.phi0(1);
% DEBUG
disp( sprintf( 'phi0Ratio=%g', phi0Ratio ) );

% The valid range of phi2 is 0 <= phi2/phi1 <= 1. Note that:
%    phi2/phi1=1 => Soft threshold
%    phi2/phi0=0 => Hard threshold

phi2FracVec = linspace( 0, 1, sParam.nPhi2-1 );
indPhi0 = find( phi2FracVec == phi0Ratio );
if isempty( indPhi0 )
    phi2FracVec = [ phi2FracVec, phi0Ratio ];
else
    if ( indPhi0 == ( sParam.nPhi2-1 ) )
        phi2FracVec = [ phi2FracVec, (phi2FracVec(indPhi0-1)+phi2FracVec(indPhi0))/2 ];
    else
        phi2FracVec = [ phi2FracVec, (phi2FracVec(indPhi0)+phi2FracVec(indPhi0+1))/2 ];
    end
end
phi2FracVec = sort( phi2FracVec );

% Initialize the element that coincides with phi0
indPhi0 = find( phi2FracVec == phi0Ratio );
if length( indPhi0 ) ~= 1
    error( 'did not find exactly one phi0Ratio in phi2FracVec' );
end

% Matrix of costs
matCost4 = zeros( length( phi1Vec ), length( phi2FracVec ) );
matCost4Approx = zeros( size( matCost4 ) );

betaHatBest4Ind = [ numPaddingBefore + 1, indPhi0 ];
% SANITY
if ( ( phi1Vec( betaHatBest4Ind(1) ) ~= sParam.phi0(1) ) | ...
     ( phi2FracVec( betaHatBest4Ind(2) ) ~= phi0Ratio ) )
    error( 'sanity on betaHatBest4Ind failed' );
end

matCost4( betaHatBest4Ind(1), betaHatBest4Ind(2) ) = sParam.phi0Cost;
matCost4Approx( betaHatBest4Ind(1), betaHatBest4Ind(2) ) = sParam.phi0Cost;

betaHatBest4 = sParam.phi0Beta;

% Go through and sweep phi1 and phi2

for i = 1:sParam.nPhi1
    
    for j = 1:sParam.nPhi2

        % Skip
        if ( ( i == ( numPaddingBefore+1) ) & ( j == indPhi0 ) ), continue, end;
        
        disp( sprintf( '(%g,%g) Doing phi1=%g, phi2 frac=%g', i, j, phi1Vec(i), phi2FracVec(j) ) );
        pause( 5 );

        % Compute using Daubechies' iterations
        [sDIter, sDIterOptions] = emgauss_restore( 'forward_op', sParam.psf, ...
            'noisy_output', reshape( sParam.y, opImgDim ), ...
            'noise_sigma', noiseSigma, ...
            'theta_true', reshape( sParam.betaTrue, ipImgDim ), ...
            'm_selector', 'daub_hard_soft', ...
            'num_em_iters', sParam.numEmIters, ...
            'num_max_grad_search_failures', -1, ...
            'theta0_selector', 'Hty2', ...
            'theta0', [], ...
            'a_val', -1, ...
            'w_val', -1, ...
            't_val', -1, ...
            'sparse_ind', 'w', ...
            'stopping_cond', 'fixed', ...
            'stopping_cond_tol', noiseSigma, ...
            'stopping_cond_win', 3e3, ...
            'stopping_cond_last_fac', 2, ...
            'thetastar', 1, ...
            'soft_thres_t', -1, ...
            'hparam_rest_interval', 1e2, ...
            'bseparate_hparam_est', 0, ...
            'lambda', 2*phi1Vec(i), ...
            'hard_soft_frac', phi2FracVec(j) ...
            );

        % Compute the cost function
        daubSol = sDIter.last(:);
                    
        sOut = comp_hhs_sure( 'y', sParam.y, ...
            'thetaHat', daubSol, ...
            'noiseSigma', noiseSigma, ...
            'phi1', phi1Vec(i), ...
            'phi2', phi2FracVec(j)*phi1Vec(i), ...
            'bComputeApprox', 0 );
        matCost4( i, j ) = sOut.hhsSureVal;
        matCost4Approx( i, j ) = sOut.hhsSureValApprox;

        % Check to see if we have a better solution

        if ( matCost4( betaHatBest4Ind(1), betaHatBest4Ind(2) ) > matCost4( i, j ) )
            betaHatBest4Ind = [i, j];
            betaHatBest4 = daubSol;
        end

    end
    
end

% Defines the structure of sOut

phiNew = phi1Vec( betaHatBest4Ind(1) ) * [ 1, phi2FracVec( betaHatBest4Ind(2) ) ];
    
sOutElems = { 'phi', 'phiNew'; ...
    'betaHatBest', 'betaHatBest4'; ...
    'betaHatBestInd', 'betaHatBest4Ind'; ...
    'cost', 'matCost4'; ...
    'costApprox', 'matCost4Approx'; ...
    'phi1Vec', 'phi1Vec'; ...   
    'phi2FracVec', 'phi2FracVec' ...
    };

for elemInd = 1:size( sOutElems, 1 )
    szCmd = sprintf( 'sOut.%s = %s;', ...
        char( sOutElems(elemInd,1) ), ...       
        char( sOutElems(elemInd,2) ) );
    try
        eval( szCmd );
    catch
        warning( sprintf( 'Could not execute: %s', szCmd ) );
    end
end

    


