
% Function to go through all the noise realizations for a particular SNR
% level and check that they're all the same. Arguments are: 'simfile',
% 'numtest'

function irecon_check_snr( varargin )

% Initialize
szSimFilename = '';
numSszDir = 0;
szCwd = pwd;

% Process input arguments

lenVarargin = length( varargin );
numArgsIn = floor( lenVarargin/2 );

if 2*numArgsIn ~= lenVarargin, error( 'even ip args expected' ), end

for argInInd = 1:numArgsIn
    
    szKey = lower( varargin{2*argInInd-1} );    
    switch szKey    
        case 'simfile', szSimFilename = varargin{2*argInInd};
        case 'numtest', numSszDir = varargin{2*argInInd};
        otherwise
            error( sprintf( 'unrecognized option %s', szKey ) );
    end
    
end

% Check variables
if isempty( szSimFilename ) | ( numSszDir < 1 )
    error( 'variables not properly initialized' );
end

disp( sprintf( 'Processing %g subdirs', numSszDir ) );
for indSszDir = 1:numSszDir
    sszDir{indSszDir} = sprintf( 't%d', indSszDir );
end

% Go through each subdirectory in sszDir and print out the SNR value
% contained in the sim file.

for indSszDir = 1:numSszDir
    szDir = char( sszDir( indSszDir ) );

    try
        cd( strcat( szCwd, '/', szDir ) );
    catch
        % Directory doesn't exist
        cd( szCwd );
        error( sprintf( 'Cannot change to %s', szDir ) );
    end

    try
        sData = load( szSimFilename );
    catch
        % Cannot load sim file
        cd( szCwd );
        error( sprintf( 'Cannot load sim file %s in subdir %s', szSimFilename, szDir ) );
    end

    try
        
%         disp( sprintf( 'subdir %s: SNR=%g, SNRdb=%g', szDir, sData.SNR, sData.SNRdb ) );
%         disp( sprintf( 'G_SPECIAL: %g', sData.G_SPECIAL ) );

%         figure( 1 ), clf;
%         plot( sData.sRestore.sStratOut.cost, 'rx' ), hold on;
%         plot( sData.sRestore.sStratOut.costApprox, 'bs' );
%         xlabel( sprintf( 'case %s', szDir ), 'FontSize', 16 );
%         [cMin, cMinInd] = min( sData.sRestore.sStratOut.cost )
%         [cApproxMin, cApproxMinInd] = min( sData.sRestore.sStratOut.costApprox )
%         pause;

        figure(1), clf, imagesc( sData.xtrue ), colorbar;
        figure(2), clf, imagesc( sData.yiDisplay ), colorbar;
        disp( sprintf( 'subdir %s', szDir ) );
        sData.propImage
        sData.propNoise
        disp( 'Press key' ), pause;

    catch
        % Ignore any variable access errors
        warning( sprintf( 'Did not find all variable(s): %s', lasterr ) );
    end
end

cd( szCwd ); % Change back to starting directory
