
% Function to go through several snr directories cull statistics from them.
% We want to display the mean and error bars of one standard deviation.
% Arguments are: 'resfile', 'snrdir', 'snrreal', 'spriv', 'fcncull'

function sRes = irecon_collect( varargin )

% Initialize
szCwd = pwd;

szResultFilename = '';
snrDirRange = [];
snrDirRealXion = [];
sPriv = struct;
szFcnCull = '';

B_VERBOSE = 1;

% Save the mean and std. dev. for each directory
statMean = [];
statStd = [];

metricsHist = []; % For debug purposes

% The noise std. dev. in the denoising sub-problem
alpham = [];

% Process input arguments

lenVarargin = length( varargin );
numArgsIn = floor( lenVarargin/2 );

if 2*numArgsIn ~= lenVarargin, error( 'even ip args expected' ), end

for argInInd = 1:numArgsIn
    
    szKey = varargin{2*argInInd-1};    
    ip = varargin{2*argInInd};
    
    switch szKey         
        case 'resfile', szResultFilename = ip;
        case 'snrdir', snrDirRange = ip;
        case 'snrreal', snrDirRealXion = ip;
        case 'spriv', sPriv = ip;
        case 'fcncull', szFcnCull = ip;
        otherwise
            error( sprintf( 'unrecognized option %s', szKey ) );
    end
    
end

% Check arguments
if isempty( szResultFilename ), error( 'must specify resfile option' ), end;
if isempty( snrDirRange ), error( 'must specify snrdir option' ), end;
if length( snrDirRange ) ~= length( snrDirRealXion )
    error( 'lengths of snrdir and snrreal options must be equal' );
end
if isempty( szFcnCull ), error( 'must specify fcncull' ), end;

numSszDir = length( snrDirRange );
disp( sprintf( 'Processing %d snr values', numSszDir ) );
for indSnr = 1:numSszDir
    sszDir{indSnr} = sprintf( 'snr%d', snrDirRange(indSnr) );
end

% Make sure that the directories in sszDir exist. 

for indSszDir = 1:numSszDir       
    szDir = char( sszDir( indSszDir ) );    
    try
        cd( strcat( szCwd, '/', szDir ) );
    catch
        % Directory doesn't exist
        cd( szCwd );
        error( sprintf( 'Cannot change to %s', szDir ) );
    end     
end

% Go through and run szFcnCull on each snr directory

for indSszDir = 1:numSszDir
    szDir = char( sszDir( indSszDir ) );
    try
        cd( strcat( szCwd, '/', szDir ) );
        
        if B_VERBOSE, disp( sprintf( 'irecon_collect: dir %s', szDir ) ), end;
        
        szCmd = sprintf( '[metrics,misc]=%s', szFcnCull );
        szCmd = strcat( szCmd, '(''resfile'',szResultFilename,' );
        szCmd = strcat( szCmd, '''numtest'',snrDirRealXion(indSszDir),' );
        szCmd = strcat( szCmd, '''spriv'',sPriv);' );
        
        if B_VERBOSE, disp( sprintf( 'irecon_collect: szCmd: %s', szCmd ) ), end;
        
        eval( szCmd );
        
        if ~isempty( metrics )
            metricsMean = mean( metrics );
            metricsStd = std( metrics );
            statMean = [statMean; metricsMean];
            statStd = [statStd; metricsStd];
            metricsHist = [metricsHist; metrics];
        end
        
        if B_VERBOSE
            statMean
            statStd
        end
        
        if ( ~isempty( misc ) & isfield( misc, 'alpha' ) ) 
            alpham = [alpham; misc.alpha]; 
        end
    catch
        cd( szCwd );
        error( sprintf( 'Error processing %s: %s', szDir, lasterr ) );
    end
end

cd( szCwd );

sRes = struct( 'mean', statMean, 'std', statStd, 'alpha', alpham );

if true % XXX DEBUG
    sRes.metrics = metricsHist;
end


        