
% Function to continue running various other reconstruction techniques in the
% simulation cases. This should be executed after running irecon_setup, as
% here, we assume that the image has already been generated. Usage is:
%    irecon_cont( varargin )
% where the input args are: 'simfile', 'resfile', 'script', 'numtest'

function irecon_cont( varargin )

% Initialize
szCwd = pwd;

szSimFilename = '';
szResultFilename = '';
szScript = '';
numTest = 0;

% Process input arguments

lenVarargin = length( varargin );
numArgsIn = floor( lenVarargin/2 );

if 2*numArgsIn ~= lenVarargin, error( 'even ip args expected' ), end

for argInInd = 1:numArgsIn
    
    szKey = lower( varargin{2*argInInd-1} );    
    switch szKey 
        case 'simfile', szSimFilename = varargin{2*argInInd};
        case 'resfile', szResultFilename = varargin{2*argInInd};
        case 'script', szScript = varargin{2*argInInd};
        case 'numtest', numSszDir = varargin{2*argInInd};
        otherwise
            error( sprintf( 'unrecognized option %s', szKey ) );
    end
    
end

% Check that initialization is complete
if isempty( szSimFilename ) | isempty( szResultFilename ) | isempty( szScript )
    error( 'Some variables have not been initialized' );
end
if numSszDir < 1, error( 'numTest < 1: nothing to do' ), end;

disp( sprintf( 'Processing %g subdirs', numSszDir ) );
for indSszDir = 1:numSszDir
    sszDir{indSszDir} = sprintf( 't%d', indSszDir );
end

% Make sure that the directories in sszDir exist. As well, check that
% szSimFilename exists in each directory. 

for indSszDir = 1:numSszDir       
    szDir = char( sszDir( indSszDir ) );    
    try
        cd( strcat( szCwd, '/', szDir ) );
    catch
        % Directory doesn't exist
        error( sprintf( 'Cannot change to %s', szDir ) );
    end    
    if ~exist( szSimFilename, 'file' )
        error( sprintf( 'Sim file %s does not exist in %s', szSimFilename, szDir ) );
    end
end

% Go through each directory and generate the result by running the script
% specified in szScript if szResultFilename doesn't already exist.

%SZ_M_METHOD = 'map2_fixed';
%G_SPECIAL = 0;

for indSszDir = 1:numSszDir
    szDir = char( sszDir( indSszDir ) );
    cd( strcat( szCwd, '/', szDir ) );
    
    if exist( szResultFilename, 'file' )
        warning( sprintf( 'Result file %s already exists in %s!', ...
            szResultFilename, szDir ) );
        continue; % Skip case
    end
    
    %try     
        % Must be careful in loading variables from the sim file. Do not
        % want to overwrite the variables indSszDir, numSszDir, szCwd,
        % sszDir, szDir, szResultFilename, szSimFilename, szScript.
        
        sszProtectedNames = { 'indSszDir', 'numSszDir', 'szCwd', ...
            'sszDir', 'szDir', 'szResultFilename', 'szSimFilename', ...
            'szScript', 'sSim', 'sSimVar', 'varInd', 'szCode', ...
            'SZ_M_METHOD', 'G_SPECIAL' };
        sSim = load( szSimFilename ); 
        sSimVar = fieldnames( sSim );
        for varInd = 1:length( sSimVar )            
            if sum( strcmp( sSimVar(varInd), sszProtectedNames ) ) == 0
                szCode = sprintf( '%s = sSim.%s;', char(sSimVar(varInd)), char(sSimVar(varInd)) );
                disp( 'szCode:' ), disp( szCode );
                eval( szCode );
            end
        end
        clear( 'sszProtectedNames', 'sSim', 'sSimVar', 'varInd', 'szCode' );                      
        
        eval( szScript );
        
        save( szResultFilename );
    %catch
    %    error( sprintf( 'Problem running %s in %s: %s', szScript, szDir, lasterr ) );
    %end        
end
