
% Function to go through all the directories and collect the cpu runtime
% statistics. Based on irecon_perf_cull. Usage is:
%    [cpuMetrics, misc] = irecon_cpu_cull( varargin ),
% where valid arguments are 'resfile', 'numtest', 'spriv'.

function [cpuMetrics, misc] = irecon_cpu_cull( varargin )

% Initialize
szCwd = pwd;

% We only need to load the result file. The true image variable, xtrue,
% should be saved in this result file.
szResultFilename = '';

sPriv = struct;

numSszDir = 0;

% The matrix of different cpu metrics for the different noise
% realizations
cpuMetrics = [];
misc = struct;

% Process input arguments

lenVarargin = length( varargin );
numArgsIn = floor( lenVarargin/2 );

if 2*numArgsIn ~= lenVarargin, error( 'even ip args expected' ), end

for argInInd = 1:numArgsIn
    
    szKey = lower( varargin{2*argInInd-1} );    
    switch szKey    
        case 'resfile', szResultFilename = varargin{2*argInInd};
        case 'numtest', numSszDir = varargin{2*argInInd};
        case 'spriv', sPriv = varargin{2*argInInd};
        otherwise
            % Ignore other inputs
    end
    
end

if ( isempty( szResultFilename )  | ( numSszDir < 1 ) )
    error( 'uninitialized variables' );
end

disp( sprintf( 'Processing %g subdirs', numSszDir ) );
for indSszDir = 1:numSszDir
    sszDir{indSszDir} = sprintf( 't%d', indSszDir );
end

% Make sure that the directories in sszDir exist. As well, check that
% szResultFilename exists in each directory.

for indSszDir = 1:numSszDir       
    szDir = char( sszDir( indSszDir ) );    
    try
        cd( strcat( szCwd, '/', szDir ) );
    catch
        % Directory doesn't exist
        cd( szCwd );
        error( sprintf( 'Cannot change to %s', szDir ) );
    end    
    if ~exist( szResultFilename, 'file' )
        cd( szCwd );
        error( sprintf( 'Result file %s does not exist in %s', szResultFilename, szDir ) );
    end
end

% Now go through each directory and compute the error metrics using
% sp_img_error.

for indSszDir = 1:numSszDir
    szDir = char( sszDir( indSszDir ) );
    cd( strcat( szCwd, '/', szDir ) );
    
    try     
        load( szResultFilename, 'sRestore' );
        
        totTime = sRestore.cputime(2) - sRestore.cputime(1);
        numIters = length( sRestore.errorNorm1 );
        perIterTime = totTime / numIters;
        
        cpuMetrics = [ cpuMetrics; totTime, numIters, perIterTime ];
    catch   
        cd( szCwd );
        error( sprintf( 'Encountered error with dir %s: %s', szDir, lasterr ) );
    end
end

cd( szCwd );
