
% Function to go through all the directories and collect the final
% hyperparameter estimates. Based on irecon_perf_cull. Usage is:
%    hyperEsts = irecon_hyp_cull( varargin ),
% where valid arguments are 'resfile', 'numtest'

function [hyperEsts, misc] = irecon_hyp_cull( varargin )

% Initialize
szCwd = pwd;

% We only need to load the result file. The true image variable, xtrue,
% should be saved in this result file.
szResultFilename = '';

numSszDir = 0;

bInitProblemProp = 0;

% The matrix of different final hyperparameter estimates for the different
% noise realizations
hyperEsts = [];
misc = struct;

bDebug = 0;

% Process input arguments

lenVarargin = length( varargin );
numArgsIn = floor( lenVarargin/2 );

if 2*numArgsIn ~= lenVarargin, error( 'even ip args expected' ), end

for argInInd = 1:numArgsIn
    
    szKey = varargin{2*argInInd-1};    
    ip = varargin{2*argInInd};
    
    switch szKey    
        case 'resfile', szResultFilename = ip;
        case 'numtest', numSszDir = ip;
        otherwise, warning( sprintf( 'unrecognized key: %s', szKey ) )
    end
    
end


if ( isempty( szResultFilename ) ) | ( numSszDir < 1 )
    error( 'uninitialized variables' );
end

disp( sprintf( 'Processing %g subdirs', numSszDir ) );
for indSszDir = 1:numSszDir
    sszDir{indSszDir} = sprintf( 't%d', indSszDir );
end

% Make sure that the directories in sszDir exist. As well, check that
% szResultFilename exists in each directory.

for indSszDir = 1:numSszDir       
    
    szDir = char( sszDir{ indSszDir } );    
    
    try
        cd( strcat( szCwd, '/', szDir ) );
    catch
        % Directory doesn't exist
        cd( szCwd );
        error( sprintf( 'Cannot change to %s', szDir ) );
    end    
   
    if bDebug, disp( sprintf( 'Check %s/%s', szCwd, szDir ) ), end
    
    if ~exist( szResultFilename, 'file' )
        cd( szCwd );
        error( sprintf( 'Result file %s does not exist in %s', szResultFilename, szDir ) );
    end
    
end

% Now go through each directory and collect the final hyperparameter
% estimates. This is less involved than irecon_perf_cull, which involves
% the execution of a function.

bEbdHack = 0;

% I don't know why I create these stupid hacks.

if strcmp( szResultFilename, 'eb_better_nov05.mat' ) == 1
    bEbdHack = 1;
    warning( 'Computing w from (a,t)' );
end

% Yet another stupid hack

if strcmp( szResultFilename, 'lars_v2.mat' ) == 1
    load( '~/work/f05/icip06/sbl_gaussian_psf.mat' );
    [XN, XColumnNorm] = lars_normx( 'x', SBL_PHI );
    clear( 'XN' ); % No need for this
end

for indSszDir = 1:numSszDir
    
    szDir = char( sszDir{ indSszDir } );
    cd( strcat( szCwd, '/', szDir ) );
    if bDebug, disp( sprintf( 'Switched to %s/%s', szCwd, szDir ) ), end

    lastHyperEst = [];
    
    try                     
        switch ( szResultFilename )
            case 'lars_v2.mat'
                load( szResultFilename, 'sRestore' );
                bestBigCHat = sRestore.bigCHat( sRestore.lastInd );
                % The soft threshold is lambda/2
                lastHyperEst = bestBigCHat * XColumnNorm(1);                

            otherwise
                warning off MATLAB:load:variableNotFound
                load( szResultFilename, 'sRestore', 'sRestoreOptions' );
                warning on MATLAB:load:variableNotFound
                
                if ( exist( 'sRestore', 'var' ) & isfield( sRestore, 'hyperParamHist' ) )
                    if ~isempty( sRestore.hyperParamHist )
                        lastHyperEst = sRestore.hyperParamHist( end, : );
                        if bEbdHack
                            w = convert_tsigma2w( lastHyperEst(1), lastHyperEst(2), sRestoreOptions.alpha, 'median' );
                            % Augment lastHyperEst
                            lastHyperEst = [lastHyperEst(:); w];
                        end
                    end
                end
        end
            
    catch   
        cd( szCwd );
        error( sprintf( 'Encountered error with dir %s: %s', szDir, lasterr ) );
    end
    
    if ~isempty( lastHyperEst )
        hyperEsts = [ hyperEsts; lastHyperEst(:)' ];
    end
    
end

cd( szCwd );