%
% Function to convert .mat files to v6/v4 format. Usage is:
%    irecon_mat_convert( varargin )
% where the valid arguments are: 'fromfile', 'tofile', 'testvec',
% 'conversion'
%

function irecon_mat_convert( varargin )

% Initialize
szCwd = pwd;

szFromFilename = '';
szToFilename = '';
numTest = 0;
szConversion = '';

% Process input arguments

lenVarargin = length( varargin );
numArgsIn = floor( lenVarargin/2 );

if 2*numArgsIn ~= lenVarargin, error( 'even ip args expected' ), end

for argInInd = 1:numArgsIn
    
    szKey = lower( varargin{2*argInInd-1} );    
    switch szKey 
        case 'fromfile', szFromFilename = varargin{2*argInInd};
        case 'tofile', szToFilename = varargin{2*argInInd};
        case 'testvec', testVec = varargin{2*argInInd};
        case 'conversion', szConversion = varargin{2*argInInd};
        otherwise
            error( sprintf( 'unrecognized option %s', szKey ) );
    end
    
end

% Check that initialization is complete
if isempty( szFromFilename ) | isempty( szToFilename ) | isempty( szConversion )
    error( 'Some variables have not been initialized' );
end

if length( testVec ) ~= 2, error( 'testVec should have 2 elements' ), end;
numSszDir = testVec(2) - testVec(1) + 1;
if numSszDir < 1, error( 'nothing to do' ), end;

disp( sprintf( 'Processing %g subdirs', numSszDir ) );
for indSszDir = 1:numSszDir
    sszDir{indSszDir} = sprintf( 't%d', testVec(1)+indSszDir-1 );
end

% Make sure that the directories in sszDir exist. As well, check that
% szFromFilename exists in each directory. 

for indSszDir = 1:numSszDir       
    szDir = char( sszDir( indSszDir ) );    
    try
        cd( strcat( szCwd, '/', szDir ) );
    catch
        % Directory doesn't exist
        cd( szCwd );
        error( sprintf( 'Cannot change to %s', szDir ) );
    end    
    if ~exist( szFromFilename, 'file' )
        cd( szCwd );
        error( sprintf( 'From-file %s does not exist in %s', szFromFilename, szDir ) );
    end
end

% Go through each directory, load the from-file, and save it as the
% to-file. Abort with an error if from-file doesn't exist. Do not overwrite
% the to-file if it already exists.

for indSszDir = 1:numSszDir
    szDir = char( sszDir( indSszDir ) );
    cd( strcat( szCwd, '/', szDir ) );
    
    if exist( szToFilename, 'file' )
        warning( sprintf( 'To-file %s already exists in %s!', ...
            szToFilename, szDir ) );
        continue; % Skip case
    end
    
    % Load the from-file
    sData = load( szFromFilename );
    
    switch szConversion
        case 'v7'
            %warning( 'v7: nothing to do' );
            % XXX: Assumes that we are running Matlab 7 when this function
            % is called
            save( szToFilename, '-struct', 'sData' );
        case 'v6'
            save( szToFilename, '-struct', 'sData', '-v6' );
        case 'v4'
            save( szToFilename, '-struct', 'sData', '-v4' );
        otherwise
            error( sprintf( 'unknown szConversion: %s', szConversion ) );
    end
end