
% Function to go through all the directories and collect the final error
% metrics. Based on sp_img_eval. Usage is:
%    [errorMetrics, misc] = irecon_perf_cull( varargin ),
% where valid arguments are 'resfile', 'numtest', 'spriv'.

function [errorMetrics, misc] = irecon_perf_cull( varargin )

% Initialize
szCwd = pwd;

% We only need to load the result file. The true image variable, xtrue,
% should be saved in this result file.
szResultFilename = '';

sPriv = struct;
szReconFieldname = '';

numSszDir = 0;

bInitProblemProp = 0;

% The matrix of different error metrics for the different noise
% realizations
errorMetrics = [];

% Process input arguments

lenVarargin = length( varargin );
numArgsIn = floor( lenVarargin/2 );

if 2*numArgsIn ~= lenVarargin, error( 'even ip args expected' ), end

for argInInd = 1:numArgsIn
    
    szKey = lower( varargin{2*argInInd-1} );    
    switch szKey    
        case 'resfile', szResultFilename = varargin{2*argInInd};
        case 'numtest', numSszDir = varargin{2*argInInd};
        case 'spriv', sPriv = varargin{2*argInInd};
        otherwise
            % Ignore other inputs
    end
    
end

if ( isempty( szResultFilename )  | ( numSszDir < 1 ) )
    error( 'uninitialized variables' );
end

if isempty( fieldnames( sPriv ) ), error( 'sPriv not properly initialized' ), end;
try
    szReconFieldname = sPriv.szReconFieldname;
catch
    error( 'cannot get szReconFieldname from sPriv' );
end
if isempty( szReconFieldname ), error( 'szReconFieldname not properly initialized' ), end;

disp( sprintf( 'Processing %g subdirs', numSszDir ) );
for indSszDir = 1:numSszDir
    sszDir{indSszDir} = sprintf( 't%d', indSszDir );
end

% Make sure that the directories in sszDir exist. As well, check that
% szResultFilename exists in each directory.

for indSszDir = 1:numSszDir       
    szDir = char( sszDir( indSszDir ) );    
    try
        cd( strcat( szCwd, '/', szDir ) );
    catch
        % Directory doesn't exist
        cd( szCwd );
        error( sprintf( 'Cannot change to %s', szDir ) );
    end    
    if ~exist( szResultFilename, 'file' )
        cd( szCwd );
        error( sprintf( 'Result file %s does not exist in %s', szResultFilename, szDir ) );
    end
end

% Now go through each directory and compute the error metrics using
% sp_img_error.
%//////////////////////////////////////////////////////////////////

for indSszDir = 1:numSszDir
    
    szDir = char( sszDir( indSszDir ) );
    cd( strcat( szCwd, '/', szDir ) );
    
    % Try to load sRestore.${szReconFieldname}, xtrue, propNoise. If we
    % cannot load sRestore, try loading $szReconFieldname directly.
    
    bLoadBaseVars = true;
    
    try
        load( szResultFilename, 'sRestore', 'xtrue', 'propNoise' );
        szCode = sprintf( 'xrecon = sRestore.%s;', szReconFieldname );
        eval( szCode );
    catch
        warning( sprintf( 'In %s, could not load sRestore, xtrue, propNoise', ...
            szDir ) );
        bLoadBaseVars = false;
    end
    
    if ( ~bLoadBaseVars )
        try
            load( szResultFilename, szReconFieldname, 'xtrue', 'propNoise' );
            szCode = sprintf( 'xrecon = %s;', szReconFieldname );
            eval( szCode );
        catch
            cd( szCwd );
            error( sprintf( 'In %s, could not load %s, xtrue, propNoise: aborting', ...
                szDir, szReconFieldname ) );
        end
    end
    
    sErr = sp_img_error( 'xrecon', xrecon, 'xtrue', xtrue );
                
    if ~bInitProblemProp

        try
            load( szResultFilename, 'SZ_IMAGE_TYPE', 'SZ_PSF_TYPE', 'SNR' );
        catch
            warning( sprintf( 'In %s, could not load SZ_IMAGE_TYPE, SZ_PSF_TYPE, SNR', ...
                szDir ) );
            SZ_IMAGE_TYPE = 'unknown'; % Use unknown settings
            SZ_PSF_TYPE = 'unknown';
            SNR = NaN;
        end

        problemProp = struct( 'SZ_IMAGE_TYPE', SZ_IMAGE_TYPE, ...
            'SZ_PSF_TYPE', SZ_PSF_TYPE, ...
            'sigma', propNoise.sigma, ...
            'snr', SNR, ...
            'nnz', sErr.trueNnz );
        bInitProblemProp = 1;

    end
    
    errorMetrics = [ errorMetrics; sErr.l0, sErr.numXorLoc, sErr.l1, ...
                                   sErr.lhard, sErr.lsoft, sErr.l2, nnz(xrecon) ];
                               
   if true % XXX DEBUG
       fprintf( 1, sprintf( '\tdir %s/%s, numXorLoc = %g\n', ...
           szCwd, szDir, sErr.numXorLoc ) );
   end
    
end

cd( szCwd );

if nargout == 2, misc = problemProp; end;