
% Function to plot various things about the reconstructed result. Variable
% arguments are: 'srestore', 'plotthings', 'figstart'. 
% Valid arguments of plotthings are: last, hyper, conv, error

function irecon_res( varargin )

% Initialize
sRestore = [];
sPlotThings = [];
nFigStart = 0;

% Process input arguments

lenVarargin = length( varargin );
numArgsIn = floor( lenVarargin/2 );

if 2*numArgsIn ~= lenVarargin, error( 'even ip args expected' ), end

for argInInd = 1:numArgsIn
    
    szKey = lower( varargin{2*argInInd-1} );    
    switch szKey      
        case 'srestore', sRestore = varargin{2*argInInd};
        case 'plotthings', sPlotThings = varargin{2*argInInd};
		case 'figstart', nFigStart = varargin{2*argInInd};
        otherwise
            error( sprintf( 'unrecognized option %s', szKey ) );
    end
    
end

if isempty( sRestore ), error( 'sRestore uninitialized' ), end;
if isempty( sPlotThings ), error( 'sPlotThings uninitialized' ), end;
if nFigStart < 1, error( 'nFigStart invalid' ), end;

nFig = nFigStart;
nPlotThings = length( sPlotThings );

% Go through and plot the fields

for i = 1:nPlotThings
	figure( nFig ), clf;
	szDesc = lower( char( sPlotThings(i) ) );
	switch szDesc
		case 'last', imagesc( sRestore.last ), colorbar;
            
		case 'hyper'
            numHyperParam = size( sRestore.hyperParamHist, 2 );
            if ( numHyperParam > 3 ), error( 'Cannot handle more than 3 hyper params' ), end;
            
            sszPlotColours = { 'r', 'k', 'b' };
            bLogy = [ 1, 0, 0 ];
            
            if numHyperParam == 1 % There is only 1 hyperparameter 
                v = sRestore.hyperParamHist(:);
                if bLogy(1) == 1, semilogy( v, char(sszPlotColours(j)) );
                else, plot( v, char(sszPlotColours(j)) );
                end
                ylabel( 'hyper' );
                text( length(v), v(end), sprintf( 'ss=%g', v(end) ), 'HorizontalAlignment', 'right' );
            else % There are 2 or 3 hyperparameters to plot
                nSubp = 100*numHyperParam + 10;                
                for hyperInd = 1:numHyperParam
                    v = sRestore.hyperParamHist( :, hyperInd );
                    subplot( nSubp+hyperInd );
                    if bLogy(hyperInd) == 1, semilogy( v, char(sszPlotColours(hyperInd)) );
                    else, plot( v, char(sszPlotColours(hyperInd)) );
                    end
                    ylabel( sprintf( 'hyper%g', hyperInd ) );
                    text( length(v), v(end), sprintf( 'ss=%g', v(end) ), 'HorizontalAlignment', 'right' );
                end
            end
           
		case 'conv'
			nSubp = 210;
			subplot( nSubp+1 ), semilogy( sRestore.thetaMaxDiff, 'm' );
			ylabel( 'l_{inf} conv' );
			subplot( nSubp+2 ), semilogy( sRestore.thetaSumDiff, 'k' );
			ylabel( 'l_1 conv' );
            
		case 'error'
			nSubp = 210;
			subplot( nSubp+1 ), plot( sRestore.errorNorm1, 'c' );
			ylabel( 'l_1 error' );
			subplot( nSubp+2 ), plot( sRestore.errorNorm2, 'k' );
			ylabel( 'l_2 error' );
            
		otherwise
			error( sprintf( 'unrecognized desc %s', szDesc ) );
	end
	nFig = nFig + 1;
end


