
% Script to run sp_img_recon and then a reconstruction technique, e.g.,
% sp_img_recon2

vecDir = [51:100];
numSszDir = length( vecDir );

disp( sprintf( 'Processing %g directories, from %g to %g', numSszDir, ...
    vecDir(1), vecDir(end) ) );

for indSszDir = 1:numSszDir
    sszDir{indSszDir} = sprintf( 't%d', vecDir(indSszDir) );
end
    
% Initialize
szCwd = pwd;
szSimFilename = 'base_v6.mat';
szResFilename = 'sbl2_v6.mat';

% Make sure that the directories in sszDir exist. If not, create them.
for indSszDir = 1:numSszDir    
    cd( szCwd );
    szDir = char( sszDir( indSszDir ) );    
    try
        cd( strcat( szCwd, '/', szDir ) );
    catch
        % Directory doesn't exist, so create it
        [bSuccess, szMsg, szMsgId] = mkdir( szCwd, szDir );
        if ~bSuccess, error( sprintf( 'Could not create dir %s', szDir ) ), end;
    end    
end

SNRdb = 7;

% Go through the directories and check to see if the simulation scenario,
% e.g., the variables xtrue, psf, y0, yi, etc., have been generated. They
% are saved in the file szSimFilename.
for indSszDir = 1:numSszDir
    szDir = char( sszDir( indSszDir ) );
    cd( strcat( szCwd, '/', szDir ) );
    if ~exist( szSimFilename, 'file' )
        % Generate simulation file. First, clear the crucial variable names
        % xtrue, psf, y0, yi.
        clear xtrue psf y0 yi;
        sp_img_reconB;
        save( szSimFilename );
        disp( 'Pausing 3 secs...' ), pause( 3 );
    end
end

% Run the specified reconstruction method
for indSszDir = 1:numSszDir
    szDir = char( sszDir( indSszDir ) );
    cd( strcat( szCwd, '/', szDir ) );
    if exist( szResFilename, 'file' )
        warning( 'Result file already exists, skipping...' );
        continue;
    end
    % Else, run the specified reconstruction method
    load( szSimFilename );
    
    % ========================================================
    % XXX: This should match with the naming of szResFilename!
    % ========================================================
    sp_img_recon2;
    
    % Save the results
    save( szResFilename );
end

cd( szCwd );