function [x_hat,a_hat,w_hat,sigma_hat]=map_revised(y,H,n_iter,step_size,sigmaReal,bUseSigmaReal);
flag=0; %%% suppress figures (flag=0)
%step_size=2e3;
%%%% y - matrix (n x m) - blurred noisy image
%%%% H - matrix (k x l) - forward operator
%%%% This algorithm runs faster when k*l is much smaller
%%%% than n*m (see help for conv2)
%%%% The output x has the same dimensions as y

% Find the largest eigenvalue of H
lmax2=sqrt(max(max(abs(fft2(H)).^2)));

% Another way of doing it. Note that the psf is zero-padded
hForwardOpDft = fft2( H, size( y, 1 ), size( y, 2 ) );
sRes = emsep_comp_alpha( hForwardOpDft, fft2(y), -1 );
lmax = sqrt( sRes.lambdaNnzMax );

disp( sprintf( 'lmax2=%g, lmax=%g', lmax2, lmax ) );
if ( abs( lmax - lmax2 ) > 1e3* eps )
    warning( 'lmax and lmax2 are different!' );
end
disp( 'Press key to continue...' ), pause;

H=H/lmax;

%%%% Constructing the adjoint of H
Had=fliplr(flipud(H));

if flag
    figure(1)
    imagesc(y);
    colormap('gray')
    colorbar
    title('Noisy Blurred Image')
end

% XXX: use Gblur
mask = logical(ones(size(y)));
G = Gblur( mask, 'psf', H, 'type', 'conv,fft' );
%G = Gblur( mask, 'psf', H, 'type', 'conv,same' );
yy = G' * y;

%%% initialization
%%% soft threshold
w_hat=0.80;

%x_hat=sort(y(:),'descend');
yySorted = sort( yy(:), 'descend' );
k=floor(w_hat*length(yySorted));
thresh=yySorted(k);
%x_hat=(y-sign(y)*thresh).*(abs(y)>thresh);
x_hat = (yy-sign(yy)*thresh).*(abs(yy)>thresh);

if 1, % SANITY
    figure( 1 ), clf, plot( yySorted, 'k.' ), hold on;
    plot( yy(:), 'ro' );
    title( 'yySorted' );
    disp( sprintf( 'thresh=%g', thresh ) );
	figure( 2 ), clf, imagesc( x_hat ), colorbar;
	title( 'initial image' );    
	disp( 'Pausing' ), pause;
end

% Estimate hyperparameters
%%% compute a,w,\sigma^2
a_hat=(1-mean(x_hat(:)==0))/mean(abs(x_hat(:)));

Gx_hat = G*x_hat;
tmp1 = y - Gx_hat;
%tmp1=y-conv2(x_hat,H,'same');

% DEBUG
% figure( pl+1 ), imagesc( x_hat ), colorbar, title( 'initial xhat' );
% figure( pl+2 ), imagesc( conv2(x_hat,H,'same') ), colorbar, title( 'initial H*xhat' );
% figure( pl+3 ), imagesc( tmp1 ), colorbar, title( 'initial tmp1' );

if bUseSigmaReal, sigma_hat = sigmaReal;
else sigma_hat=sqrt(mean((tmp1(:)).^2));
end

w_hat=1-mean(x_hat(:)==0);
%%% restarting with x_hat^{(0}}=y
%x_hat=y;
if 1
    figure(3)
    imagesc(x_hat);
    colormap('gray')
    colorbar
    title('Reconstructed Image')
    a_hat,w_hat,sigma_hat
    pause
end
l=0;
fac=1;
for ctr1=1:n_iter;
    
    % XXX
    tmp1 = y - G * x_hat;
    z = x_hat + fac^2 * (G' * tmp1);
    %tmp1=y-conv2(x_hat,H,'same');    
    %z=x_hat+fac^2*conv2(tmp1,Had,'same');
    
    %%% MAP thresholding
    if w_hat>0.5
        t=a_hat*sigma_hat^2;
    else
        t=a_hat*sigma_hat^2+sigma_hat*sqrt(2*log((1-w_hat)/(w_hat)));
    end
    x_hat=(z-sign(z)*a_hat*sigma_hat^2).*(abs(z)>t);

    %         l=l+1;
    %         a_hat1=(1-mean(x_hat(:)==0))/mean(abs(x_hat(:)));
    %         sigma_hat1=sqrt(mean((tmp1(:)).^2));
    %         w_hat1=1-mean(x_hat(:)==0);
    %          if w_hat1>0.5
    %                t1=a_hat1*sigma_hat1^2;
    %          else
    %                 t1=a_hat1*sigma_hat1^2+sigma_hat1*sqrt(2*log((1-w_hat1)/(w_hat1)));
    %          end
    %          aa(l)=a_hat1;
    %          ss(l)=sigma_hat1;
    %          ww(l)=w_hat1;
    %     pause
    if (ctr1-1)/step_size==floor((ctr1-1)/step_size)
        if flag
            figure(3)
            imagesc(x_hat/lmax);
            colormap('gray')
            colorbar
            title('Reconstructed Image')
        end
        
        % XXX
        tmp1 = y - G * x_hat;
        %tmp1=y-conv2(x_hat,H,'same');

		% Re-estimate the hyperparameters
        a_hat=(1-mean(x_hat(:)==0))/mean(abs(x_hat(:)));
        if bUseSigmaReal, sigma_hat = sigmaReal;
		else sigma_hat=fac*sqrt(mean((tmp1(:)).^2));
		end
        w_hat=1-mean(x_hat(:)==0);

        % DEBUG
        disp( sprintf( 'a_hat=%g, w_hat=%g, sigma_hat=%g', a_hat, w_hat, sigma_hat ) );
        
        z_hat=z;
        fz1=(1-w_hat)/sqrt(2*pi*sigma_hat^2)*exp(-z_hat.^2/2/sigma_hat^2);
        fz2=a_hat*w_hat/4*( ...
            (1-erf((a_hat*sigma_hat+z_hat/sigma_hat)/sqrt(2))).*...
            exp((a_hat^2*sigma_hat^2+2*a_hat*z_hat)/2)+...
            (1+erf((-a_hat*sigma_hat+z_hat/sigma_hat)/sqrt(2))).*...
            exp((a_hat^2*sigma_hat^2-2*a_hat*z_hat)/2));
        fz=fz1+fz2;
        conf=fz2./fz;
        if flag
            figure(5)
            imagesc(conf);
            colormap('gray')
            colorbar
            title('Confidence Image')

            %         figure(6)
            %         plot(1:step_size,ww,1:step_size,aa,1:step_size,ss)
            %         l=0;
            a_hat,w_hat,sigma_hat
            pause
        end
    end
end
x_hat=x_hat/lmax;
if flag
    figure(3)
    imagesc(x_hat)
    colormap('gray')
    colorbar
    title('Reconstructed Image')
end
