
% Script to test LARS function

B_TEST_LARS_PLOT = 0;

B_VERIFY = 0;

N_LARS = 3;

IP_IMG_SIZE = [ 64, 64 ];
OP_IMG_SIZE = [ 64, 64 ];

THRES = 1e3*eps;

% load( '~/work/f05/icip06/sbl_gaussian_psf.mat' );    
% load( '~/work/w06/tip/2d/1uud/sbl_mrfm_psf2d64.mat' );
load( '~/work/w06/tip/2d/benz/psf_a/sbl_mrfm_psf2d64.mat' );
% load( '~/work/w06/tip/2d/benz/psf_b/sbl_mrfm_psf2d64.mat' );

[XN, XColumnNorm] = lars_normx( 'x', SBL_PHI );

% Check that all elements of XColumnNorm are equal
for i = 2:length(XColumnNorm)
    if abs( XColumnNorm(i) - XColumnNorm(1) ) > THRES
        error( 'XColumnNorm not constant' );
    end
end

maxLarsIters = 50;

switch N_LARS
    case 3
        sRestore = lars3( 'noisy_output', yi(:), ...
            'beta_true', xtrue(:), ...
            'num_iters', maxLarsIters, ...
            'x', XN, ...
            'x_column_norm', XColumnNorm, ...
            'n_debug', 2, ...
            'psf', psf, ...
            'noiseSigma', propNoise.sigma, ...
			'ip_img_dim', IP_IMG_SIZE, ...
			'op_img_dim', OP_IMG_SIZE, ...
            'recon_sanity_interval', 2 );
        
    case 5
        sRestore = lars5( 'noisy_output', yi(:), ...
            'beta_true', xtrue(:), ...
            'num_iters', maxLarsIters, ...
            'x', XN, ...
            'x_column_norm', XColumnNorm, ...
            'n_debug', 0, ...
            'psf', psf, ...
            'noiseSigma', propNoise.sigma, ...
			'ip_img_dim', IP_IMG_SIZE, ...
			'op_img_dim', OP_IMG_SIZE, ...
            'n_phi2', 20, ...
            'n_phi1', 20, ...
            'n_strat', 3 );
        
    otherwise
        error( 'unrecognized N_LARS' );
end

% Check that the best image index is strictly less than maxLarsIters
if sRestore.lastInd1 == maxLarsIters
    warning( 'best index occurred at maxLarsIters!' );
elseif sRestore.lastInd1 == 1
    warning( 'best index occurred at 1?' );
elseif ( ( sRestore.lastInd1 < 1 ) | ( sRestore.lastInd1 > maxLarsIters ) )
    warning ( 'illegal value of sRestore.lastInd' );
end

% Reshape vectors into images of correct size
bestBigCHat = sRestore.bigCHat( sRestore.lastInd1 );

nFig = 1;

if B_TEST_LARS_PLOT

    switch N_LARS
        case 3
            figure( nFig ), clf, nFig = nFig + 1;
            imagesc( reshape( sRestore.last, IP_IMG_SIZE ) ), colorbar;
            ylabel( 'last = betaHatBest' );

            figure( nFig ), clf, nFig = nFig + 1;
            plot( sRestore.cost, 'm' );
            [costMin, costMinInd] = min( sRestore.cost );
            text( costMinInd, costMin, sprintf( 'min ind=%g', costMinInd ) );
            ylabel( 'cost' );

            figure( nFig ), clf, nFig = nFig + 1;
            plot( sRestore.bigCHat, 'b' );
            text( sRestore.lastInd, bestBigCHat, sprintf( 'CHat=%g', bestBigCHat ) );
            ylabel( 'CHat' );

        case 4
            figure( nFig ), clf, nFig = nFig + 1;
            imagesc( reshape( sRestore.bestBeta, IP_IMG_SIZE ) ), colorbar;
            ylabel( 'bestBeta' );

            figure( nFig ), clf, nFig = nFig + 1;
            imagesc( reshape( sRestore.bestBeta2, IP_IMG_SIZE ) ), colorbar;
            ylabel( 'betaHatBest2' );
            
            figure( nFig ), clf, nFig = nFig + 1;
            imagesc( reshape( sRestore.bestBeta3, IP_IMG_SIZE ) ), colorbar;
            ylabel( 'betaHatBest3' );

            figure( nFig ), clf, nFig = nFig + 1;
            plot( sRestore.cost, 'm' );
            [costMin, costMinInd] = min( sRestore.cost );
            if length( costMinInd ) == 1
                text( costMinInd, costMin, sprintf( 'min ind=%g', costMinInd ) );
            end
            ylabel( 'cost' );

            figure( nFig ), clf, nFig = nFig + 1;
            plot( sRestore.cost2, 'k' );
            [costMin, costMinInd] = min( sRestore.cost2 );
            if length( costMinInd ) == 1
                x = sRestore.phi2FracVec( costMinInd );
                text( costMinInd, costMin, sprintf( '(%g,%g)', x, costMin ) );
            end
            ylabel( 'cost2' );
            
            figure( nFig ), clf, nFig = nFig + 1;
            plot( sRestore.cost3, 'c' );
            [costMin, costMinInd] = min( sRestore.cost3 );
            if length( costMinInd ) == 1
                x = sRestore.phi1Vec( costMinInd );
                text( costMinInd, costMin, sprintf( '(%g,%g)', x, costMin ) );
            end
            ylabel( 'cost3' );
    end
    
end

% Compute using Daubechies' iterations

if ( ( N_LARS == 3 ) & ( B_VERIFY == 1 ) )
    
    [sDIter, sDIterOptions] = emgauss_restore( 'forward_op', psf, ...
        'noisy_output', yi, ...
        'noise_sigma', propNoise.sigma, ...
        'theta_true', xtrue, ...
        'm_selector', 'daub2', ...
        'num_em_iters', 2e4, ...
        'num_max_grad_search_failures', -1, ...
        'theta0_selector', 'Hty2', ...
        'theta0', [], ...
        'a_val', -1, ...
        'w_val', -1, ...
        't_val', -1, ...
        'sparse_ind', 'w', ...
        'stopping_cond', 'fixed', ...
        'stopping_cond_tol', propNoise.sigma, ...
        'stopping_cond_win', 3e3, ...
        'stopping_cond_last_fac', 2, ...
        'thetastar', 1, ...
        'soft_thres_t', -1, ...
        'bseparate_hparam_est', 0, ...
        'lambda', 2*bestBigCHat*XColumnNorm(1) ...
        );

    figure( nFig ), clf, nFig = nFig + 1;
    plot( sRestore.last(:), 'bx' ), hold on;
    plot( sDIter.last(:), 'ro' );
    ylabel( 'l1 solution' );
    legend( 'LARS-LASSO', 'Daubechies', 0 );
    
end

clear( 'XN', 'SBL_PHI' );

% XXX: Toy example

% X = [ (1:10)', rand(10,1), [1, zeros(1,9)]', [zeros(1,9), -1]' ];
% beta_true = [ -1, 0, 1, 0 ]';
% y = X*beta_true + 0.1*randn( 10, 1 );
% [XN, XColumnNorm] = lars_normx( 'x', X );
% sRestore = lars2( 'noisy_output', y(:), ...
%     'beta_true', beta_true(:), ...
%     'num_iters', 4, ...
%     'x', XN, ...
%     'x_column_norm', XColumnNorm );
% 
% if 1
%     figure( 1 ), clf, plot( XColumnNorm, 'bs' );
%     figure( 2 ), clf, plot( sRestore.last, 'ro' );
%     ylabel( 'last = betaHat' );
%     figure( 3 ), clf, plot( sRestore.muHat, 'bx' );
%     ylabel( 'muHat' );
%     figure( 4 ), clf, plot( sRestore.muCheck, 'kd' );
%     ylabel( 'muCheck' );
% end

