
% Script to test LARS function. Derived from test_lars3.

% Jan 22, 2007: Another copy of test_ll

% Constants
%%%

ll_defs; % Load definitions from file

SZ_LARS_FUNC = 'll_small';

LL_SMALL_COL_THRES = 1e3*eps;

maxLarsIters = 3e2;

SBL_PHI_FILE = { '~/work/f05/icip06/sbl_gaussian_psf.mat', ...
    '~/work/w06/tip/2d/1uud/sbl_mrfm_psf2d64.mat', ...
    '~/work/w06/tip/2d/benz/psf_a/sbl_mrfm_psf2d64.mat', ...
    '~/work/w06/tip/2d/benz/psf_b/sbl_mrfm_psf2d64.mat' };

% DEBUG_WORD = LL_DEBUG_LOOP1+LL_DEBUG_LOOP2+LL_DEBUG_PLOT+LL_DEBUG_DAUB;
DEBUG_WORD = LL_DEBUG_LOOP1+LL_DEBUG_LOOP2+LL_DEBUG_PLOT+LL_DEBUG_ITER_PAUSE+LL_DEBUG_DISP_GA;

% Run LARS-LASSO
%%%

if ~exist( 'yi', 'var' ) | ~exist( 'xtrue', 'var' ) | ~exist( 'psf', 'var' ) | ~exist( 'propNoise', 'var' )
    error( 'Missing variables yi, xtrue, psf, propNoise' );
else
    IP_IMG_SIZE = size( xtrue )
    OP_IMG_SIZE = size( yi )
end

switch SZ_LARS_FUNC
    
    case 'll_small'

        % Load psf
        if ~exist( 'SBL_PHI', 'var' )
            warning( 'Loading SBL_PHI' )
            load( SBL_PHI_FILE{ 3 } )
        else
            warning( 'Reusing SBL_PHI' )
        end

        % Normalize X matrix so that it has unit column norm
        %%%

        [XN, XColumnNorm] = lars_normx( 'x', SBL_PHI );

        % Check that all elements of XColumnNorm are equal
        for i = 2:length(XColumnNorm)
            if abs( XColumnNorm(i) - XColumnNorm(1) ) > LL_SMALL_COL_THRES
                error( 'XColumnNorm not constant' );
            end
        end

        disp( sprintf( 'll_small: column norm is %g', XColumnNorm( 1 ) ) )

        sRestore = ll_small( 'noisy_output', yi(:), ...
            'beta_true', xtrue(:), ...
            'num_iters', maxLarsIters, ...
            'x', XN, ...
            'x_column_norm', XColumnNorm, ...
            'n_debug', 2, ...
            'psf', psf, ...
            'noise_sigma', propNoise.sigma, ...
            'ip_img_dim', IP_IMG_SIZE, ...
            'op_img_dim', OP_IMG_SIZE, ...
            'recon_sanity_interval', 10, ...
            'debug_word',  DEBUG_WORD );

    case 'll_big'
        
        sRestore = ll_big( 'noisy_output', yi(:), ...
            'beta_true', xtrue(:), ...
            'num_iters', maxLarsIters, ...
            'n_debug', 2, ...
            'psf', psf, ...
            'noise_sigma', propNoise.sigma, ...
            'ip_img_dim', IP_IMG_SIZE, ...
            'op_img_dim', OP_IMG_SIZE, ...
            'recon_sanity_interval', 10, ...
            'debug_word',  DEBUG_WORD );

    otherwise
        error( 'unrecognized SZ_LARS_FUNC' );
        
end

% Check that the best image index is strictly less than maxLarsIters
if sRestore.lastInd1 == maxLarsIters
    warning( 'best index occurred at maxLarsIters!' );
elseif sRestore.lastInd1 == 1
    warning( 'best index occurred at 1?' );
elseif ( ( sRestore.lastInd1 < 1 ) | ( sRestore.lastInd1 > maxLarsIters ) )
    warning ( 'illegal value of sRestore.lastInd' );
end

bestBigCHat = sRestore.bigCHat( sRestore.lastInd1 );

