
% Script to test out probjs_wrapper

if ~exist( 'propImage', 'var' )
    % Construct propImage structure
    propImage = struct( 'n', 1e4, 'a', sqrt(2)/2, 'w', 0.1, 'sigma', 0.2 );
    propImage.t = convert_wsigma2t( propImage.a, propImage.w, propImage.sigma, ...
        'median' );
end

if ~exist( 'fclean', 'var' ) || ~exist( 'fnoisy', 'var' )
    % Generate fclean and fnoisy. See the pseudo-random number generator to
    % ensure that we get the same image.
    %rand( 'state', 1 );
    [fclean,fnoisy] = gen_laplace_data( propImage.n, propImage.a, propImage.w, propImage.sigma );
end

empSdev = std( fnoisy(:) );
tUniv = sqrt( 2*log(propImage.n) ) * empSdev;
paramLowerBound = [1e-3/empSdev, 1e-3*empSdev, 1e-3*empSdev];
paramUpperBound = [3/empSdev, tUniv, empSdev];
tol = min( 1e-4 * ones(1,3), 1e-3*[propImage.a, propImage.t, propImage.sigma] );
disp( 'tol' ), disp( tol );

%--- Old search functions ---
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% sOptim2 = john_optim2( fnoisy(:), propImage.sigma );
% sOptim1a = john_optim1a( fnoisy(:), propImage.sigma,  propImage.w );

%--- New versions ---
%%%%%%%%%%%%%%%%%%%%%

maxIters = 2e2;
debugFlag = 0;

tic;

% Search space = (a)
% sOptim1 = probjs_wrapper( 'y', fnoisy, ...
%     'ylength', propImage.n, ...
%     'lowerbound', paramLowerBound(1), ...
%     'upperbound', paramUpperBound(1), ...
%     'tol', tol, ...
%     'maxiters', maxIters, ...
%     'w', propImage.w, ...
%     'sigma', propImage.sigma, ...
% 	'debugflag', debugFlag ...
%     );
% disp( 'sOptim1:' ), disp( sOptim1 );

% Search space = (a,t)
% sOptim2 = probjs_wrapper( 'y', fnoisy, ...
%    'ylength', propImage.n, ...
%    'lowerbound', paramLowerBound(1:2), ...
%    'upperbound', paramUpperBound(1:2), ...
%    'tol', tol, ...
%    'maxiters', maxIters, ...
%    'sigma', propImage.sigma, ...
% 	'debugflag', debugFlag ...
%    );
% disp( 'sOptim2:' ), disp( sOptim2 );
% w2 = convert_tsigma2w( sOptim2.voptim(1), sOptim2.voptim(2), ...
% 	propImage.sigma, 'median' );
% disp( 'w2:' ), disp( w2 );

% Search space = (a,t,sigma)
%maxIters = 5e3;
sOptim3 = probjs_wrapper( 'y', fnoisy, ...
   'ylength', propImage.n, ...   
   'lowerbound', paramLowerBound, ...
   'upperbound', paramUpperBound, ...
   'tol', tol, ...
   'maxiters', maxIters, ...
	'debugflag', debugFlag ...
   );
disp( 'sOptim3:' ), disp( sOptim3 );
w3 = convert_tsigma2w( sOptim3.voptim(1), sOptim3.voptim(2), ...
    sOptim3.voptim(3), 'median' );
disp( 'w3:' ), disp( w3 );

tElapsed = toc