
% 24-oct-2006: Test psf2mat3Select and psf2mat_xa

% Assume that all the relevant variables are already loaded

rand( 'state', sum( 100*clock ) );

N = length( yi(:) );
M = length( xtrue(:) );

OP_IMG_SIZE = size( yi );
IP_IMG_SIZE = size( xtrue );

nConvMethod = 1; % Keep this--see comment below.

% Initialize Gblur object. 
%//////////////////////////////////////////////////////////////////////////

if ~exist( 'GX', 'var' ), disp( 'Creating Gblur object' )

    if nConvMethod == 1
        GX = Gblur( ones( OP_IMG_SIZE ), ...
            'psf', psf, ...
            'type', 'conv,fft', ...
            'nDim', length( IP_IMG_SIZE ) );
    elseif nConvMethod == 2
        GX = Gblur( ones( OP_IMG_SIZE ), ...
            'psf', psf, ...
            'type', 'conv,same', ...
            'nDim', length( IP_IMG_SIZE ) );
    else
        error( 'illegal nConvMethod' );
    end

end

% Randomly select several columns and choose a random set of coefficients.
% The testing method is as follows. X*( \sum_i alpha_i e_i ) should equal
% \sum_i alpha_i x_i, where x_i are the columns of X. Similarly, if we take
% v = \sum_i \alpha_i e_i, and do the convolution of psf * v, we should get
% the same answer.
%//////////////////////////////////////////////////////////////////////////

K = 20;

colRandom = 1 + floor( rand(1,K) * M );
% Sort colRandom for psf2mat3Select
colRandom = sort( colRandom(:) );

coeffRandom = rand( size( colRandom ) );

% Construct the sign vector. It should be the same size as the # of columns
% of X. Make it all 1s.

signVec = ones( M, 1 );

% New method: should work with 2d and 3d psfs.
XANew = psf2mat_xa( psf, colRandom, ones( K, 1 ) );

if ndims( psf ) == 3
    % Old method: only works with psf that's 3d
    [XAOld, sInfoOld] = psf2mat3Select( psf, signVec, colRandom );
    
    % SANITY 1: check to see if the columns of XAOld and XANew match
    for k = 1:K
        normDiffXa = norm( XANew( :, k ) - XAOld( :, k ), 1 );
        disp( sprintf( '|diff|_1 of column %g = %g', k, normDiffXa ) );
    end
end

% Assuming that SANITY 1 passed, we want to compare that with the DFT based
% method. MUST have nConvMethod set to 1: otherwise, the matrix
% representation will NOT be what's in psf2mat_xa.

xOutput = zeros( N, 1 );
for k = 1:K
    xOutput = xOutput + coeffRandom( k ) * XANew( :, k );
end

v = zeros( numel(psf), 1 );
for k = 1:K
    v( colRandom( k ) ) = coeffRandom( k );
end

xOutputGx = GX * reshape( v, IP_IMG_SIZE );
xOutputGx = xOutputGx(:);

% SANITY 2: check to see if it the DFT method agrees with the column
% selection method.
normDiffDft = norm( xOutputGx - xOutput, 1 )

