% ts_dig: Time series dig
% Keys: 'y'

function ts_dig( varargin )

n = length( varargin );
if ( floor(n/2)*2 ~= n ), error( 'even number of inputs expected' ), end;
numArgs = floor(n/2);

y = [];

for i = 1:numArgs
    szKey = varargin{2*i-1};
    switch( lower( szKey ) )
        case 'y', y = varargin{2*i};
        otherwise
            error( sprintf( 'unrecognized: %s', szKey ) );
    end
end

y = y(:);
if isempty( y ), error( 'y not specified' ), end;

yLen = length( y );
yMean = mean( y );
yVar = var( y );
numHistBins = max( [floor(yLen/60), 30] );

figure(1), clf, plot( y, 'mx' ), title( 'Time series' );
figure(2), clf, hist( y, numHistBins ), title( 'Histogram' );

% Plot the smoothed periodogram
yZeroMean = y - yMean; % Zero-mean version of y
rTaper = 0.2;
hTaper = window( @tukeywin, yLen, rTaper );
yZMT = yZeroMean .* hTaper; % Zero-mean and tapered version of y
yZMTDft = fft( yZMT );
ySmoothedPgram = yZMTDft .* conj( yZMTDft ) / sum( hTaper.^2 );

pgramRange = floor(yLen/2);
figure(3), clf, stem( ySmoothedPgram(1:pgramRange), 'r' );
title( 'Smoothed periodogram' );

disp( [sprintf( '%-10s %-10s\n', 'mean', 'var' ), ...
       sprintf( '%-10g %-10g\n', yMean, yVar )] );